function adamsmain
% Forward Erluer method (3.13) and Adams method (3.27) for FODE: 
% D^{0,t}^alpha y(t) = f(t,y)
% Table 3.1: sel = 4, no = 1
% Table 3.2: sel = 4, no = 0
% Table 3.3: sel = 4, no = 1 for first row, N = 10000;  sel = 4, no = 0 for second row, N = 10000.
% Table 3.4: sel = 5, no = 1
% Table 3.5: sel = 5, no = 0
% Table 3.6: sel = 5, no = 1 for first row, N = 10000;  sel = 4, no = 0 for second row, N = 10000.
sel = 5; 
no = 0;  % no = 0, fractional Adams method (3.27); no = 1, fractional forward Euler method (3.13)
[yv,y0,fv,alpha,T,N]=solution(sel);
n=length(N);   err=zeros(n,1);
fprintf('\n\n');
fprintf('alpha = %s\n',num2str(alpha));
for k=1:n
    if no>0.5
        u=fadams(y0,fv,alpha,T,N(k),1); % fractional forward Euler method (3.13)
    else
        u=fadams(y0,fv,alpha,T,N(k)); % Adams method (3.27)
    end
    tt=0:T/N(k):T+1e-10;
    switch sel
        case 2
            x = tt.^alpha;
            E1=mlf(alpha,1,-x,10);
            E5=mlf(alpha,5,-x,10);
            e=u-yv(tt,E5,E1);
        case 1
            e=u-yv(tt);
        case 3
            E1=mlf(alpha,1,-tt.^alpha,10);
            e=u-yv(E1);
        case 4
            e=u-yv(tt);
        case 5
%             yv=@(t)t.^4.*mlf(alpha,5,-t.^alpha,12);
            e=u-yv(tt);
        otherwise
    end
    err(k,1)=max(abs(e(end)));
    
end

odr=myorder(1./N',ones(n,1),err);
fprintf('maxerror      order\n',err);
for k=1:length(err)
fprintf('%10.4e %8.4f\n',err(k),odr(k));
end


function [u]=fadams(y0,fv,alpha,T,N,varargin)
% D^{0,t}^alpha y(t) = f(t,y)
% syms s t y alpha;
% y=t^8-3*t^(4+alpha/2)+9*t^alpha/4;
% n=ceil(alpha);
% sel=1;
% [yv,y0,fv,alpha,T,N]=solution(sel);
no=numel(varargin);
n=ceil(alpha);
steps=N(1); 
tau=T/steps;
tt=0:tau:T+1e-8;
ny0=0;
for j=0:n-1
    ny0=ny0+tt(2:end).^j*y0(j+1)/factorial(j);
end
u=zeros(1,steps+1); u(1)=y0(1);
for k=0:steps-1
   j=0:k;
   b=(k+1-j).^alpha-(k-j).^alpha;
   b=tau^alpha/(gamma(alpha)*alpha)*b;
%    bb=(tt(2:end).^alpha-tt(1:end-1).^alpha)/(gamma(alpha)*alpha);
%    bb=fliplr(bb);
    f=fv(tt(1:k+1),u(1:k+1));
    yP = ny0(k+1) + f*b(1:k+1)';
    u(k+2)=yP;
    if no<0.5
       a=ones(1,k+2);    j=1:k;
       a(1)=k^(alpha+1)-(k-alpha)*(k+1)^alpha;
       if k>0
            a(2:end-1)=(k+2-j).^(alpha+1) + (k-j).^(alpha+1) - 2*(k+1-j).^(alpha+1); 
       end
       a=tau^alpha/(gamma(alpha)*alpha*(alpha+1))*a;
%        f=fv(tt(1:k+1),u(1:k+1));
       u(k+2) = ny0(k+1) + f*a(1:k+1)' + a(k+2)*fv(tt(k+2),yP);
    end
end
% bb-b
% e=u-yv(tt);
% e=max(abs(e));
% fprintf('maxerr = %10.6e\n',e);


function [y,y0,f,alpha,T,N]=solution(sel)
% D^{0,t}^alpha y(t) = f(t,y)
switch sel
    case 1
        syms t y a b;
        alpha=0.25;  T=1;  N=[10,20,40,80,160,320,640];
        f=40320*t^(8-alpha)/gamma(9-alpha)-3*gamma(5+alpha/2)/gamma(5-alpha/2)*t^(4-alpha/2)...
        +9/4*gamma(alpha+1)+(3/2*t^(alpha/2)-t^4)^3-y^(3/2);
        f=sym2funchdl('ty',f);
        y=t^8-3*t^(4+alpha/2)+9/4*t^(alpha);
        n=ceil(alpha);
        y0=zeros(1,n);
        for j=0:n-1
            yy=diff(y,j);
            yyv=sym2funchdl('t',yy);
            y0(j+1)=yyv(0);
        end
        y=sym2funchdl('t',y);
    case 2
        syms t y a b;
        alpha=0.1;  T=1;  N=[10,20,40,80,160,320,640];
        f= -y + t^(4-alpha)/gamma(5-alpha);
        f=sym2funchdl('ty',f);
        y0=1; 
        y = t^4*a + b;
        y=sym2funchdl('tab',y);
    case 3
        syms t y a b;
        alpha=0.4;  T=1;  N=[10,20,40,80,160,320,640];
        f = - y;
        f=sym2funchdl('ty',f);
        y0=[1,0]; 
        y=t;
        y=sym2funchdl('t',y);
    case 4
        syms t y a b;
        alpha = 0.7; % fractional order
        alf = alpha;
        T = 1;
        N = [10,20,40,80,160,320,640]; % number of grid points
        nu1 = 5; nu2 = 4; nu3 = 3;
        f = - y^2 + (t^nu1 - 3*t^nu2 + 2*t^nu3).^2....
            +   gamma(nu1+1)/gamma(nu1+1-alf)*t^(nu1-alf)....
            - 3*gamma(nu2+1)/gamma(nu2+1-alf)*t^(nu2-alf)....
            + 2*gamma(nu3+1)/gamma(nu3+1-alf)*t^(nu3-alf);
        f = sym2funchdl('ty',f);
        y = t^nu1 - 3*t^nu2 + 2*t^nu3;
        dy = diff(y,t);
        y = sym2funchdl('t',y);
        y0 = [y(0),subs(dy,t,0)];
    case 5
        alpha = 0.2;  % fractional order
        T = 1;
        N = [10,20,40,80,160,320,640]; 
%         N = 10000;
        bta = 5;
        f = @(t,y) - y + t.^(bta-1-alf)/gamma(bta-alf);
        y = @(t) t.^(bta-1).*mlf(alf,bta,-t.^alf,12) + 0*mlf(alf,1,-t.^alf,12);
%         y=@(t,alf)t.^4.*mlf(alf,5,-t.^alf,12);
        y0 = y(0);
end

function str=sym2str(s)
% convert string s to vector string
% input argument s is a sym string or numerical number
% or a char string containing '+ - * / ^ ( ) a,A,...,z,Z,0,...,9' 
% or space chars
s0=s;
if ~ischar(s)
    s0=char(sym(s));
end
n=length(s0);
for k=n:-1:1
    if  s0(k)=='.' || s0(k)=='+' || s0(k)=='-' || s0(k)=='*' ||s0(k)=='/'...
      ||s0(k)=='^' || s0(k)==' ' || s0(k)=='_' || s0(k)==')' || s0(k)=='('...
      ||(s0(k)>='a' && s0(k)<='z') || (s0(k)>='A' && s0(k)<='Z').....
      || (s0(k)>='0' && s0(k)<='9') || s0(k)==',' || s0(k)==']' || s0(k)=='['
%  delete  spaces ' 's
        if  strcmp(s0(k),' ') 
            s0(k)=[];
        end
    else
        fprintf('Not a valid string');
        return;
    end
end
n=length(s0);
str=[];
for k=1:n
    switch s0(k)
        case '*'
            str=[str,'.*'];
        case '/'
            str=[str,'./'];
        case '^'
            str=[str,'.^'];
        otherwise
            str=[str,s0(k)];
    end              
end
%
function f=sym2funchdl(v,s0)
% s is symbolic function about v=(x_1,...,x_n)
% return function handle about s = f(x_1,...,x_n)
n=length(v);
if n<1
    fprintf('error: No Variables input!');
    fprintf('\n'); f=[];  return;
end
s=s0;
if isfloat(s0)
    s={num2str(s0)};    
end
if ~isfloat(s0) && ~iscell(s0)
    s={char(s0)};
end
len=length(s);

str0='[';
for kk=1:len-1
    str0=[str0,sym2str(s{kk}),','];
end
str0=[str0,sym2str(s{len}),']'];
if abs(len-1)<1/2
    str0(end)=[];str0(1)=[];
end
%
stru='@(';
if iscell(v)
    for k=1:n-1
        stru=[stru,v{k},','];
    end
    stru=[stru,v{n},')',str0,';'];
else
    for k=1:n-1
        stru=[stru,v(k),','];
    end
    stru=[stru,v(n),')',str0,';'];
end
f=eval(stru);
%============= END sym2funchdl =============
%
function odr=myorder(ntau,NN,err)
n=length(ntau); 
tau(1:n,1)=ntau; N(1:n,1)=NN(1:n); 
er(1:n,1)=err(1:n);
if n==1
    odr=0; return;
end
if n>1
    if abs(ntau(1)-ntau(2))>1e-10
        odr=(log(er(1:end-1)./er(2:end)))./log(tau(1:n-1,:)./tau(2:n,:));
    else 
        odr=(log(er(1:end-1)./er(2:end)))./log(N(1:n-1,:)./N(2:n,:));
    end
end
% d1=(log(er(1:end-1)./er(2:end)))./log(tau(1:n-1,:)./tau(2:n,:));
% d2=(log(er(1:end-1)./er(2:end)))./log(N(1:n-1,:)./N(2:n,:));
% odr=d1;
% for k=1:n-1
%     if abs(d1(k))<100 && ~isnan(d1(k))
%         odr(k)=d1(k);
%     end
%     if abs(d2(k))<100
%         odr(k)=d2(k);
%     end
% end
odr=[0;odr];

function [e]=mlf(alf,bet,c,fi)
%
% MLF -- Mittag-Leffler function. 
%           MLF (alpha,beta,Z,P) is the Mittag-Leffler function E_{alpha,beta}(Z) 
%           evaluated with accuracy 10^(-P) for each element of Z. 
%           alpha and beta are scalars, P is integer, Z can be a vector or
%           a two-dimensional array. The output is of the same size as Z.
% (C) 2001-2009 Igor Podlubny, Martin Kacenak


[cRows, cCols] = size(c);
c=m2v(c);

if nargin<4 , fi=6;		end
if nargin<3 | alf<=0 | fi<=0
else 
   [r,s]=size(c); [r1,s1]=size(alf); [r2,s2]=size(bet);
   mx=max([r,s]); mx1=max([r1,s1]); mx2=max([r2,s2]);
   if (r>1 & s>1) | (r1>1 & s1>1) | (r2>1 & s2>1) | (mx1>1 & mx2>1)
      sprintf('wrong number of input parameters')
   else
      if mx1>mx2 , mxx=mx1; e=zeros(mx,mx1);
      else, mxx=mx2; e=zeros(mx,mx2);end;
      for i1= 1:mx
         for i2=1:mxx
            
            if r>s , z=c(i1,1); else,z=c(1,i1); end
            if mx1>mx2 , if r1>s1 , alfa=alf(i2,1); else, alfa=alf(1,i2);end, beta=bet;
            else if r2>s2 ,beta=bet(i2,1); else, beta=bet(1,i2); end, alfa=alf; end
            if beta<0 , rc=(-2*log(10^(-fi)*pi/(6*(abs(beta)+2)*(2*abs(beta))^(abs(beta)))))^alfa;
               else , rc=(-2*log(10^(-fi)*pi/6))^alfa; end
            r0=max([1,2*abs(z),rc]);
            if (alfa==1 & beta==1)
               e(i1,i2)=exp(z);
            else
               if (alfa<1 & abs(z)<=1) | (1<=alfa<2 & abs(z)<=floor(20/(2.1-alfa)^(5.5-2*alfa))) | (alfa>=2 & abs(z)<=50)
                  oldsum=0;
                  k=0;
                  while (alfa*k+beta)<=0 
                     k=k+1;
                  end
                  newsum=z^k/gamma(alfa*k+beta);
                  while newsum~=oldsum
                     oldsum=newsum;
                     k=k+1;
                     term=z^k/gamma(alfa*k+beta);
                     newsum=newsum+term;
                     k=k+1;
                     term=z^k/gamma(alfa*k+beta);
                     newsum=newsum+term;
                  end
                  e(i1,i2)=newsum;
               else
                  if (alfa<=1 & abs(z)<=fix(5*alfa+10))
                     if ((abs(angle(z))>pi*alfa) & (abs(abs(angle(z))-(pi*alfa))>10^(-fi)))
                        if beta<=1
                           e(i1,i2)=rombint('K',0,r0,fi,alfa,beta,z);
                        else
                          eps=1;
                          e(i1,i2)=rombint('K',eps,r0,fi,alfa,beta,z)+ ...
                             rombint('P',-pi*alfa,pi*alfa,fi,alfa,beta,z,eps);
                       end
                    elseif (abs(angle(z))<pi*alfa & abs(abs(angle(z))-(pi*alfa))>10^(-fi))
                       if beta<=1
                           e(i1,i2)=rombint('K',0,r0,fi,alfa,beta,z)+ ...
                              (z^((1-beta)/alfa))*(exp(z^(1/alfa))/alfa);
                       else
                           eps=abs(z)/2;
                           e(i1,i2)=rombint('K',eps,r0,fi,alfa,beta,z)+ ...
                              rombint('P',-pi*alfa,pi*alfa,fi,alfa,beta,z,eps)+ ...
                              (z^((1-beta)/alfa))*(exp(z^(1/alfa))/alfa);
                        end
                     else
                        eps=abs(z)+0.5;
                        e(i1,i2)=rombint('K',eps,r0,fi,alfa,beta,z)+ ...
                           rombint('P',-pi*alfa,pi*alfa,fi,alfa,beta,z,eps);
                     end
                  else
                     if alfa<=1
                        if (abs(angle(z))<(pi*alfa/2+min(pi,pi*alfa))/2)
                           % alfa
                           newsum=(z^((1-beta)/alfa))*exp(z^(1/alfa))/alfa;
                           for k=1:floor(fi/log10(abs(z)))
                              newsum=newsum-((z^(-k))/gamma(beta-alfa*k));
                              % k
                           end
                           e(i1,i2)=newsum;
                        else
                           newsum=0;
                           for k=1:floor(fi/log10(abs(z)))
                              newsum=newsum-((z^-k)/gamma(beta-alfa*k));
                           end
                           e(i1,i2)=newsum;
                        end
                     else
                        if alfa>=2
                           m=floor(alfa/2);
                           sum=0;
                           for h=0:m
                              zn=(z^(1/(m+1)))*exp((2*pi*i*h)/(m+1));
                              sum=sum+mlf(alfa/(m+1),beta,zn,fi);
                           end
                           e(i1,i2)=(1/(m+1))*sum;
                        else
                           e(i1,i2)=(mlf(alfa/2,beta,z^(1/2),fi)+mlf(alfa/2,beta,-z^(1/2),fi))/2;
                        end
                     end
                  end
               end
            end
         end
      end
   end
end

if isreal(c) 
    e = real(e);
end

e = v2m(e,cRows,cCols);


function [res]=rombint(funfcn,a,b,order,varargin);
if nargin<4 ,order=6; end
if nargin<3
   Warning ('Error in input format')
else 
   rom=zeros(2,order);
   h=b-a;
   rom(1,1)=h*(feval(funfcn,a,varargin{:})+feval(funfcn,b,varargin{:}))/2;
   
   ipower=1;
   for i= 2:order
      sum=0;
      for j=1:ipower
      	sum=sum+feval(funfcn,(a+h*(j-0.5)),varargin{:});
      end
      rom(2,1)=(rom(1,1)+h*sum)/2;
      for k=1:i-1
         rom(2,k+1)=((4^k)*rom(2,k)-rom(1,k))/((4^k)-1);
      end
      
      for j=0:i-1
         rom(1,j+1)=rom(2,j+1);
      end
      ipower=ipower*2;
      h=h/2;
   end
   res=rom(1,order);
end


function res=K(r,alfa,beta,z)
res=r.^((1-beta)/alfa).*exp(-r.^(1/alfa)).*(r*sin(pi*(1-beta))-...
z*sin(pi*(1-beta+alfa)))/(pi*alfa*(r.^2-2*r*z*cos(pi*alfa)+z.^2));


function res=P(r,alfa,beta,z,eps)
w=(eps^(1/alfa))*sin(r/alfa)+r*(1+(1-beta)/alfa);
res=((eps^(1+(1-beta)/alfa))/(2*pi*alfa))*((exp((eps^(1/alfa))*cos(r/alfa)).*...
   (cos(w)+i*sin(w))))/(eps*exp(i*r)-z);

function A = v2m(V, M, N)
if numel(V)==M*N, 
    A = reshape(V, [N, M]);
    A = A' ;
else
    warning('Wrong dimensions of the output in V2M.')
end

function V = m2v(A)
M = A'; V = M(:);



