function flmm_fdm_st
% space-time differential equation in Caputo sense
% D^{alf}_{0,t}u = nu*D_{|x|}^{beta}u(x,t) + f;
tic
fdm = 5;
sel = 0; 
q = 4; 
nalf = [0.5 0.5 0.5];  % fractional order in time
nbeta = [1.2 1.5 1.8]; % fractional order in space
% Table 4.19: fdm = 7, q = 1,2,3,4, nalf = [0.8 0.8 0.8]
% Table 4.20: fdm = 5, q = 1,2,3,4, nalf = [0.5 0.5 0.5]
% Table 4.21: fdm = 6, q = 1,2,3,4, nalf = [0.2 0.2 0.2]
for kk=1:length(nalf)
    alf = [nalf(kk),nbeta(kk)];
    [uv,gv,N,tau,nodes,nu,T,alpha,uav,ubv,ut0,a1,a2,a3] = solution(sel,alf(1),alf(2));
    nn = length(tau); ee = zeros(nn,1); ee2 = ee;  ee3 = ee;
    for k=1:nn
        switch fdm
%             case 1
%                 [uu,e,e2,e3]=pro_fdm1(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,q);
%             case 2
%                 [uu,e,e2,e3]=pro_fdm2(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,q);
%             case 3
%                 [uu,e,e2,e3]=pro_fdm3(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,q);
%             case 4
%                 [uu,e,e2,e3]=pro_fdm4(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,q);
            case 5
                [uu,e,e2,e3]=pro_fdm5(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,a1,a2,a3,q);
            case 6
                [uu,e,e2,e3]=pro_fdm6(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,a1,a2,a3,q);
            case 7
                [uu,e,e2,e3]=L1_fdm(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,q);
        end
        ee(k) = e; ee2(k) = e2; ee3(k) = e3;
    end

% % % % % % % % % % % % % % % % % % % % % % % % 
odr = myorder(tau,1./N,ee);  odr2 = myorder(tau,1./N,ee2);  
odr3 = myorder(tau,1./N,ee3);
% filename = ['sisc_flmm_fdm_',num2str(fdm),'.txt'];
filename = 'sisc_flmm_fdm_comparison.txt';
fid = fopen(filename,'a');
fprintf('alf = %s,  T = %s\n',num2str(alf),num2str(T));
A=[N,1./tau,ee,odr,ee2,odr2,ee3,odr3];
s={'N','1/tau','max-2error','order','AV-L^2 error','order','L^2-error','order'};
output(s,A);
fprintf('cputime = %-10.4f\n\n',toc);
% % % % % % % % % % % % % % % % % % % % % 
fprintf(fid,'alf = %s, T = %s\n',num2str(alf),num2str(T));
output1(s,A,fid); 
fprintf(fid,'cputime = %-10.4f\n\n',toc);
end
% END



function [uu,e,e2,e3]=pro_fdm1(uv,gv,uav,ubv,N,tau,nodes,nu,T,alpha,ut0,q)
% % uniform grids
xa = nodes(1);   xb=nodes(2);   alf = alpha(1);  bta = alpha(2); 
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
p=1;
% % % % % % 
ne = round(T/tau)+1;    r = nu*(tau/2)^alf;             
w = weights(alf,ne,1);
w2 = w;  
w2(2:2:end) = -w2(2:2:end); 
w3 = (tau/2)^alf*w2;    w2 = r*w2;
b = w; 
for k=1:ne
    b(k+1)=b(k)+w(k+1);
end
w = -w;
% % % % % % % % % % % %
% [mm,ss] = basis(p,N,xx);  mm0=mm;
% % mm = eye(N+1)*h; ss = zeros(N+1);
% % for i=2:N
% %     ss(i,i-1:i+1) = [-1,2,-1];
% % end
% % ss = ss/h;
mm = eye(N+1)*h; mm0 = mm;  mm = mm(2:end-1,:); 
% sel = 4;          
[ss1,ss2] = frac_basis(bta,N,q);
ss = -(ss1 + ss2)/h^(bta-1);
% % % % % % % % % % % % % % % % % % 
% mm0 = mm;
arr = mm + nu*(tau/2)^alf*ss;
% arr = arr(2:end-1,:);  
bnd = [arr(:,1),arr(:,end)]; 
arr = (arr(:,2:end-1)); 
% mm = sparse(mm(2:end-1,:)); ss=sparse(ss(2:end-1,:));
%------------- initial ------------
xx=xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);   uu(:,1) = u0; 
ff = zeros(p*N+1,ne);
tt = 0:tau:T+1e-10;
for k=1:ne
    ff(:,k)=gv(xx,tt(k));
end
%-----------------------------
for n = 1:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs = w3(1)*ff(:,n+1) + b(n+1)*u0; 
    rhs1 = 0; 
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,n+1-k) + w3(k+1)*ff(:,n+1-k);
        rhs1 = rhs1 + w2(k+1)*uu(:,n+1-k);
    end
    rhs = mm*rhs - ss*rhs1 - bnd*[ua;ub];
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
e3 = funuv(uv,xx,tt)-uu;  e = max(max(abs(e3(:,ne))));  
e4 = zeros(1,ne);  
for k=1:ne
    e4(k) = (e3(:,k)'*mm0*e3(:,k));
end
e = max(sqrt((e4)));   
e3 = (sqrt(e4(ne))); 
e2 = sqrt(tau*sum(e4));
%END

function [uu,e,e2,e3]=pro_fdm2(uv,gv,uav,ubv,N,tau,nodes,nu,T,alpha,ut0,q)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    alf = alpha(1);  bta = alpha(2);
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
p=1;
% % % % % % 
ne = round(T/tau)+1;  r = nu*tau^alf*alf/2;             
w = weights(alf,ne,1);
w3 = tau^alf*[1-alf/2,alf/2];
w2 = r;
b = w; 
for k=1:ne
    b(k+1) = b(k) + w(k+1);
end
w = -w;
% % % % % % % % % % % % % % % % % %
% [mm,ss] = basis(p,N,xx); mm0=mm;
% mm = eye(N+1)*h; ss = zeros(N+1);
% for i=2:N
%     ss(i,i-1:i+1) = [-1,2,-1];
% end
% ss = ss/h;
% mm0=mm;

mm = eye(N+1)*h; mm0 = mm;  mm = mm(2:end-1,:); 
% sel = 4;          
[ss1,ss2] = frac_basis(bta,N,q);
ss = -(ss1 + ss2)/h^(bta-1);
% % % % % % % % % % 
arr = mm + nu*tau^alf*(1-alf/2)*ss;
% arr = arr(2:end-1,:);
bnd = [arr(:,1),arr(:,end)]; 
arr = arr(:,2:end-1);       
% arr = sparse(arr);
% mm = sparse(mm(2:end-1,:)); ss=sparse(ss(2:end-1,:));
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);   uu(:,1) = u0;  
ff = zeros(p*N+1,ne);
tt = 0:tau:T+1e-10;
for k=1:ne
    ff(:,k)=gv(xx,tt(k));
end
%-----------------------------
for n = 1:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs = w3(1)*ff(:,n+1) + w3(2)*ff(:,n) + b(n+1)*u0; 
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,n+1-k);
    end
    rhs = mm*rhs - w2*(ss*uu(:,n)) - bnd*[ua;ub];
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
e3 = funuv(uv,xx,tt)-uu;   e = max(max(abs(e3(:,ne))));
e4 = zeros(1,ne);  
for k=1:ne
    e4(k) = e3(:,k)'*mm0*e3(:,k);
end
e = max(sqrt((e4)));   
e3 = sqrt(e4(ne)); 
e2 = sqrt(tau*sum(e4));
%END

function [uu,e,e2,e3]=pro_fdm3(uv,gv,uav,ubv,N,tau,nodes,nu,T,alpha,ut0,q)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    alf = alpha(1);  bta = alpha(2);
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
p=1;
% % % % % % 
ne = round(T/tau)+1;    r = nu*(tau/2)^alf;             
w = weights(alf,ne,1);
w2 = w;   w2(2:2:end) = -w2(2:2:end); 
w3 = (tau/2)^alf*w2;    w2 = r*w2;
b = w; 
for k=1:ne
    b(k+1) = b(k)+w(k+1);
end
w = -w;
% 
ww3 = (1/2)^alf*weights(alf,ne,1); ww3(2:2:end)=-ww3(2:2:end);
k=0:ne-1;
B0 = k.^(alf)/gamma(alf+1);  B = B0;
for k=1:ne
    B(k) = (-B0(1:k)*w(k:-1:1)-sum(ww3(1:k)));
end
B = nu*tau^alf*B;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
om0 = weights(alf,ne,1);
om1 = om0/2^(alf);      om1(2:2:end) = - om1(2:2:end);
om2 = weights(-alf,ne,1).';
om = zeros(ne,1);
for k=1:ne
    om(k) = om2(1:k)*om1(k:-1:1);
end
wn1 = (1:ne).^(1+alf)/gamma(2+alf);
k = 1:ne;
for n=1:ne
    wn1(n) =  wn1(n) - k(1:n)*om(n:-1:1);
end
kk = k.^(alf+1)/gamma(2+alf);
C = B; CC = C;
for n=1:ne
    C(n) = wn1(1:n)*om0(n:-1:1);
    CC(n) = kk(1:n)*om0(n:-1:1) - k(1:n)*om1(n:-1:1);
end
C = nu*tau^alf*CC*0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% [mm,ss] = basis(p,N,xx);  mm0 = mm;
% % mm = eye(N+1)*h; ss = zeros(N+1);
% % for i=2:N
% %     ss(i,i-1:i+1) = [-1,2,-1];
% % end
% % ss = ss/h;
% % mm0=mm;
mm = eye(N+1)*h; mm0 = mm;  mm = mm(2:end-1,:); 
% sel = 4;          
[ss1,ss2] = frac_basis(bta,N,q);
ss = -(ss1 + ss2)/h^(bta-1);
% % % % % % % % % % 
arr = mm + nu*(tau/2)^alf*ss; 
% arr = arr(2:end-1,:);
arr0 = mm + nu*(tau/2)^alf*ss + C(1)*ss;  
% arr0 = arr0(2:end-1,:);
bnd = [arr(:,1),arr(:,end)];    bnd0 = [arr0(:,1),arr0(:,end)]; 
arr = (arr(:,2:end-1));   arr0 = (arr0(:,2:end-1)); 
% mm = sparse(mm(2:end-1,:)); ss = sparse(ss(2:end-1,:));
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);   uu(:,1) = u0; 
tt = 0:tau:T+1e-10;
ff = zeros(p*N+1,ne);
for k=1:ne
    ff(:,k) = gv(xx,tt(k));
end
a0 = 1-alf;  % 
% gg2 = fint(ff,tt,alf,a0);   % fractonal integral of the right hand side
gg2 = fint2(ff,tt,alf,a0,1);   % fractonal integral of the right hand side
gg = mm*gg2;    

% nv = 2 + alf;
% gint = @(x,t)(t^(nv) + t)*sin(x).....
%     + nu*(gamma(nv+1)/gamma(nv+1+alf)*t^(nv+alf) ....
%     + 1/gamma(2+alf)*t^(1+alf) + 2/gamma(1+alf)*t^(alf))*sin(x);
% for k=1:ne
%     gg2(:,k) = gint(xx,tt(k));
% end
% gg = mm*gg2;
% % % % % % % % % % % First step % % % % % % % % % % % % 
n = 1;      
ua = uav(tt(n+1));   ub = ubv(tt(n+1));
rhs =  b(n+1)*u0;
rhs1 = 0;
rhs2 = w(1)*gg(:,n+1);
for k=1:n
    rhs = rhs + w(k+1)*uu(:,n+1-k);
    rhs1 = rhs1 + w2(k+1)*uu(:,n+1-k);
    rhs2 = rhs2 + w(k+1)*gg(:,n+1-k);
end
rhs = mm*rhs - ss*rhs1 - bnd0*[ua;ub] - rhs2 - B(n+1)*(ss*u0) + C(n)*(ss*u0);
u2 = arr0\rhs;
uu(:,n+1) = [ua;u2;ub];  
u1 = [ua;u2;ub];     u10 = u1-u0;
% % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
for n = 2:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs =  b(n+1)*u0; 
    rhs1 = 0;  
    rhs2 = w(1)*gg(:,n+1);
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,n+1-k);
        rhs1 = rhs1 + w2(k+1)*uu(:,n+1-k);
        rhs2 = rhs2 + w(k+1)*gg(:,n+1-k);
    end
    rhs = mm*rhs - ss*rhs1 - bnd*[ua;ub] - rhs2....
        - B(n+1)*(ss*u0) - C(n)*(ss*u10);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
% e = norm(uu(:,end)-uv(xx,T),inf);
% % e2 = h/3*(uu(:,end)-uv(xx,T)).^2;  e2 = sqrt(sum(e2));
% e2 = uu(:,end)-uv(xx,T); e2=sqrt(e2'*mm0*e2);
% % e3 = funuv(uv,xx,tt);    e = max(max(abs(uu(:,3:end)-e3(:,3:end))));
% % e = norm(uu(:,end)-uv(xx,T),inf);
% % e2 = (uu(:,end)-uv(xx,T)); 
% % e2 = sqrt(sum(h/p*e2.^2));
% e2 = sqrt(e2'*mm0*e2);
e3 = funuv(uv,xx,tt)-uu;  e = max(max(abs(e3)));  
e4 = zeros(1,ne);  
for k=1:ne
    e4(k) = (e3(:,k)'*mm0*e3(:,k));
end
e = max(sqrt(e4));    
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
%END

function [uu,e,e2,e3]=pro_fdm4(uv,gv,uav,ubv,N,tau,nodes,nu,T,alpha,ut0,q)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    alf = alpha(1);  bta = alpha(2);
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
p=1;
% % % % % % 
ne = round(T/tau)+1;  r = nu*tau^alf*alf/2;             
w = weights(alf,ne,1);  
w3 = tau^alf*[1-alf/2,alf/2];
w2 = r;
b = w; 
for k=1:ne
    b(k+1) = b(k) + w(k+1);
end
w = -w;
% % % % % % % % % % % % % % % % % % % % % % % % % %
ww3 = zeros(ne,1); ww3(1:2) = [1-alf/2,alf/2];
k=0:ne;
B0 = k.^(alf)/gamma(alf+1);  B=B0;
for k=1:ne
    B(k) = nu*tau^alf*(-B0(1:k)*w(k:-1:1)-sum(ww3(1:k)));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
om0 = weights(alf,ne,1);
om1 = zeros(ne+1,1);    om1(1:2) = [1-alf/2,alf/2];
om2 = weights(-alf,ne,1).';
om = zeros(ne,1);
for k=1:ne
    om(k) = om2(1:k)*om1(k:-1:1);
end
wn1 = (1:ne).^(1+alf)/gamma(2+alf);
k = 1:ne;
for n=1:ne
    wn1(n) =  wn1(n) - k(1:n)*om(n:-1:1);
end
kk = k.^(alf+1)/gamma(2+alf);
C = B; CC = C;
for n=1:ne
    C(n) = wn1(1:n)*om0(n:-1:1);
    CC(n) = kk(1:n)*om0(n:-1:1) - k(1:n)*om1(n:-1:1);
end
C = nu*tau^alf*CC*0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % % % % % % % % % % % % % % % % %
% [mm,ss] = basis(p,N,xx); mm0 = mm;
% mm = eye(N+1)*h; ss = zeros(N+1);
% for i=2:N
%     ss(i,i-1:i+1) = [-1,2,-1];
% end
% ss = ss/h;
% mm0=mm;
% % % % % % % % % % % % 
mm = eye(N+1)*h; mm0 = mm;  mm = mm(2:end-1,:); 
% sel = 4;          
[ss1,ss2] = frac_basis(bta,N,q);
ss = -(ss1 + ss2)/h^(bta-1);
% % % % % % % % % % 
arr = mm + nu*tau^alf*(1-alf/2)*ss;
arr0 = mm + nu*tau^alf*(1-alf/2)*ss + C(1)*ss;  
% arr = arr(2:end-1,:);          arr0 = sparse(arr0(2:end-1,:));
bnd = [arr(:,1),arr(:,end)];   bnd0 = [arr0(:,1),arr0(:,end)];
arr = (arr(:,2:end-1));  arr0 = (arr0(:,2:end-1)); 
% mm = sparse(mm(2:end-1,:)); ss=sparse(ss(2:end-1,:));
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);   uu(:,1) = u0;  
ff = zeros(p*N+1,ne);   tt = 0:tau:T+1e-10;
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
for k=1:ne
    ff(:,k) = gv(xx,tt(k));
end
a0 = 1-alf;
% gg2 = fint(ff,tt,alf,a0);
gg2 = fint2(ff,tt,alf,a0,1);   % fractonal integral of the right hand side
gg = mm*gg2;
% % % % % % % % % % % First step % % % % % % % % % % % % 
n = 1;      
ua = uav(tt(n+1));   ub = ubv(tt(n+1));
rhs =  b(n+1)*u0;
rhs1 = w(1)*gg(:,n+1);
for k=1:n
    rhs = rhs + w(k+1)*uu(:,n+1-k);
    rhs1 = rhs1 + w(k+1)*gg(:,n+1-k);
end
rhs = mm*rhs - w2*(ss*uu(:,n)) - bnd0*[ua;ub] - B(n+1)*(ss*u0)....
    - rhs1 + C(n)*(ss*u0);
u2 = arr0\rhs;
uu(:,n+1) = [ua;u2;ub];  
u1 = [ua;u2;ub];     u10 = u1-u0;
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
for n = 2:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
%     rhs = w3(1)*ff(:,n+1) + w3(2)*ff(:,n) + b(n+1)*u0; 
    rhs =  b(n+1)*u0; 
    rhs1 = w(1)*gg(:,n+1);
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,n+1-k);
        rhs1 = rhs1 + w(k+1)*gg(:,n+1-k);
    end
    rhs = mm*rhs - w2*(ss*uu(:,n)) - bnd*[ua;ub] - B(n+1)*(ss*u0).....
        - rhs1 - C(n)*(ss*u10);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
% % e = norm(uu(:,end)-uv(xx,T),inf);
% % e2 = (uu(:,end)-uv(xx,T)); 
% % e2 = sqrt(sum(h/p*e2.^2));

e3 = funuv(uv,xx,tt)-uu;   e = max(max(abs(e3))); 
e4 = zeros(1,ne);  
for k=1:ne
    e4(k) = (e3(:,k)'*mm0*e3(:,k));
end
e = max(sqrt(e4)); 
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
% % e2 = sqrt(e2'*mm0*e2);
% % e3 = funuv(uv,xx,tt);    e = max(max(abs(uu(:,3:end)-e3(:,3:end))));
% % eee = max(abs(uu-e3));
% % eee;
% %END
function [uu,e,e2,e3]=pro_fdm5(uv,gv,uav,ubv,N,tau,nodes,nu,T,alpha,ut0,a1,a2,a3,q)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
alf = alpha(1);  bta = alpha(2); 
p=1;
% % % % % % 
ne = round(T/tau)+1;    r = nu*(tau/2)^alf;             
w = weights(alf,ne,1);
w2 = w;   w2(2:2:end) = -w2(2:2:end); 
w3 = (tau/2)^alf*w2;    w2 = r*w2;
b = w; 
for k=1:ne
    b(k+1) = b(k)+w(k+1);
end
w = -w;
% 
ww3 = (1/2)^alf*weights(alf,ne,1); ww3(2:2:end)=-ww3(2:2:end);
k=0:ne-1;
B0 = k.^(alf)/gamma(alf+1);  B = B0;
for k=1:ne
    B(k) = (-B0(1:k)*w(k:-1:1)-sum(ww3(1:k)));
end
B = nu*tau^alf*B;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
om0 = weights(alf,ne,1);
om1 = om0/2^(alf);      om1(2:2:end) = - om1(2:2:end);
c0 = B;      
nv = a1;    %% The method is exact for u(t) = t^nv
k = gamma(nv+1)/gamma(nv+1+alf)*(1:ne).^(alf+nv);
kk = (1:ne).^nv;
for n = 1:ne
    c0(n) = k(1:n)*om0(n:-1:1) - kk(1:n)*om1(n:-1:1);
end
C = nu*tau^alf*c0;  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% [mm,ss] = basis(p,N,xx);  mm0 = mm;
% mm = eye(N+1)*h; ss = zeros(N+1);
% for i=2:N
%     ss(i,i-1:i+1) = [-1,2,-1];
% end
% ss = ss/h;
% mm0=mm;

mm = eye(N+1)*h; mm0 = mm;  mm = mm(2:end-1,:); 
% sel = 1;          
[ss1,ss2] = frac_basis(bta,N,q);
ss = -(ss1 + ss2)/h^(bta-1);
% % % % % % % % % % 
arr = mm + nu*(tau/2)^alf*ss;   
arr0 = mm + nu*(tau/2)^alf*ss + C(1)*ss;   
bnd = [arr(:,1),arr(:,end)];    bnd0 = [arr0(:,1),arr0(:,end)]; 
arr = arr(:,2:end-1);           arr0 = arr0(:,2:end-1); 
% mm = sparse(mm(2:end-1,:));     ss = sparse(ss(2:end-1,:));
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);   uu(:,1) = u0; 
tt = 0:tau:T+1e-10;
ff = zeros(p*N+1,ne);
for k=1:ne
    ff(:,k) = gv(xx,tt(k));
end
% a0 = 1-alf; a0=alf; a1=1; % 
% gg2 = fint(ff,tt,alf,a0);   % fractonal integral of the right hand side
gg2 = fint2(ff,tt,alf,a2,a3);   % fractonal integral of the right hand side
gg = mm*gg2;    

% nv = 2 + alf;
% gint = @(x,t)(t^(nv) + t)*sin(x).....
%     + nu*(gamma(nv+1)/gamma(nv+1+alf)*t^(nv+alf) ....
%     + 1/gamma(2+alf)*t^(1+alf) + 2/gamma(1+alf)*t^(alf))*sin(x);
% for k=1:ne
%     gg2(:,k) = gint(xx,tt(k));
% end
% gg = mm*gg2;
% % % % % % % % % % % First step % % % % % % % % % % % % 
n = 1;      
ua = uav(tt(n+1));   ub = ubv(tt(n+1));
rhs =  b(n+1)*u0;
rhs1 = 0;
rhs2 = w(1)*gg(:,n+1);
for k=1:n
    rhs = rhs + w(k+1)*uu(:,n+1-k);
    rhs1 = rhs1 + w2(k+1)*uu(:,n+1-k);
    rhs2 = rhs2 + w(k+1)*gg(:,n+1-k);
end
rhs = mm*rhs - ss*rhs1 - bnd0*[ua;ub] - rhs2 - B(n+1)*(ss*u0) + C(n)*(ss*u0);
u2 = arr0\rhs;
uu(:,n+1) = [ua;u2;ub];  
u1 = [ua;u2;ub];     u10 = u1-u0;
% % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
for n = 2:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs =  b(n+1)*u0; 
    rhs1 = 0;  
    rhs2 = w(1)*gg(:,n+1);
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,n+1-k);
        rhs1 = rhs1 + w2(k+1)*uu(:,n+1-k);
        rhs2 = rhs2 + w(k+1)*gg(:,n+1-k);
    end
    rhs = mm*rhs - ss*rhs1 - bnd*[ua;ub] - rhs2....
        - B(n+1)*(ss*u0) - C(n)*(ss*u10);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
% e = norm(uu(:,end)-uv(xx,T),inf);
% % e2 = h/3*(uu(:,end)-uv(xx,T)).^2;  e2 = sqrt(sum(e2));
% e2 = uu(:,end)-uv(xx,T); e2=sqrt(e2'*mm0*e2);
% % e3 = funuv(uv,xx,tt);    e = max(max(abs(uu(:,3:end)-e3(:,3:end))));
% % e = norm(uu(:,end)-uv(xx,T),inf);
% % e2 = (uu(:,end)-uv(xx,T)); 
% % e2 = sqrt(sum(h/p*e2.^2));
% e2 = sqrt(e2'*mm0*e2);
e3 = funuv(uv,xx,tt)-uu;   e = max(max(abs(e3(:,ne))));
e4 = zeros(1,ne);  
for k=1:ne
    e4(k) = (e3(:,k)'*mm0*e3(:,k));
end
e = max(sqrt(e4));    
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
%END

function [uu,e,e2,e3]=pro_fdm6(uv,gv,uav,ubv,N,tau,nodes,nu,T,alpha,ut0,a1,a2,a3,q)
% % uniform grids
xa = nodes(1);   xb=nodes(2);     alf = alpha(1);  bta = alpha(2);
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
p=1;
% % % % % % 
ne = round(T/tau)+1;  r = nu*tau^alf*alf/2;             
w = weights(alf,ne,1);  
w3 = tau^alf*[1-alf/2,alf/2];
w2 = r;
b = w; 
for k=1:ne
    b(k+1) = b(k) + w(k+1);
end
w = -w;
% % % % % % % % % % % % % % % % % % % % % % % % % %
ww3 = zeros(ne,1); ww3(1:2) = [1-alf/2,alf/2];
k=0:ne;
B0 = k.^(alf)/gamma(alf+1);  B=B0;
for k=1:ne
    B(k) = nu*tau^alf*(-B0(1:k)*w(k:-1:1)-sum(ww3(1:k)));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
om0 = weights(alf,ne,1);
% c0 = B;  
nv = a1;  %% The method is exact for u(t) = t^nv
k = gamma(nv+1)/gamma(nv+1+alf)*(1:ne).^(alf+nv);
% for n = 1:ne
%     c0(n) = k(1:n)*om0(n:-1:1) - (ww3(1)*n^nv + ww3(2)*(n-1)^nv);
% end
k1=(1:ne).^nv;
c0 = fast_conv(k(1:ne),om0(1:ne)) - fast_conv(ww3(1:ne),k1(1:ne));
C = nu*tau^alf*c0;
% n0 = log2(ne); n0 = ceil(n0); n0 = 2^n0;
% k0 = zeros(1,n0);  k1 = k0; k2 = k0;  k3 = k0;
% k0(1:ne) = gamma(nv+1)/gamma(nv+1+alf)*(1:ne).^(alf+nv);
% k1(1:ne) = om0(1:ne);  
% k2(1:ne) = (0:ne-1).^nv;
% k3(1:2) = ww3(1:2);
% 
% fftk0 = fft(k0);  fftk1 = fft(k1);
% k0k1 = ifft(fftk0.*fftk1);
% 
% fftk2 = fft(k2);  fftk3 = fft(k3);
% k2k3 = ifft(fftk2.*fftk3);
% cc2 = nu*tau^alf*(k0k1-k2k3);
% cc = nu*tau^alf*(conv(k0,k1)-conv(k2,k3));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % % % % % % % % % % % % % % % % %
% [mm,ss] = basis(p,N,xx); mm0 = mm;
% mm = eye(N+1)*h; ss = zeros(N+1);
% for i=2:N
%     ss(i,i-1:i+1) = [-1,2,-1];
% end
% ss = ss/h;
% mm0=mm;
% % % % % % % % % % % % % % 
mm = eye(N+1)*h; mm0 = mm;  mm = mm(2:end-1,:); 
% sel = 4;          
[ss1,ss2] = frac_basis(bta,N,q);
ss = -(ss1 + ss2)/h^(bta-1);
% % % % % % % % % % 
arr = mm + nu*tau^alf*(1-alf/2)*ss;
arr0 = mm + nu*tau^alf*(1-alf/2)*ss + C(1)*ss;  
% arr = arr(2:end-1,:);          arr0 = sparse(arr0(2:end-1,:));
bnd = [arr(:,1),arr(:,end)];   bnd0 = [arr0(:,1),arr0(:,end)];
arr = (arr(:,2:end-1));  arr0 = (arr0(:,2:end-1)); 
% mm = sparse(mm(2:end-1,:));  ss = (ss(2:end-1,:));
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);   uu(:,1) = u0;  
ff = zeros(p*N+1,ne);   tt = 0:tau:T+1e-10;
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
for k=1:ne
    ff(:,k) = gv(xx,tt(k));
end
% a0 = 1-alf;   a0 = alf;
% gg2 = fint(ff,tt,alf,a0);
gg2 = fint2(ff,tt,alf,a2,a3);   % fractonal integral of the right hand side
gg = mm*gg2;
% % % % % % % % % % % First step % % % % % % % % % % % % 
n = 1;      
ua = uav(tt(n+1));   ub = ubv(tt(n+1));
rhs =  b(n+1)*u0;
rhs1 = w(1)*gg(:,n+1);
for k=1:n
    rhs = rhs + w(k+1)*uu(:,n+1-k);
    rhs1 = rhs1 + w(k+1)*gg(:,n+1-k);
end
rhs = mm*rhs - w2*(ss*uu(:,n)) - bnd0*[ua;ub] - B(n+1)*(ss*u0)....
    - rhs1 + C(n)*(ss*u0);
u2 = arr0\rhs;
uu(:,n+1) = [ua;u2;ub];  
u1 = [ua;u2;ub];     u10 = u1-u0;
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
for n = 2:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
%     rhs = w3(1)*ff(:,n+1) + w3(2)*ff(:,n) + b(n+1)*u0; 
    rhs =  b(n+1)*u0; 
    rhs1 = w(1)*gg(:,n+1);
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,n+1-k);
        rhs1 = rhs1 + w(k+1)*gg(:,n+1-k);
    end
    rhs = mm*rhs - w2*(ss*uu(:,n)) - bnd*[ua;ub] - B(n+1)*(ss*u0).....
        - rhs1 - C(n)*(ss*u10);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
% % e = norm(uu(:,end)-uv(xx,T),inf);
% % e2 = (uu(:,end)-uv(xx,T)); 
% % e2 = sqrt(sum(h/p*e2.^2));

e3 = funuv(uv,xx,tt)-uu;    e = max(max(abs(e3(:,ne))));
e4 = zeros(1,ne);  
for k=1:ne
    e4(k) = (e3(:,k)'*mm0*e3(:,k));
end
e = max(sqrt(e4)); 
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
% % e2 = sqrt(e2'*mm0*e2);
% % e3 = funuv(uv,xx,tt);    e = max(max(abs(uu(:,3:end)-e3(:,3:end))));
% % eee = max(abs(uu-e3));
% % eee;
% %END

function [uu,e,e2,e3]=L1_fdm(uv,gv,uav,ubv,N,tau,nodes,nu,T,alpha,ut0,q)
% % uniform grids
xa = nodes(1);   xb=nodes(2);     alf = alpha(1);  bta = alpha(2);
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
p=1;
% % % % % % 
ne = round(T/tau)+1;  
r = nu*tau^alf*gamma(2-alf);     r0 = tau^alf*gamma(2-alf);        
% w = weights(alf,ne,1);  
k = 0:ne; 
b = (k+1).^(1-alf) - k.^(1-alf);
w = b(1:end-1) - b(2:end);
mm = eye(N+1)*h; mm0 = mm;  mm = mm(2:end-1,:); 
% sel = 4;          
[ss1,ss2] = frac_basis(bta,N,q);
ss = -(ss1 + ss2)/h^(bta-1);
% % % % % % % % % % % % % % % % % % % % 
arr = mm + r*ss;
bnd = [arr(:,1),arr(:,end)]; 
arr = arr(:,2:end-1);       
% arr = sparse(arr);
% mm = sparse(mm(2:end-1,:)); ss= (ss(2:end-1,:));
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);   uu(:,1) = u0;  
ff = zeros(p*N+1,ne);   tt = 0:tau:T+1e-10;
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
for k=1:ne
    ff(:,k) = gv(xx,tt(k));
end
% a0 = 1-alf;
% gg2 = fint(ff,tt,alf,a0);
gg = r0*(mm*ff);
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
for n = 1:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
%     rhs = w3(1)*ff(:,n+1) + w3(2)*ff(:,n) + b(n+1)*u0; 
    rhs =  b(n)*u0; 
    for k=1:n-1
        rhs = rhs + w(k)*uu(:,n+1-k);
    end
    rhs = mm*rhs  - bnd*[ua;ub]  + gg(:,n+1);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
% 
e3 = funuv(uv,xx,tt)-uu;   e = max(max(abs(e3))); 
e4 = zeros(1,ne);  
for k=1:ne
    e4(k) = (e3(:,k)'*mm0*e3(:,k));
end
e = sqrt(max(max(abs(e4)))); 
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
%END



function [uv,gv,N,tau,nodes,nu,T,alpha,uav,ubv,ut0,a1,a2,a3]=solution(sel,alf,bta)
switch sel
    case 0
        nu = 1;          nodes = [0,1];
        xa = nodes(1);   xb = nodes(2);
%         alf = 0.9;       
        alpha = alf;  T = 1;  
%         N = [20,40,80,160,320,640,800]';
%         tau = 1./N;
%         N = [40,80,160,320,640]'/4;
%         N = [16,32,64,128,256]'/2;  
%         N=[10 50 100 400 800]';
%         N = [200,400,800,1000,2000]';  
%         tau = 1./N;
        N = [1000,1000,1000,1000,1000]';
%         N = [1000 1000 1000 1000 1000 1000 1000]';
        tau = 1./N; N = [16,32,64,128,256]'/2; 
%         N = [20,40,80,160,320,640,800]';
        nv = 2 + alf;  mu = 1;  c = 1;  d = 2;
        a1 = mu; a2 = 1-alf; a3 = 1;
        uv = @(x,t) (x.^4.*(1-x).^4)*(t^nv + c*t^mu + d);
        gv = @(x,t)(gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf).....
                 + c*gamma(mu+1)/gamma(mu+1-alf)*t^(mu-alf))*(x.^4.*(1-x).^4).....
                 - nu*(t^nv + c*t^mu + d)*(gamma(9)/gamma(9-bta)*x.^(8-bta) - 4*gamma(8)/gamma(8-bta)*x.^(7-bta).....
            + 6*gamma(7)/gamma(7-bta)*x.^(6-bta) - 4*gamma(6)/gamma(6-bta)*x.^(5-bta) + gamma(5)/gamma(5-bta)*x.^(4-bta))....
            - nu*(t^nv + c*t^mu + d)*(gamma(9)/gamma(9-bta)*((1-x).^(8-bta)) - 4*gamma(8)/gamma(8-bta)*((1-x).^(7-bta)).....
            + 6*gamma(7)/gamma(7-bta)*((1-x).^(6-bta)) - 4*gamma(6)/gamma(6-bta)*((1-x).^(5-bta)) + gamma(5)/gamma(5-bta)*((1-x).^(4-bta)));
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
    case 1
        % Gao and Sun, JCP, 2010
        nu = 1;          nodes = [0,1];
        xa = nodes(1);   xb = nodes(2);
%         alf = 0.8;       
        alpha = alf;  T = 1;  
%         N = [5,10,15,20,25]';
%         tau = 1./N;
%         N = [1000,1000,1000,1000,1000]';
%         N=[400,400,400,400,400,400]';
%         N=[2,4,8]'; tau=1./[1000,1000,1000]'/10;
%         N=[4,8,16,8,16,32]'; tau=1./[4,64,1028,8,128,2048]';  
%         N = [2 4 8 16 32]'; tau = 1./[10000 10000 10000 10000 10000]';
%         N = [100,100,100,100,100,100,100,100]'*10; tau=1./[100 200 400 800 1000 1600 2000 3200]';
        N = [10 20 40 80 160]'; tau=1./N;
        N = [1000,1000,1000,1000,1000]'/10;
        
        nv = alf+1;   
        a1 = nv; a2 = nv-alf; a3 = nv;
%         nv=2.5;
%         nv = 1;
        d=0;
        uv = @(x,t) exp(x)*(t^nv + d);
        gv = @(x,t)gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf)*exp(x).....
                 - (nu*t^nv + d)*exp(x);
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
    case 2
        % Lin and Xun Chuanju, JCP, 225 (2007) 1533-1552
        nu = 1;          nodes = [0,1];
        xa = nodes(1);   xb = nodes(2);
%         alf = 0.5;       
        alpha = alf;  T = 1;  
        N = [10,50,100,400,800,1000]';
        tau = 1./N;
        tau = 1./[16 32 64 128 256 512 1024].';
        N = [200,200,200,200,200,200,200]'*5;
        N=[200,400,800,1000,2000]';  tau=1./N;
        N = [100,100,100,100,100]'*10; 
%         tau=1./[5,10,15,20,25,30,35,40]';  N=[500,500,500,500,500,500,500,500]'*1;
%         N = [10,20,40,80,160,320,500]'; tau=1./N; 
%         N=[200,200,200,200,200,200,200]'; tau=1./[1000,3000,5000,7000,9000,11000,13000]';
%         N=[80,80,80,80,80,80,80]'; tau=1./[1000,3000,5000,7000,9000,11000,13000]';
%         N=[80,80,80,80,80,80,80]'/2; tau=1./[1000,3000,5000,7000,9000,11000,13000]';
        nv = 2;  k = 2*pi;
        a1 = mv; a2 = nv-alf; a3 = 2;
        uv = @(x,t) sin(k*x)*t^nv;
        gv = @(x,t)gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf)*sin(k*x).....
                 + k^2*nu*t^nv*sin(k*x);
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
 case 3
        % Zhang Ya-nan and Sun, SIAM, 2011: Example 2
        nu = 1;          nodes = [0,1];
        xa = nodes(1);   xb = nodes(2);
%         alf = 0.8;       
        alpha = alf;  T = 1;  
%         N = [5,10,15,20,25]';
%         tau = 1./N;
%         N = [1000,1000,1000,1000,1000]';;
%         N=[2,4,8]'; tau=1./[1000,1000,1000]'/10;
        N = [100,100,100,100,100]'*10; 
%         N=[16,32,64,128,256]';
        tau=1./[16,32,64,128,256]'/2;
%         tau = 1./[10 20 40 80 120 160]'; N=[1000,1000,1000,1000,1000,1000]';
%         nv = alf+1;   nv=2.5;
        N=[200,400,800,1000,2000]';  tau=1./N;
        N = [100,100,100,100,100]'*10; 
        nv = 2;  mu = 1;  c = 0;  d = 1;
        a1 = mu; a2 = mu-alf; a3 = 1;
        uv = @(x,t) exp(x)*(t^nv + c*t^mu + d);
        gv = @(x,t)(gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf).....
                 + c*gamma(mu+1)/gamma(mu+1-alf)*t^(mu-alf))*exp(x).....
                 - nu*(t^nv + c*t^mu + d)*exp(x);
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
case 4
        % Zhang Ya-nan and Sun, SIAM, 2011:  Example 1
        nu = 1;          nodes = [0,1];
        xa = nodes(1);   xb = nodes(2);
%         alf = 0.8;       
        alpha = alf;  T = 1;  
%         N = [5,10,15,20,25]';
%         tau = 1./N;
%         N = [1000,1000,1000,1000,1000]';
% %         N=[2,4,8]'; tau=1./[1000,1000,1000]'/10;
%         N = [100,100,100,100,100]'*5; 
        N=[200,400,800,1000,2000]';  tau=1./N;
        N = [100,100,100,100,100]'*2; 
%         N = [10 20 40 80 160 320 640 1280]'; tau=1./N;
%         N = [1000,1000,1000,1000,1000,1000,1000,1000]'*2;
        nv = alf+1;   nv = 2.5;
        nv = 1+alf;
        a1 = nv; a2 = nv-alf; a3 = nv;
%         nv = 1;
        uv = @(x,t) exp(x)*(t^nv);
        gv = @(x,t)(gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf))*exp(x).....
                 - nu*(t^nv)*exp(x);
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
case 5
        % Kassem Mustapha, IMA JNA 2011 (31), 719-739
        nu = 1;          nodes = [0,1];
        xa = nodes(1);   xb = nodes(2);     
        alpha = alf;  T = 1;  
        N=[64,128,256,512]';
%         N=[1000,1000,1000,1000].'*2;
        tau = 1./[81,256,625,1296].';
        tau = 1./[10 64 256 500].';
        nv = alf+1; 
        nv = alf;
        a1 = nv; a2 = nv-alf; a3 = nv;
        uv = @(x,t) (t^nv - gamma(1+alf)/pi^2)*sin(pi*x);
        gv = @(x,t)(gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf))*sin(pi*x).....
                 + nu*(pi^2*t^nv - gamma(1+alf))*sin(pi*x);
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
case 6
 % B. Jin, R. Lazarov, and Z. Zhou,  SIAM J Numer Anal,51,(2013) 445C466
%  (a)
        nu = 1;          nodes = [0,1];    
        alpha = alf;  T = 1;  
        N=[2^3,2^4,2^5,2^6,2^7]';
        a1 = alf; a2 = alf; a3 = 1;
        tau = 1./[100,100,100,100,100].'/10;
%         N = 10; tau=1/1000;
%         nv = 1;
        uv = @(x,t) func(alf,x,t);
        gv = @(x,t) 0.*x;
        uav = @(t) 0;
        ubv = @(t) 0;
        ut0 = @(x) -4*x.^2 + 4*x;
    otherwise
        uv=[]; gv=[];  N=[]; tau=[]; 
        nodes=[]; nu=[]; T=[]; alpha=[]; 
end
%END

function y = func(alf,x,t)
y=0;
t0 = pi^2*t^alf;
for n=50:-1:1
    y = y + (1-(-1)^n)/n^3*mlf(alf,1,-n^2*t0,12)*sin(n*pi*x);
end
y=16/pi^3*y;

function odr=myorder(ntau,NN,err)
n=length(ntau); 
tau(1:n,1)=ntau; N(1:n,1)=NN(1:n); 
er(1:n,1)=err(1:n);
if n==1
    odr=0; return;
end
if n>1
    if abs(ntau(1)-ntau(2))>1e-10
        odr=(log(er(1:end-1)./er(2:end)))./log(tau(1:n-1,:)./tau(2:n,:));
    else 
        odr=(log(er(1:end-1)./er(2:end)))./log(N(1:n-1,:)./N(2:n,:));
    end
end
odr=[0;odr];
%END 

function [w2,w]=weights(alf,n0,p)
n=n0+1;   j = 0:n-1;   x0 = j.'*2*pi/n;
switch p
    case 1
        w = ones(n,1); 
        f=@(z)(1-z).^alf;
        for k=1:n-1
            w(k+1)=(k-1-alf)/k*w(k); 
        end
        w2=w;
        w = real(fft(f(exp(i*x0))))/n;
    case 2
        f=@(z)(3/2 - 2*z + 0.5*z.^2).^alf;
%         fs=(3/2 - 2*t + t^2/2).^alf;
%         d1f=diff(fs,t); d1f=subs(d1f,t,0);
%         d2f=diff(fs,t,2); d2f=subs(d2f,t,0);
        w = real(fft(f(exp(i*x0))))/n;
        u=zeros(n,1);  u(1:3) = [3/2;  -2; 1/2];  
        w2=w; w2(1) = (3/2)^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
%         eee=max(w2-w)
%         eee
    case 3
        f=@(z)(11/6 - 3*z + 3/2*z.^2 - z.^3/3).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [11/6; -3; 3/2;-1/3];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
        eee;
    case 4
        f=@(z)(25/12 - 4*z + 3*z.^2 - 4/3*z.^3 + 1/4*z.^4).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [25/12; -4; 3;-4/3;1/4];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
        eee;
    case 5
        f=@(z)(137/60 - 5*z + 5*z.^2 - 10*z.^3/3 + 5*z.^4/4 - z.^5/5).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [137/60; -5; 5; -10/3; 5/4; -1/5];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
        eee;
    case 6
        f=@(z)(147/60 - 6*z + 15/2*z.^2 - 20/3*z.^3 + 15/4*z.^4 - 6/5*z.^5 +z.^6/6).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [147/60; -6; 15/2; -20/3; 15/4; -6/5; 1/6];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
%         eee
end


function output(s,A)
[row,col] = size(A);
fprintf('%-10s%-10s%-16s%-8s%-16s%-8s%-16s%-8s',s{1},s{2},s{3},s{4},s{5},s{6},s{7},s{8});
fprintf('\n');
for k=1:row
    fprintf('%-10s%-10s%-16.4e%-8.4f%-16.4e%-8.4f%-16.4e%-8.4f',.....
        num2str(A(k,1)),num2str(A(k,2)),A(k,3),A(k,4),A(k,5),A(k,6),A(k,7),A(k,8));
    fprintf('\n');
end
fprintf('\n');
% END

function output1(s,A,fid)
[row,col] = size(A);
fprintf(fid,'%-10s%-10s%-16s%-8s%-16s%-8s%-16s%-8s',s{1},s{2},s{3},s{4},s{5},s{6},s{7},s{8});
fprintf(fid,'\n');
for k=1:row
    fprintf(fid,'%-10s%-10s%-16.4e%-8.4f%-16.4e%-8.4f%-16.4e%-8.4f',.....
        num2str(A(k,1)),num2str(A(k,2)),A(k,3),A(k,4),A(k,5),A(k,6),A(k,7),A(k,8));
    fprintf(fid,'\n');
end
fprintf(fid,'\n');
% END
function u=funuv(uv,x,t)
nt=length(t); N=length(x);
u=zeros(N,nt);
for k=1:nt
    u(:,k)=uv(x,t(k));
end

function gg2 = fint2(ff,tt,alf,a0,a1)
% ne = length(tt);
[rows,ne] = size(ff);      tau = tt(2)-tt(1);
w0 = weights(-alf,ne,1);   ww2 = w0;
ww2(1) = (1-alf/2)*w0(1);  
ww2(2:end) = (1-alf/2)*w0(2:end)+alf/2*w0(1:end-1);
% a0 = 1-alf; 
wj = w0;   j = (0:ne).^a0;
wj1 = w0;  j1 = (0:ne).^a1;
for n=0:ne
    b1 = gamma(a0+1)/gamma(a0+1+alf)*n^(alf+a0) - j(1:n+1)*ww2(n+1:-1:1);
    b2 = gamma(a1+1)/gamma(a1+1+alf)*n^(alf+a1) - j1(1:n+1)*ww2(n+1:-1:1);
    wj1(n+1) = (b2-b1)/(2^a1-2^a0);
    wj(n+1) = b2 - wj1(n+1)*2^a1;
end
% 
ww2 = (tau)^alf*ww2;    wj = (tau)^alf*wj;  wj1 = (tau)^alf*wj1;
gv0 = ff(:,1);  ff2 = ff;
for k=1:ne
    ff2(:,k) = ff(:,k) - gv0;
end
gg2 = zeros(rows,ne);
for n = 1:ne
    tem = wj(n)*ff2(:,2) + wj1(n)*ff2(:,3);
    for k=1:n
        tem = tem + ww2(n+1-k)*ff2(:,k);
    end
    gg2(:,n) = tem + tt(n)^alf/gamma(alf+1)*gv0;
end
%END


function y = fast_conv(u,v)
n1 = length(u); n2 = length(v);
n = n1+n2-1;    nm = max(n1,n2);
u1 = zeros(1,n); v1 = u1;
u1(1:n1) = u; v1(1:n2) = v;
y = ifft(fft(u1).*fft(v1));
y = y(1:nm);
%END

function [arr1,arr2] = frac_basis(alf,N,sel)
arr1 = zeros(N-1, N+1); arr2 = arr1;
w  = weights(alf,N+2,1)';
switch sel
    case 1
        for i=1:N-1
            arr1(i,1:i+2) = w(i+2:-1:1);  arr2(i,i:N+1) = w(1:N+2-i);
        end
    case 2
        arr1 = w_arr_or2(alf,N);  arr1 = arr1(2:end-1,:);  arr2 = arr1;
        arr1 = arr1*cos(alf*pi/2); arr2 = arr2*cos(alf*pi/2); 
    case 3
        ww = w; ww(1) = alf/2*w(1); 
        ww(2:end) = alf/2*w(2:end) + (2-alf)/2*w(1:end-1);
        for i=1:N-1
            arr1(i,1:i+2) = ww(i+2:-1:1);  arr2(i,i:N+1) = ww(1:N+2-i);
        end
    case 4
        ww = w; ww(1) = (2+alf)/4*w(1);  ww(2) = (2+alf)/4*w(2);
        ww(3:end) = (2+alf)/4*w(3:end) + (2-alf)/4*w(1:end-2);
        for i=1:N-1
            arr1(i,1:i+2) = ww(i+2:-1:1);  arr2(i,i:N+1) = ww(1:N+2-i);
        end
    case 5
        for i=1:N-1
            arr1(i,1:i+1) = w(i+1:-1:1);  arr2(i,i+1:N+1) = w(1:N+1-i);
        end
    otherwise
        arr1 = []; arr2 = [];
end


function w = w_arr_or2(alf,N)
% second order matrix
w0 = weight_order2(N+1,alf);   w = zeros(N+1);        
for i=1:N+1
    w(i,1:i) = w0(i:-1:1);  w(i,i+1:N+1) = w0(2:N+2-i);   
end
w;
% END
function w = weight_order2(ne,alf)
w = zeros(ne+1,1);
w(1) = gamma(alf+1)/gamma(alf/2+1)^2;
for k=0:ne
    w(k+2) = (k-alf/2)/(alf/2+k+1)*w(k+1);
end
