function space_frac_eq_rl
% u_t = nu*D^{1-alf}_{0,t}D_{|x|}^{beta}u + g
% Parameters for Tables 4.22 (page 184) and 4.23 (page 185)
% Table 4.22: sel = 0, fdm = 1
% Table 4.23: sel = 1, fdm = 4
sel = 1; 
fdm = 4; 
q = 2;  % q = 1,2,3,4
nalf = [0.2 0.5 0.8];  % fractional order in time
nbeta = [1.2 1.5 1.8]; % fractional order in space
tic
for kk=1:length(nalf)
    alf = [nalf(kk),nbeta(kk)];
    [uv,gv,N,tau,nodes,nu,T,alpha,uav,ubv,ut0,dtu0] = solution(sel,alf(1),alf(2));
    nn=length(tau); ee=zeros(nn,1); ee2=ee;
for k=1:nn
    switch fdm
        case 1
            [uu,e,e2]=fem_L1(uv,gv,uav,ubv,N(k),tau(k),nodes,nu(1),T,alf,ut0,q);
            ee(k) = e; ee2(k) = e2;
        case 2
            [uu,e,e2]=euler_f(uv,gv,uav,ubv,N(k),tau(k),nodes,nu(1),T,alf,ut0,q);
            ee(k) = e; ee2(k) = e2;
        case 3
            [uu,e,e2]=euler_b(uv,gv,uav,ubv,N(k),tau(k),nodes,nu(1),T,alf,ut0,q);
            ee(k) = e; ee2(k) = e2;
        case 4
            [uu,e,e2]=integration_euler_b(uv,gv,uav,ubv,N(k),tau(k),nodes,nu(1),T,alf,ut0,q);
            ee(k) = e; ee2(k) = e2;
        case 5
            [uu,e,e2]=integration_trapezoidal(uv,gv,uav,ubv,N(k),tau(k),nodes,nu(1),T,alf,ut0,q);
            ee(k) = e; ee2(k) = e2;
        case 6
            [uu,e,e2]=integration_euler_f(uv,gv,uav,ubv,N(k),tau(k),nodes,nu(1),T,alf,ut0,q);
            ee(k) = e; ee2(k) = e2;
    end
end
odr = myorder(1./N,tau,ee);  odr2 = myorder(1./N,tau,ee2);  
fid = fopen('fem_L1.txt','a');
fprintf('alf = %s,  T = %s\n',num2str(alf),num2str(T));
A=[N,1./tau,ee,odr,ee2,odr2];
s={'N','1/tau','max-error','order','L^2-error','order'};
output(s,A);
fprintf('cputime = %-10.4f\n\n',toc);
% % % % % % % % % % % % % % % % % % % % % 
fprintf(fid,'alf = %s, T = %s\n',num2str(alf),num2str(T));
output1(s,A,fid); 
fprintf(fid,'cputime = %-10.4f\n\n',toc);
end
% END

function [uu,e,e2]=fdm_flmm(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf,ut0,dtu0)
% % uniform grids
%%% FLMM
xa=nodes(1);          xb=nodes(2);    
h = (xb-xa)/N;        xx = xa:h:xb+1e-10;  xx=xx';
% % % % % % 
ne = round(T/tau)+1;  r = nu*tau^alf/h^2;             
w = weights(1-alf,ne,1);
w = w*r;
% b(k+1) = b(k) + w(k+1);
% w = -w;
% % % % % % 
ss = sparse(zeros(N-1,N+1));
for k=1:N-1
    ss(k,k:k+2)=[1,-2,1];
end
mm = sparse(eye(N-1)); 
mm = [sparse(zeros(N-1,1)),mm,sparse(zeros(N-1,1))];
arr =  (1+alf)*mm - r*ss;
arrR = (1+alf)*mm + r*ss;
bnd = [arr(:,1),arr(:,end)]; 
arr = arr(:,2:end-1); 
arr = sparse(arr); ss=sparse(ss);
%------------- initial ------------
uu = zeros(N+1,ne);  
uu(:,2)=ut0(0)+tau*dtu0(xx,0);
tt = 0:tau:T+1e-10;
ff = zeros(N-1,ne);
for k=1:ne
    ff(:,k)=gv(xx(2:end-1),tt(k));
end
ff = (1+alf)*(ff(:,2:end-1)+ff(:,3:end))/2....
    + (1-alf)*(ff(:,1:end-2)+ff(:,2:end-1))/2;
ff = ff*tau;
%-----------------------------
for n = 2:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs1 = -bnd*[ua;ub] + arrR*uu(:,n) + ff(:,n-1).....
        - (1-alf)*(uu(2:end-1,n) - uu(2:end-1,n-1)); 
    rhs = 0;
    for k=1:n-1
        rhs = rhs + w(n+1-k)*(uu(:,k)+uu(:,k+1));
    end    
    rhs = rhs1 + (ss*rhs);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
e = norm(uu(:,end)-uv(xx,T),inf);
e2 = h*(uu(:,end)-uv(xx,T)).^2;
e2 = sqrt(sum(e2));
e;
%END

function [uu,e,e2]=fdm_L1(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf0,ut0)
% % uniform grids
xa = nodes(1);         xb = nodes(2);    
h = (xb-xa)/N;         xx = xa:h:xb+1e-10;  xx=xx';
alf = 1-alf0;
% % % % % % 
ne = round(T/tau);
n = 0:ne;
B = 2*((n+1/2).^(1-alf)-n.^(1-alf));  
A = B - (1-alf)*(n+1/2).^(-alf);
b = (n+1).^(1-alf) - n.^(1-alf);
B = b - B;
r = nu*tau^(1-alf)/gamma(2-alf)/h^2;
% % % % % % % % % % % % % % % % % % % % % % %  
ss = sparse(zeros(N-1,N+1));
for k=1:N-1
    ss(k,k) = 1;  ss(k,k+1) = -2;  ss(k,k+2) = 1;
end
mm = sparse(eye(N-1)); mm = [sparse(zeros(N-1,1)),mm,sparse(zeros(N-1,1))];
arr = mm - r/2*ss;
arrR= mm + r/2*ss;
bnd = [arr(:,1),arr(:,end)]; 
arr = arr(:,2:end-1); 
arr = sparse(arr); ss=sparse(ss);
%------------- initial ------------
uu = zeros(N+1,ne); 
u0 = ut0(xx);    uu(:,1) = u0;  
u1 = uv(xx,tau); uu(:,2) = u1;   
tt = 0:tau:T+tau/10;  
ttt= (tt(1:end-1)+tt(2:end))/2;
ff = zeros(N-1,ne);
for k=1:ne
    ff(:,k)=gv(xx(2:end-1),ttt(k));
end
ff=tau*ff;
%-----------------------------
for n = 2:ne      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs = -bnd*[ua;ub] + arrR*uu(:,n) + ff(:,n); 
    rhs1 =  B(n)*(u0+u1) + 2*A(n)*u0; 
    for k=1:n-1
        rhs1 = rhs1 + (b(n-k)-b(n-k+1))*(uu(:,k+1)+uu(:,k));
    end
% % % 
    rhs = rhs - r/2*(ss*rhs1);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
e = norm(uu(:,end)-uv(xx,T),inf);
e2 = h*(uu(:,end)-uv(xx,T)).^2;
e2 = sqrt(sum(e2));
e;
%END

function [uu,e,e2]=cn_zhang(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf0,ut0)
% % uniform grids
% zhang yanan, sun zhizhong
xa = nodes(1);         xb = nodes(2);    
h = (xb-xa)/N;         xx = xa:h:xb+1e-10;  xx=xx';
alf = 1-alf0;
% % % % % % 
ne = round(T/tau);   n = 0:ne;
b = (n+1).^(1-alf) - n.^(1-alf);
r = nu*tau^(1-alf)/gamma(2-alf)/h^2;
% % % % % % % % % % % % % % % % % % % % % % %  
ss = sparse(zeros(N-1,N+1));
for k=1:N-1
    ss(k,k) = 1;  ss(k,k+1) = -2;  ss(k,k+2) = 1;
end
mm = sparse(eye(N-1)); mm = [sparse(zeros(N-1,1)),mm,sparse(zeros(N-1,1))];
arr = mm - r/2*ss;      arrR= mm + r/2*ss;
bnd = [arr(:,1),arr(:,end)]; 
arr = arr(:,2:end-1);    
arr = sparse(arr); ss=sparse(ss);
%------------- initial ------------
uu = zeros(N+1,ne); 
u0 = ut0(xx);    uu(:,1) = u0;  
u1 = uv(xx,tau); uu(:,2) = u1;   
tt = 0:tau:T+tau/10;  
R = r*(1-alf)*(n.^(-alf)+(n+1).^(-alf))/2;
d2u0 = ss*u0;
ff = zeros(N-1,ne);
for k=1:ne
    ff(:,k)=(gv(xx(2:end-1),tt(k))+gv(xx(2:end-1),tt(k+1)))/2;
end
ff=tau*ff;
%--------- first step --------
arr0 = mm - r*ss;      arrR0 = mm - r*alf*ss;
bnd0 = [arr0(:,1),arr0(:,end)]; 
arr0 = arr0(:,2:end-1);  
ua = uav(tt(2));   ub = ubv(tt(2));
rhs = -bnd0*[ua;ub] + arrR0*uu(:,1) + tau*gv(xx(2:end-1),tt(2));
uu(:,2) = [ua;arr0\rhs;ub];
%-----------------------------
for n = 2:ne      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs = -bnd*[ua;ub] + arrR*uu(:,n) + R(n)*d2u0 + ff(:,n); 
    rhs1 =  2*b(n)*u0; 
    for k=1:n-1
        rhs1 = rhs1 + (b(n-k)-b(n-k+1))*(uu(:,k+1)+uu(:,k));
    end
% % % 
    rhs = rhs - r/2*(ss*rhs1) ;
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
e = norm(uu(:,end)-uv(xx,T),inf);
e2 = h*(uu(:,end)-uv(xx,T)).^2;  e2 = sqrt(sum(e2));
e3 = funuv(uv,xx,tt);   e = max(max(abs(uu-e3)));
e2 = norm(sqrt(sum(h*(e3-uu).^2)),inf);
e;
%END

function [uu,e,e2]=fem_L1(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf0,ut0,q)
% % uniform grids
% FDM based on the modified L1 method
xa = nodes(1);   xb=nodes(2);    alf = 1-alf0(1);  bta = alf0(2); 
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
p=1;

% % % % % % 
ne = round(T/tau);
n = 0:ne;
B = 2*((n+1/2).^(1-alf)-n.^(1-alf));   B0 = B(1);
A = B - (1-alf)*(n+1/2).^(-alf);       A0 = A(1);
b = (n+1).^(1-alf) - n.^(1-alf);
B = b - B;
r = nu*tau^(1-alf)/gamma(2-alf);
% % % % % % % % % % % % % % % % % % % % % % %
mm = eye(N+1)*h;   mm = mm(2:end-1,:);  
sel = 4;          
[ss1,ss2] = frac_basis(bta,N,q);
ss = -(ss1 + ss2)/h^(bta-1);

arr = mm + r/2*ss;           arrR= mm - r/2*ss;
bnd = [arr(:,1),arr(:,end)]; 
arr = arr(:,2:end-1);        
% arr = sparse(arr);
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);    uu(:,1) = u0;  
u1 = uv(xx,tau); uu(:,2) = u1;
ff = zeros(p*N+1,ne);
tt = 0:tau:T+1e-10; ttt = (tt(1:end-1)+tt(2:end))/2;
for k=1:ne
    ff(:,k)=gv(xx,ttt(k));
end
ff = tau*ff;
% % % % % % First step:  t = tau   % % % % % % % % % % 
% A00=2*(1/2)^(1-alf)-(1-alf)*(1/2)^(-alf); 
% B00=2*(1/2)^(1-alf); 
arr0 = mm + B0*r/2*ss;    arrR0 = mm - B0*r/2*ss;
bnd0 = [arr0(:,1),arr0(:,end)]; 
arr0 = arr0(:,2:end-1);
ua = uav(tt(2));   ub = ubv(tt(2));
rhs = -bnd0*[ua;ub] + mm*ff(:,1) + arrR0*u0 + r*A0*(ss*u0);
u1 = [ua;arr0\rhs;ub];
uu(:,2) = u1;
% % % % % % % % % % % % % % % % % % % % % % % % % % % 
% u1=uv(xx,tau);  uu(:,2) = u1;
% u1=0;  uu(:,2) = 0;
% 
%-----------------------------
for n = 2:ne      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs = -bnd*[ua;ub] + arrR*uu(:,n) + mm*ff(:,n); 
    rhs1 =  B(n)*(u0+u1) + 2*A(n)*u0; 
    for k=1:n-1
        rhs1 = rhs1 + (b(n-k)-b(n-k+1))*(uu(:,k+1)+uu(:,k));
    end
% % % 
    rhs = rhs + r/2*(ss*rhs1);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
e = norm(uu(:,end)-uv(xx,T),inf);
e2 = h/p*(uu(:,end)-uv(xx,T)).^2;
e2 = sqrt(sum(e2));
e3 = funuv(uv,xx,tt);   e = max(max(abs(uu-e3)));
e2 = norm(sqrt(sum(h/p*(e3-uu).^2)),inf);
e;
%END


function [uu,e,e2]=euler_f(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf0,ut0,q)
% % uniform grids
% Grunwald-Letnikov formula, forward Euler 
xa = nodes(1);   xb=nodes(2);    alf = 1-alf0(1);  bta = alf0(2); 
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
p=1;
% alf = 1-alf0;
% % % % % % 
ne = round(T/tau);
r = nu*tau^alf0(1);
w = weights(alf,ne,1);
% % % % % % % % % % % % % % % % % % % % % % %
% [mm,ss] = basis(p,N,xx);
% mm = eye(N+1)*h; ss = zeros(N+1);
% for i=2:N
%     ss(i,i-1:i+1) = [-1,2,-1];
% end
% ss = ss/h;

mm = eye(N+1)*h;   mm = mm(2:end-1,:);  
sel = 3;          
[ss1,ss2] = frac_basis(bta,N,q);
ss = -(ss1 + ss2)/h^(bta-1);

% mm = sparse(mm(2:end-1,:));  ss=sparse(ss(2:end-1,:));
arr = mm;           arrR = mm;
% arr = arr(2:end-1,:); arrR = arrR(2:end-1,:);
bnd = [arr(:,1),arr(:,end)]; 
arr = arr(:,2:end-1);        
% arr = sparse(arr);
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);    uu(:,1) = u0;  
% u1 = uv(xx,tau); uu(:,2) = u1;
ff = zeros(p*N+1,ne); tt = 0:tau:T+1e-10; 
for k=1:ne
    ff(:,k)=gv(xx,tt(k+1));
end
ff = tau*ff;
%-----------------------------
for n = 1:ne      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs = -bnd*[ua;ub] + arrR*uu(:,n) + mm*ff(:,n); 
    rhs1 =  uu(:,1:n)*w(n:-1:1); 
% % % 
    rhs = rhs - r*(ss*rhs1);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
e = norm(uu(:,end)-uv(xx,T),inf);
e2 = h/p*(uu(:,end)-uv(xx,T)).^2;
e2 = sqrt(sum(e2));
e3 = funuv(uv,xx,tt);   e = max(max(abs(uu-e3)));
e2 = norm(sqrt(sum(h/p*(e3-uu).^2)),inf);
e;
%END
function [uu,e,e2]=euler_b(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf0,ut0,q)
% % uniform grids
%  Grunwald-Letnikov formula, backward Euler 
xa = nodes(1);   xb=nodes(2);    alf = 1-alf0(1);  bta = alf0(2); 
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx; 
p=1;
% alf = 1-alf0;    
r = nu*tau^alf0(1);
% % % % % % 
ne = round(T/tau);      
w = weights(alf,ne,1);  b = w;
for k=1:ne
    b(k+1) = w(k+1) + b(k);
end
b = b - ((0:ne).^(-alf)/gamma(1-alf)).';
% % % % % % % % % % % % % % % % % % % % % % %
% % [mm,ss] = basis(p,N,xx);
% mm = eye(N+1)*h; ss = zeros(N+1);
% for i=2:N
%     ss(i,i-1:i+1) = [-1,2,-1];
% end
% ss = ss/h;
% % mm0=mm;
mm = eye(N+1)*h;   mm = mm(2:end-1,:);  
% sel = 3;          
[ss1,ss2] = frac_basis(bta,N,q);
ss = -(ss1 + ss2)/h^(bta-1);
% mm = sparse(mm(2:end-1,:));  ss = sparse(ss(2:end-1,:));
arr = mm + r*ss;             arrR = mm;
% arr = arr(2:end-1,:); arrR = arrR(2:end-1,:);
bnd = [arr(:,1),arr(:,end)]; 
arr = arr(:,2:end-1);        
% arr = sparse(arr);
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne);  uuu = uu;
u0 = ut0(xx);    uu(:,1) = u0;  
% u1 = uv(xx,tau); uu(:,2) = u1;
ff = zeros(p*N+1,ne); tt = 0:tau:T+1e-10; 
for k=1:ne
    ff(:,k)=gv(xx,tt(k+1));
end
ff = tau*ff;
%-----------------------------
for n = 1:ne      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs = -bnd*[ua;ub] + arrR*uu(:,n) + mm*ff(:,n); 
    rhs1 =  uu(:,1:n)*w(n+1:-1:2); 
% % % 
    rhs = rhs - r*(ss*rhs1) + r*b(n+1)*(ss*u0);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
    uuu(:,n+1) = uu(:,n+1)-u0;
end
e = norm(uu(:,end)-uv(xx,T),inf);
e2 = h/p*(uu(:,end)-uv(xx,T)).^2;
e2 = sqrt(sum(e2));
% e3 = funuv(uv,xx,tt);   
% e = max(max(abs(uu-e3)));
% e2 = norm(sqrt(sum(h/p*(e3-uu).^2)),inf);
% e;
%END

function [uu,e,e2]=integration_euler_f(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf0,ut0,q)
% % uniform grids
%  Integration method, backward Euler 
xa = nodes(1);   xb=nodes(2);    alf = 1-alf0(1);  bta = alf0(2); 
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
p=1;
% alf = 1-alf0;    
r = nu*tau^alf;
% % % % % % 
ne = round(T/tau);   
k = (0:ne).';   b = ((k+1).^alf-k.^alf)/gamma(1+alf);
% % % % % % % % % % % % % % % % % % % % % % %
% % [mm,ss] = basis(p,N,xx);
% mm = eye(N+1)*h; ss = zeros(N+1);
% for i=2:N
%     ss(i,i-1:i+1) = [-1,2,-1];
% end
% ss = ss/h;
% % mm0=mm;

mm = eye(N+1)*h;   mm = mm(2:end-1,:);  
% sel = 3;          
[ss1,ss2] = frac_basis(bta,N,q);
ss = -(ss1 + ss2)/h^(bta-1);

% mm = sparse(mm(2:end-1,:));  ss = sparse(ss(2:end-1,:));
arr = mm;             arrR = mm;
% arr = arr(2:end-1,:); arrR = arrR(2:end-1,:);
bnd = [arr(:,1),arr(:,end)]; 
arr = arr(:,2:end-1);        
% arr = sparse(arr);
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne);  uuu = uu;
u0 = ut0(xx);    uu(:,1) = u0;  
u1 = uv(xx,tau); uu(:,2) = u1;
ff = zeros(p*N+1,ne); tt = 0:tau:T+1e-10; 
for k=1:ne
    ff(:,k) = gv(xx,tt(k+1));
end
ff = tau*ff;
% % % % % % % % % % % % % % % % % % % % % % % % % % % 
n = 1;
ua = uav(tt(n+1));   ub = ubv(tt(n+1));
rhs = -bnd*[ua;ub] + arrR*uu(:,n) + mm*ff(:,n);
rhs1 =  uu(:,1:n)*b(n:-1:1);
% % %
rhs = rhs - r*(ss*rhs1);
u2 = arr\rhs;
uu(:,n+1) = [ua;u2;ub];
%-----------------------------
for n = 2:ne      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs = -bnd*[ua;ub] + arrR*uu(:,n) + mm*ff(:,n); 
    rhs1 =  uu(:,1:n)*b(n:-1:1) - uu(:,1:n-1)*b(n-1:-1:1); 
% % % 
    rhs = rhs - r*(ss*rhs1);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
    uuu(:,n+1) = uu(:,n+1)-u0;
end
e = norm(uu(:,end)-uv(xx,T),inf);
e2 = h/p*(uu(:,end)-uv(xx,T)).^2;
e2 = sqrt(sum(e2));
% e3 = funuv(uv,xx,tt);   
% e = max(max(abs(uu-e3)));
% e2 = norm(sqrt(sum(h/p*(e3-uu).^2)),inf);
% e;
%END

function [uu,e,e2]=integration_euler_b(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf0,ut0,q)
% % uniform grids
%  Integration method, backward Euler 
xa = nodes(1);   xb=nodes(2);    alf = alf0(1);  bta = alf0(2); 
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
p=1;
% alf = 1-alf0;    
r = nu*tau^alf;
% % % % % % 
ne = round(T/tau);   
k = (0:ne).';   b = ((k+1).^alf-k.^alf)/gamma(1+alf);


% % % % % % % % % % % % % % % % % % % % % % %
% % [mm,ss] = basis(p,N,xx);
% mm = eye(N+1)*h; ss = zeros(N+1);
% for i=2:N
%     ss(i,i-1:i+1) = [-1,2,-1];
% end
% ss = ss/h;
% % mm0=mm;

mm = eye(N+1)*h;   mm = mm(2:end-1,:);  
% sel = 4;          
[ss1,ss2] = frac_basis(bta,N,q);
ss = -(ss1 + ss2)/h^(bta-1);

% mm = sparse(mm(2:end-1,:));  ss = sparse(ss(2:end-1,:));
arr = mm + r*b(1)*ss;             arrR = mm;
% arr = arr(2:end-1,:); arrR = arrR(2:end-1,:);
bnd = [arr(:,1),arr(:,end)]; 
arr = arr(:,2:end-1);        
% arr = sparse(arr);
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne);  uuu = uu;
u0 = ut0(xx);    uu(:,1) = u0;  
% u1 = uv(xx,tau); uu(:,2) = u1;
ff = zeros(p*N+1,ne); tt = 0:tau:T+1e-10; 
for k=1:ne
    ff(:,k+1)=gv(xx,tt(k+1));
end
% 
% ff = fint2(ff,tt,1,alf0-1,1); 
% ff = ff(:,2:end) - ff(:,1:end-1);
ff = tau*ff;

% % % % % % % % % % % % % % % % % % % % % % % % % % % 
n=1;
ua = uav(tt(n+1));   ub = ubv(tt(n+1));
rhs = -bnd*[ua;ub] + arrR*uu(:,n) + mm*ff(:,n);
% rhs1 =  uu(:,1:n)*b(n+1:-1:2)- uu(:,1:n)*b(n:-1:1);
% % % %
% rhs = rhs - r*(ss*rhs1);
u2 = arr\rhs;
uu(:,n+1) = [ua;u2;ub];
%-----------------------------
for n = 2:ne      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs = -bnd*[ua;ub] + arrR*uu(:,n) + mm*ff(:,n); 
    rhs1 =  uu(:,2:n)*b(n:-1:2)- uu(:,2:n)*b(n-1:-1:1); 
% % % 
    rhs = rhs - r*(ss*rhs1);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
e = norm(uu(:,end)-uv(xx,T),inf);
e2 = h/p*(uu(:,end)-uv(xx,T)).^2;
e2 = sqrt(sum(e2));
% e3 = funuv(uv,xx,tt);   
% e = max(max(abs(uu-e3)));
% e2 = norm(sqrt(sum(h/p*(e3-uu).^2)),inf);
% e;
%END

function [uu,e,e2]=integration_trapezoidal(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf,ut0,q)
% % uniform grids
%  Integration method, backward Euler 
xa = nodes(1);   xb=nodes(2);    alf0=alf(1); bta=alf(2);
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
p=1;
% alf = 1-alf0;    
r = nu*tau^alf0;
% % % % % % 
ne = round(T/tau);   
k = (0:ne).';   
b = (k+2).^(alf0+1) - 2*(k+1).^(alf0+1) + k.^(alf0+1);
b = [1;b]/gamma(2+alf0);
a = k.^(alf0+1) - (k-alf0).*(k+1).^alf0;  a = a/gamma(2+alf0);
% % % % % % % % % % % % % % % % % % % % % % %
% % [mm,ss] = basis(p,N,xx);
% mm = eye(N+1)*h; ss = zeros(N+1);
% for i=2:N
%     ss(i,i-1:i+1) = [-1,2,-1];
% end
% ss = ss/h;
% % mm0=mm;

mm = eye(N+1)*h; mm0 = mm;  mm = mm(2:end-1,:); 
% sel = 1;          
[ss1,ss2] = frac_basis(bta,N,q);
ss = -(ss1 + ss2)/h^(bta-1);

% mm = sparse(mm(2:end-1,:));  ss = sparse(ss(2:end-1,:));
arr = mm + r*b(1)*ss;             arrR = mm;
% arr = arr(2:end-1,:); arrR = arrR(2:end-1,:);
bnd = [arr(:,1),arr(:,end)];
arr = arr(:,2:end-1);        
% arr = sparse(arr);
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne);  
u0 = ut0(xx);    uu(:,1) = u0;  
% u1 = uv(xx,tau); uu(:,2) = u1;
ff = zeros(p*N+1,ne+1); tt = 0:tau:T+1e-10; 
ttt=tt(1:end-1)+tt(2:end); ttt=ttt/2;
for k=1:ne
    ff(:,k)=gv(xx,ttt(k));
end
ff=tau*ff;
% 
% ff = fint2(ff,tt,1,alf0-1,1); 
% ff = ff(:,2:end) - ff(:,1:end-1);
% ff = tau*(ff(:,1:end-1)+ff(:,2:end))/2;
% % % % % % % % % % % % % % % % % % % % % % % % % % % 
n=1;
ua = uav(tt(n+1));   ub = ubv(tt(n+1));
rhs = -bnd*[ua;ub] + arrR*uu(:,n) + mm*ff(:,n) - r*a(1)*(ss*u0);
% rhs1 =  uu(:,1:n)*b(n+1:-1:2)- uu(:,1:n)*b(n:-1:1);
% % % %
% rhs = rhs - r*(ss*rhs1);
u2 = arr\rhs;
uu(:,n+1) = [ua;u2;ub];
%-----------------------------
for n = 2:ne      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs = -bnd*[ua;ub] + arrR*uu(:,n) + mm*ff(:,n); 
    rhs1 =  uu(:,2:n)*b(n:-1:2)- uu(:,2:n)*b(n-1:-1:1) + (a(n)-a(n-1))*u0; 
% % % 
    rhs = rhs - r*(ss*rhs1);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
e = norm(uu(:,end)-uv(xx,T),inf);
e2 = h/p*(uu(:,end)-uv(xx,T)).^2;
e2 = sqrt(sum(e2));
% e3 = funuv(uv,xx,tt);   
% e = max(max(abs(uu-e3)));
% e2 = norm(sqrt(sum(h/p*(e3-uu).^2)),inf);
% e;
%END


function [uv,gv,N,tau,nodes,nu,T,alpha,uav,ubv,ut0,dtu0]=solution(sel,alf0,bta)
switch sel
    case 0
        nu = 1;          nodes = [0,1]; nu2=0;
        xa = nodes(1);   xb = nodes(2);     
        T = 1;           alf = 1-alf0;
        N = [8,16,32,64,128]';   % Number of grid points in spcace for  Table 4.22 
        tau = 1./[1000 1000 1000 1000 1000].'; % the time stepsize for Table 4.22      
        nv = 2.5;   c = 1; d = 0; mu = 1;
        uv = @(x,t) (x.^4.*(1-x).^4)*(t^nv + c*t^mu + d);
        gv = @(x,t)(nv*t^(nv-1) + c*mu*t^(mu-1))*(x.^4.*(1-x).^4).....
            - nu*((gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf) + c*gamma(mu+1)/gamma(mu+1-alf)*t^(mu-alf) + d*(1-alf)/gamma(2-alf)*t^(-alf))).....
                 *(gamma(9)/gamma(9-bta)*x.^(8-bta) - 4*gamma(8)/gamma(8-bta)*x.^(7-bta).....
            + 6*gamma(7)/gamma(7-bta)*x.^(6-bta) - 4*gamma(6)/gamma(6-bta)*x.^(5-bta) + gamma(5)/gamma(5-bta)*x.^(4-bta))....
            - nu*((gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf) + c*gamma(mu+1)/gamma(mu+1-alf)*t^(mu-alf) + d*(1-alf)/gamma(2-alf)*t^(-alf))).....
                 *(gamma(9)/gamma(9-bta)*((1-x).^(8-bta)) - 4*gamma(8)/gamma(8-bta)*((1-x).^(7-bta)).....
            + 6*gamma(7)/gamma(7-bta)*((1-x).^(6-bta)) - 4*gamma(6)/gamma(6-bta)*((1-x).^(5-bta)) + gamma(5)/gamma(5-bta)*((1-x).^(4-bta)));
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
        dtu0 = @(x,t) nv*t^(nv-1)*sin(x);
        alpha = alf0;
    case 1
        nu = 1;          nodes = [0,1]; nu2=0;
        xa = nodes(1);   xb = nodes(2);     
        T = 1;           alf = 1-alf0;
        tau = 1./[8,16,32,64,128]';   % stepsize in time for Table 4.23
        N = [1000 1000 1000 1000 1000].';   % Number of grid points in spcace for  Table 4.23 
        nv = 2.5;   c = 1; d = 0; mu = 1;
        uv = @(x,t) (x.^4.*(1-x).^4)*(t^nv + c*t^mu + d);
        gv = @(x,t)(nv*t^(nv-1) + c*mu*t^(mu-1))*(x.^4.*(1-x).^4).....
            - nu*((gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf) + c*gamma(mu+1)/gamma(mu+1-alf)*t^(mu-alf) + d*(1-alf)/gamma(2-alf)*t^(-alf))).....
                 *(gamma(9)/gamma(9-bta)*x.^(8-bta) - 4*gamma(8)/gamma(8-bta)*x.^(7-bta).....
            + 6*gamma(7)/gamma(7-bta)*x.^(6-bta) - 4*gamma(6)/gamma(6-bta)*x.^(5-bta) + gamma(5)/gamma(5-bta)*x.^(4-bta))....
            - nu*((gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf) + c*gamma(mu+1)/gamma(mu+1-alf)*t^(mu-alf) + d*(1-alf)/gamma(2-alf)*t^(-alf))).....
                 *(gamma(9)/gamma(9-bta)*((1-x).^(8-bta)) - 4*gamma(8)/gamma(8-bta)*((1-x).^(7-bta)).....
            + 6*gamma(7)/gamma(7-bta)*((1-x).^(6-bta)) - 4*gamma(6)/gamma(6-bta)*((1-x).^(5-bta)) + gamma(5)/gamma(5-bta)*((1-x).^(4-bta)));
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
        dtu0 = @(x,t) nv*t^(nv-1)*sin(x);
        alpha = alf0;
    case 2
        % Chang-Ming Chen F. Liu I. Turner V. Anh,  JCP, 2007
        nu = 1;          nodes = [0,1]; nu2=0;
        xa = nodes(1);   xb = nodes(2);     
        alpha = alf0;  T = 1;  
        alf=1-alf0;
        N = [8 32 8 32]'; tau=1./[8 32 64 1024]';
%         N = [20,40,80,160,320,640]';  tau = 1./N;  
        nv = alf0+1;   
        uv = @(x,t) exp(x)*t^nv ;
        gv = @(x,t)nv*t^(nv-1)*exp(x) - nu*gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf)*exp(x);
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
        dtu0=0;
    case 3
        % Cui, JCP, 2009
        nu = 1;          nodes = [0,1]; nu2=0;
        xa = nodes(1);   xb = nodes(2);     
        alpha = alf0;  T = 1;  
        alf=1-alf0;
        N = [4 8 16 8 16 32]'; tau=1./[4 64 1024 8 128 2048]';
        nv = alf0+1;   
        uv = @(x,t) exp(x)*t^nv ;
        gv = @(x,t)nv*t^(nv-1)*exp(x) - nu*gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf)*exp(x);
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
        dtu0=0;
 case 4
        % P. ZHUANG, F. LIU, V. ANH, AND I. TURNER, SIAM Numer Anal, 2008 
        nu = 0.5;          nodes = [0,1]; nu2=0;
        xa = nodes(1);   xb = nodes(2);     
        alpha = alf0;  T = 1;  
        alf=1-alf0;
        N = [10 15 20 25]'; tau=1./[10 15 20 25]';
%         tau=1./[1000 1000 1000 1000].'/10;
        nv = 3;   
        uv = @(x,t) exp(x)*t^nv ;
        gv = @(x,t)nv*t^(nv-1)*exp(x) - nu*gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf)*exp(x);
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
        dtu0=0;
case 5
        % Zhang Ya-nan and Sun, SIAM, 2011:  Example 1
        nu = 1;          nodes = [0,1];  nu2=0;
        xa = nodes(1);   xb = nodes(2);     
        alpha = alf0;  T = 1;  
        alf=1-alf0;
        N = [8 12 16 20 25]';  tau=1./[1000 1000 1000 1000 1000]';
%         N = [2000 2000 2000 2000 2000]'; tau=1./[16 32 64 128 256]';
%         N = [2000 2000 2000 2000 2000]'/2; tau=1./[32 64 128 256 512]';
%         N = [4 8,16,32,64]'; 
%         tau = 1./[10000 10000,10000,10000,10000]';
        nv = 2;   
        uv = @(x,t) exp(x)*(t^nv + t + 1);
        gv = @(x,t)(nv*t^(nv-1) + 1)*exp(x)....
            - nu*(gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf)....
            + 1/gamma(2-alf)*t^(1-alf)+ 1/gamma(1-alf)*t^(-alf))*exp(x);
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
        dtu0=0;
    case 6
% Liu, Qianqian Yang,Ian Turner,J. Comput. Nonlinear Dynamics
% Xiuling Hu, Luming Zhang,pplied Mathematical Modelling 36 (2012) 4027C4043
        nu = 1;          nodes = [0,1];    nu2 = 1; 
        if no>0 
            alf2 = 1 - varargin{1};
        end
        xa = nodes(1);   xb = nodes(2);     
        alpha = alf0;  T = 1;  
        alf = 1-alf0;  
        N = [8 16 32 64]';  tau=1./[8 32 128 516]';
%         N = [8 16 32 64]';  tau=1./[8 128 2048 32768]';
        N = [2000 2000 2000 2000 2000 2000 2000]'/2; tau=1./[16 32 64 128 256 512 1024]';
%         N=[5 10 20 40]';  tau=1./[25 100 400 1600]';
%         N=[25 50 100 200 400 800 1000]'; tau=1./[25 50 100 200 400 800 1000]';
%           N = 100; tau = 1e-4;
        nv = 2;   
        uv = @(x,t) sin(pi*x)*(t^nv);
        gv = @(x,t)(nv*t^(nv-1)....
            + nu*pi^2*gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf)....
            + nu2*gamma(nv+1)/gamma(nv+1-alf2)*t^(nv-alf2))*sin(pi*x);
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
        dtu0=0;
    otherwise
        uv=[]; gv=[];  N=[]; tau=[]; 
        nodes=[]; nu=[]; T=[]; alpha=[]; 
end
nu=[nu,nu2];
%END

function odr=myorder(ntau,NN,err)
n=length(ntau); 
tau(1:n,1)=ntau; N(1:n,1)=NN(1:n); 
er(1:n,1)=err(1:n);
if n==1
    odr=0; return;
end
if n>1
    if abs(ntau(1)-ntau(2))>1e-10
        odr=(log(er(1:end-1)./er(2:end)))./log(tau(1:n-1,:)./tau(2:n,:));
    else 
        odr=(log(er(1:end-1)./er(2:end)))./log(N(1:n-1,:)./N(2:n,:));
    end
end
odr=[0;odr];
%END 

function [w2,w]=weights(alf,n0,p)
n=n0+1;   j = 0:n-1;   x0 = j.'*2*pi/n;
switch p
    case 1
        w = ones(n,1); 
        f=@(z)(1-z).^alf;
        for k=1:n-1
            w(k+1)=(k-1-alf)/k*w(k); 
        end
        w2=w;
        w = real(fft(f(exp(i*x0))))/n;
    case 2
        f=@(z)(3/2 - 2*z + 0.5*z.^2).^alf;
%         fs=(3/2 - 2*t + t^2/2).^alf;
%         d1f=diff(fs,t); d1f=subs(d1f,t,0);
%         d2f=diff(fs,t,2); d2f=subs(d2f,t,0);
        w = real(fft(f(exp(i*x0))))/n;
        u=zeros(n,1);  u(1:3) = [3/2;  -2; 1/2];  
        w2=w; w2(1) = (3/2)^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
%         eee=max(w2-w)
%         eee
    case 3
        f=@(z)(11/6 - 3*z + 3/2*z.^2 - z.^3/3).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [11/6; -3; 3/2;-1/3];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
        eee;
    case 4
        f=@(z)(25/12 - 4*z + 3*z.^2 - 4/3*z.^3 + 1/4*z.^4).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [25/12; -4; 3;-4/3;1/4];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
        eee;
    case 5
        f=@(z)(137/60 - 5*z + 5*z.^2 - 10*z.^3/3 + 5*z.^4/4 - z.^5/5).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [137/60; -5; 5; -10/3; 5/4; -1/5];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
        eee;
    case 6
        f=@(z)(147/60 - 6*z + 15/2*z.^2 - 20/3*z.^3 + 15/4*z.^4 - 6/5*z.^5 +z.^6/6).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [147/60; -6; 15/2; -20/3; 15/4; -6/5; 1/6];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
%         eee
end

function output(s,A)
[row,col] = size(A);
fprintf('%-10s%-10s%-16s%-8s%-16s%-8s',s{1},s{2},s{3},s{4},s{5},s{6});
fprintf('\n');
for k=1:row
    fprintf('%-10s%-10s%-16.4e%-8.4f%-16.4e %-8.4f',......
        num2str(A(k,1)),num2str(A(k,2)),A(k,3),A(k,4),A(k,5),A(k,6));
    fprintf('\n');
end
% fprintf('\n');
% END

function output1(s,A,fid)
[row,col] = size(A);
fprintf(fid,'%-10s%-10s%-16s%-8s%-16s%-8s',s{1},s{2},s{3},s{4},s{5},s{6});
fprintf(fid,'\n');
for k=1:row
    fprintf(fid,'%-10s%-10s%-16.6e%-8.4f%-16.6e %-8.4f',.....
        num2str(A(k,1)),num2str(A(k,2)),A(k,3),A(k,4),A(k,5),A(k,6));
    fprintf(fid,'\n');
end
fprintf(fid,'\n');
% END
function u=funuv(uv,x,t)
nt=length(t); N=length(x);
u=zeros(N,nt);
for k=1:nt
    u(:,k)=uv(x,t(k));
end


function [arr1,arr2] = frac_basis(alf,N,sel)
arr1 = zeros(N-1, N+1); arr2 = arr1;
w  = weights(alf,N+2,1)';
switch sel
    case 1
        for i=1:N-1
            arr1(i,1:i+2) = w(i+2:-1:1);  arr2(i,i:N+1) = w(1:N+2-i);
        end
    case 2
        arr1 = w_arr_or2(alf,N);  arr1 = arr1(2:end-1,:);  arr2 = arr1;
        arr1 = arr1*cos(alf*pi/2); arr2 = arr2*cos(alf*pi/2); 
    case 3
        ww = w; ww(1) = alf/2*w(1); 
        ww(2:end) = alf/2*w(2:end) + (2-alf)/2*w(1:end-1);
        for i=1:N-1
            arr1(i,1:i+2) = ww(i+2:-1:1);  arr2(i,i:N+1) = ww(1:N+2-i);
        end
    case 4
        ww = w; ww(1) = (2+alf)/4*w(1);  ww(2) = (2+alf)/4*w(2);
        ww(3:end) = (2+alf)/4*w(3:end) + (2-alf)/4*w(1:end-2);
        for i=1:N-1
            arr1(i,1:i+2) = ww(i+2:-1:1);  arr2(i,i:N+1) = ww(1:N+2-i);
        end
    case 5
        for i=1:N-1
            arr1(i,1:i+1) = w(i+1:-1:1);  arr2(i,i+1:N+1) = w(1:N+1-i);
        end
    otherwise
        arr1 = []; arr2 = [];
end


function w = w_arr_or2(alf,N)
% second order matrix
w0 = weight_order2(N+1,alf);   w = zeros(N+1);        
for i=1:N+1
    w(i,1:i) = w0(i:-1:1);  w(i,i+1:N+1) = w0(2:N+2-i);   
end
w;
% END
function w = weight_order2(ne,alf)
w = zeros(ne+1,1);
w(1) = gamma(alf+1)/gamma(alf/2+1)^2;
for k=0:ne
    w(k+2) = (k-alf/2)/(alf/2+k+1)*w(k+1);
end

