function adi_flmm_space_time_eq 
% FLMM finite difference methods for D_t^{alf}u = D^{beta}_{|x|}u + D^{beta}_{|y|}u + f(x,y,t)
% 0<alf<=1 and 1<beta<=2 are fractional orders in time and space respectively
%  Table 4.25: fdm = 5, sel = 1
%  Table 4.26: fdm = 6, sel = 1
%  Table 4.27: alf = 1, fdm = 6, sel = 0
tic
sel = 1;  % select the solution
q = 2; % select the discretization in space
fdm = 5; % select lect the method
[uv,gv,Nx,Ny,ntau,nodes,nodes2,nu,T,nalf,uav,ubv,ucv,udv,ut0,nbeta]=solution(sel);
% [uv,gv,N,tau,nodes,nu,T,alpha,uav,ubv,ut0] = solution(sel,alf);
nn=length(ntau);  ee=zeros(nn,1);  ee2=ee; ee3=ee;ee4=ee;ee5=ee;
for kk=1:nn
    alf = nalf(kk); tau=ntau(kk); N=Nx(kk); N2=Ny(kk); bta=nbeta(kk);
    switch fdm
        case 3 % program for method (4.329)
        [uu,e,e2,e3,abse1,abse2]=adi_fdm21(uv,gv,uav,ubv,ucv,udv,N,N2,tau,nodes,nodes2,nu,T,alf,ut0,bta,q);
        case 4 % program for method (4.331)
        [uu,e,e2,e3,abse1,abse2]=adi_fdm22(uv,gv,uav,ubv,ucv,udv,N,N2,tau,nodes,nodes2,nu,T,alf,ut0,bta,q);
        case 5 % program for method (4.333)
        [uu,e,e2,e3,abse1,abse2]=adi_fdm31(uv,gv,uav,ubv,ucv,udv,N,N2,tau,nodes,nodes2,nu,T,alf,ut0,bta,q);
        case 6 % program for method (4.335)
        [uu,e,e2,e3,abse1,abse2]=adi_fdm32(uv,gv,uav,ubv,ucv,udv,N,N2,tau,nodes,nodes2,nu,T,alf,ut0,bta,q);
        case 7 % program for method (4.327)
        [uu,e,e2,e3,abse1,abse2]=adi_L1_fdm(uv,gv,uav,ubv,ucv,udv,N,N2,tau,nodes,nodes2,nu,T,alf,ut0,bta,q);
    end
    ee(kk) = e; ee2(kk) = e2; ee3(kk) = e3;
    ee4(kk) = abse1; ee5(kk) = abse2;
end
odr = myorder(1./Nx,ntau,ee);    odr2 = myorder(1./Nx,ntau,ee2);  
odr3 = myorder(1./Nx,ntau,ee3);  odr4 = myorder(1./Nx,ntau,ee4); 
odr5 = myorder(1./Nx,ntau,ee5);  
fid = fopen('adi_flmm_Sub_Eq_I.txt','a');
fprintf('alf = %s,  T = %s\n',num2str(alf),num2str(T));
A=[Nx,1./ntau,ee,odr,ee2,odr2,ee3,odr3,ee4,odr4,ee5,odr5];
s={'N','1/tau','max-L2','order','average-L^2','order','L^2','order','max-L^inf','order','L^inf','order'};
output(s,A);
fprintf('cputime = %-10.4f\n\n',toc);
% % % % % % % % % % % % % % % % % % % % % 
fprintf(fid,'alf = %s, T = %s\n',num2str(alf),num2str(T));
output1(s,A,fid); 
fprintf(fid,'cputime = %-10.4f\n\n',toc);
% END

function [uu,e,e2,e3,abse1,abse2]=adi_fdm21(uv,gv,uav,ubv,ucv,udv,N,N2,tau,nodes,nodes2,nu,T,alf,ut0,bta,q)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    xa2=nodes2(1);  xb2=nodes2(2);
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
h2 = (xb2-xa2)/N2;   yy = xa2:h2:xb2+1e-10;  yy=yy';   yyy=yy;
p = 1;
[mhx,mhy] = meshgrid(xxx,yyy);  mhx = mhx'; mhy = mhy';
% % % % % % 
ne = round(T/tau)+1;    r = (tau/2)^alf;             
w = weights(alf,ne,1);
w2 = w;   w2(2:2:end) = -w2(2:2:end); 
w3 = (tau/2)^alf*w2;    w2 = r*w2;
b = w; 
for k=1:ne
    b(k+1) = b(k)+w(k+1);
end
w = -w;
% 
ww3 = (1/2)^alf*weights(alf,ne,1); ww3(2:2:end)=-ww3(2:2:end);
k=0:ne;
B0 = k.^(alf)/gamma(alf+1);  B=B0;
for k=1:ne
    B(k) = (-B0(1:k)*w(k:-1:1)-sum(ww3(1:k)));
end
B = tau^alf*B;
% % % % % % % % % % % %
% [mm,ss] = basis(p,N,xx);     mm0 = mm;
% [mmy,ssy] = basis(p,N2,yy);  mm0y = mmy;

% q = 4;   %%%%%%%%%%   
[ss1,ss2] = frac_basis(bta,N,q);
mm = eye(N+1)*h;  mm0 = mm;  mm = mm(2:end-1,:); 
ss = -(ss1+ss2)/h^(bta-1);   

[ssy1,ssy2] = frac_basis(bta,N2,q);
mmy = eye(N2+1)*h2;  mm0y = mmy;  mmy=mmy(2:end-1,:); 
ssy = -(ssy1+ssy2)/h2^(bta-1);   

arr = mm + nu(1)*(tau/2)^alf*ss;  
arry = mmy + nu(2)*(tau/2)^alf*ssy; 
mat = (arr);             matyt = (arry');
arr = (arr(:,2:end-1));   arryt =(arry(:,2:end-1))';
mmyt = (mmy)';  ssyt = (ssy)';
%------------- initial ------------
uu = zeros(p*N+1,p*N2+1,ne); 
u0 = ut0(mhx,mhy,alf);   uu(:,:,1) = u0; 
tt = 0:tau:T+1e-10;
ff = zeros(p*N+1,p*N2+1,ne);
for k=1:ne
    ff(:,:,k) = gv(mhx,mhy,tt(k),alf,bta);
end
a0 = 1-alf;  % 
gg2 = fint(ff,tt,alf,a0);   % fractonal integral of the right hand side
gg=zeros(p*N-1,p*N2-1,ne);
for k=1:ne
    gg(:,:,k) = mm*gg2(:,:,k)*mmyt;     %
end
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
nu0 = nu(1)*nu(2)*(tau/2)^(2*alf);
for n = 1:ne-1      
    bndu = uv(mhx,mhy,tt(n+1));  bndu(2:end-1,2:end-1) = 0;
    rhs =  b(n+1)*u0; 
    rhs1 = 0;  
    rhs2 = w(1)*gg(:,:,n+1);
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,:,n+1-k);
        rhs1 = rhs1 + w2(k+1)*uu(:,:,n+1-k);
        rhs2 = rhs2 + w(k+1)*gg(:,:,n+1-k);
    end
    rhs = mm*rhs*mmyt - (nu(2)*mm*rhs1*ssyt+nu(1)*ss*rhs1*mmyt) - mat*bndu*matyt - rhs2.....
        - B(n+1)*(nu(2)*mm*u0*ssyt+nu(1)*ss*u0*mmyt) + nu0*(ss*uu(:,:,n)*ssyt);
    u2 = arr\rhs;  u2 = (arryt\u2')';
    uu(:,:,n+1) = bndu;  uu(2:end-1,2:end-1,n+1) = u2;
end
e3 = funuv(uv,mhx,mhy,tt,alf)-uu;  
abse1 = max(max(max(abs(e3))));
abse2 = max(max(abs(e3(:,:,ne))));
e4 = zeros(1,ne);   e3 = e3.^2;
for k=1:ne
    e4(k) = sum(sum((mm0*e3(:,:,k)*mm0y')));
end
e = max(sqrt((e4))); 
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
%END
function [uu,e,e2,e3,abse1,abse2]=adi_fdm22(uv,gv,uav,ubv,ucv,udv,N,N2,tau,nodes,nodes2,nu,T,alf,ut0,bta,q)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    xa2=nodes2(1);  xb2=nodes2(2);
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
h2 = (xb2-xa2)/N2;   yy = xa2:h2:xb2+1e-10;  yy=yy';   yyy=yy;
p = 1;
[mhx,mhy] = meshgrid(xxx,yyy);  mhx = mhx'; mhy = mhy';
% % % % % % % % % % % %
ne = round(T/tau)+1;  r = tau^alf*alf/2;             
w = weights(alf,ne,1);  
w3 = tau^alf*[1-alf/2,alf/2];
w2 = r;
b = w; 
for k=1:ne
    b(k+1) = b(k) + w(k+1);
end
w = -w;
% % % % % % % % % % % % % % % % % % % % % % % % % %
ww3 = zeros(ne,1); ww3(1:2) = [1-alf/2,alf/2];
k=0:ne;
B0 = k.^(alf)/gamma(alf+1);  B=B0;
for k=1:ne
    B(k) = tau^alf*(-B0(1:k)*w(k:-1:1)-sum(ww3(1:k)));
end
% % % % % % % % % % % % % % % % % %
% q = 2;   %%%%%%%%%%   
[ss1,ss2] = frac_basis(bta,N,q);
mm = eye(N+1)*h;  mm0 = mm;  mm = mm(2:end-1,:); 
ss = -(ss1+ss2)/h^(bta-1);   

[ssy1,ssy2] = frac_basis(bta,N2,q);
mmy = eye(N2+1)*h2;  mm0y = mmy;  mmy=mmy(2:end-1,:); 
ssy = -(ssy1+ssy2)/h2^(bta-1);   

arr = mm + nu(1)*tau^alf*(1-alf/2)*ss;  
arry = mmy + nu(2)*tau^alf*(1-alf/2)*ssy; 
mat = (arr);             matyt = (arry');
arr = (arr(:,2:end-1));   arryt =(arry(:,2:end-1))';
mmyt = (mmy)';  ssyt = (ssy)';
%------------- initial ------------
uu = zeros(p*N+1,p*N2+1,ne); 
u0 = ut0(mhx,mhy,alf);   uu(:,:,1) = u0; 
tt = 0:tau:T+1e-10;
ff = zeros(p*N+1,p*N2+1,ne);
for k=1:ne
    ff(:,:,k) = gv(mhx,mhy,tt(k),alf,bta);
end
a0 = 1-alf;  % 
gg2 = fint(ff,tt,alf,a0);   % fractonal integral of the right hand side
gg=zeros(p*N-1,p*N2-1,ne);
for k=1:ne
    gg(:,:,k) = mm*gg2(:,:,k)*mmyt;     %
end
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
nu0 = nu(1)*nu(2)*(tau^alf*(1-alf/2))^2;
for n = 1:ne-1      
    bndu = uv(mhx,mhy,tt(n+1));  bndu(2:end-1,2:end-1) = 0;
%     ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs =  b(n+1)*u0; 
    rhs1 = w(1)*gg(:,:,n+1);  
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,:,n+1-k);
        rhs1 = rhs1 + w(k+1)*gg(:,:,n+1-k);
    end
    rhs = mm*rhs*mmyt - w2*(nu(2)*mm*uu(:,:,n)*ssyt+nu(1)*ss*uu(:,:,n)*mmyt)....
        - mat*bndu*matyt - B(n+1)*(nu(2)*mm*u0*ssyt+nu(1)*ss*u0*mmyt) - rhs1....
        + nu0*(ss*uu(:,:,n)*ssyt);
    u2 = arr\rhs;  u2 = (arryt\u2')';
    uu(:,:,n+1) = bndu;  uu(2:end-1,2:end-1,n+1) = u2;
end
e3 = funuv(uv,mhx,mhy,tt,alf)-uu;  
abse1 = max(max(max(abs(e3))));
abse2 = max(max(abs(e3(:,:,ne))));
e4 = zeros(1,ne);  
e3 = e3.^2;
for k=1:ne
    e4(k) = sum(sum((mm0*e3(:,:,k)*mm0y')));
end
e = max(sqrt((e4))); 
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
%END

function [uu,e,e2,e3,abse1,abse2]=adi_fdm31(uv,gv,uav,ubv,ucv,udv,N,N2,tau,nodes,nodes2,nu,T,alf,ut0,bta,q)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    xa2=nodes2(1);  xb2=nodes2(2);
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
h2 = (xb2-xa2)/N2;   yy = xa2:h2:xb2+1e-10;  yy=yy';   yyy=yy;
p = 1;
[mhx,mhy] = meshgrid(xxx,yyy);  mhx = mhx'; mhy = mhy';
% % % % % % 
ne = round(T/tau)+1;    r = (tau/2)^alf;             
w = weights(alf,ne,1);
w2 = w;   w2(2:2:end) = -w2(2:2:end); 
w3 = (tau/2)^alf*w2;    w2 = r*w2;
b = w; 
for k=1:ne
    b(k+1) = b(k)+w(k+1);
end
w = -w;
% 
ww3 = (1/2)^alf*weights(alf,ne,1); ww3(2:2:end)=-ww3(2:2:end);
k=0:ne;
B0 = k.^(alf)/gamma(alf+1);  B=B0;
for k=1:ne
    B(k) = (-B0(1:k)*w(k:-1:1)-sum(ww3(1:k)));
end
B = tau^alf*B;
% % % % % % % % % % % %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
om0 = weights(alf,ne,1);
om1 = om0/2^(alf);      om1(2:2:end) = - om1(2:2:end);
om2 = weights(-alf,ne,1).';
om = zeros(ne,1);
for k=1:ne
    om(k) = om2(1:k)*om1(k:-1:1);
end
wn1 = (1:ne).^(1+alf)/gamma(2+alf);
k = 1:ne;
for n=1:ne
    wn1(n) =  wn1(n) - k(1:n)*om(n:-1:1);
end
kk = k.^(alf+1)/gamma(2+alf);
C = B; CC = C;
for n=1:ne
    C(n) = wn1(1:n)*om0(n:-1:1);
    CC(n) = kk(1:n)*om0(n:-1:1) - k(1:n)*om1(n:-1:1);
end
C = tau^alf*CC;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% q = 4;   %%%%%%%%%%   
[ss1,ss2] = frac_basis(bta,N,q);
mm = eye(N+1)*h;  mm0 = mm;  mm = mm(2:end-1,:); 
ss = -(ss1+ss2)/h^(bta-1);   

[ssy1,ssy2] = frac_basis(bta,N2,q);
mmy = eye(N2+1)*h2;  mm0y = mmy;  mmy=mmy(2:end-1,:); 
ssy = -(ssy1+ssy2)/h2^(bta-1);   

arr = mm + nu(1)*(tau/2)^alf*ss;  
arry = mmy + nu(2)*(tau/2)^alf*ssy; 
mat = (arr);             matyt = (arry');
arr = (arr(:,2:end-1));   arryt =(arry(:,2:end-1))';
mmyt = (mmy)';  ssyt = (ssy)';
% % % % % % % % % % % % % % % % % % % % % % % % % % %
%------------- initial ------------
uu = zeros(p*N+1,p*N2+1,ne); 
u0 = ut0(mhx,mhy,alf);   uu(:,:,1) = u0; 
u1 = uv(mhx,mhy,tau,alf);   uu(:,:,2) = u1; 
tt = 0:tau:T+1e-10;
ff = zeros(p*N+1,p*N2+1,ne);
for k=1:ne
    ff(:,:,k) = gv(mhx,mhy,tt(k),alf,bta);
end
a0 = 1-alf;  % 
% gg2 = fint(ff,tt,alf,a0);   % fractonal integral of the right hand side
gg2 = fint3(ff,tt,alf,a0,1);   % fractonal integral of the right hand side
gg=zeros(p*N-1,p*N2-1,ne);
for k=1:ne
    gg(:,:,k) = mm*gg2(:,:,k)*mmyt;     %
end
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
nu0 = nu(1)*nu(2)*(tau/2)^(2*alf); u10 = u1-u0;
for n = 2:ne-1      
    bndu = uv(mhx,mhy,tt(n+1));  bndu(2:end-1,2:end-1) = 0;
    rhs =  b(n+1)*u0; 
    rhs1 = 0;  
    rhs2 = w(1)*gg(:,:,n+1);
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,:,n+1-k);
        rhs1 = rhs1 + w2(k+1)*uu(:,:,n+1-k);
        rhs2 = rhs2 + w(k+1)*gg(:,:,n+1-k);
    end
    rhs = mm*rhs*mmyt - (nu(2)*mm*rhs1*ssyt+nu(1)*ss*rhs1*mmyt) - mat*bndu*matyt - rhs2.....
        - B(n+1)*(nu(2)*mm*u0*ssyt+nu(1)*ss*u0*mmyt) + nu0*(ss*uu(:,:,n)*ssyt).....
        - C(n)*(nu(2)*mm*u10*ssyt+nu(1)*ss*u10*mmyt);
    u2 = arr\rhs;  u2 = (arryt\u2')';
    uu(:,:,n+1) = bndu;  uu(2:end-1,2:end-1,n+1) = u2;
end
e3 = funuv(uv,mhx,mhy,tt,alf)-uu;  
abse1 = max(max(max(abs(e3))));
abse2 = max(max(abs(e3(:,:,ne))));
e4 = zeros(1,ne);  
e3 = e3.^2;
for k=1:ne
    e4(k) = sum(sum((mm0*e3(:,:,k)*mm0y')));
end
e = max(sqrt((e4))); 
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
%END
function [uu,e,e2,e3,abse1,abse2]=adi_fdm32(uv,gv,uav,ubv,ucv,udv,N,N2,tau,nodes,nodes2,nu,T,alf,ut0,bta,q)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    xa2=nodes2(1);  xb2=nodes2(2);
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
h2 = (xb2-xa2)/N2;   yy = xa2:h2:xb2+1e-10;  yy=yy';   yyy=yy;
p = 1;
[mhx,mhy] = meshgrid(xxx,yyy);  mhx = mhx'; mhy = mhy';
% % % % % % % % % % % %
ne = round(T/tau)+1;  r = tau^alf*alf/2;             
w = weights(alf,ne,1);  
w3 = tau^alf*[1-alf/2,alf/2];
w2 = r;
b = w; 
for k=1:ne
    b(k+1) = b(k) + w(k+1);
end
w = -w;
% % % % % % % % % % % % % % % % % % % % % % % % % %
ww3 = zeros(ne,1); ww3(1:2) = [1-alf/2,alf/2];
k=0:ne;
B0 = k.^(alf)/gamma(alf+1);  B=B0;
for k=1:ne
    B(k) = tau^alf*(-B0(1:k)*w(k:-1:1)-sum(ww3(1:k)));
end
% % % % % % % % % % % % % % % % % %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
om0 = weights(alf,ne,1);
om1 = zeros(ne+1,1);    om1(1:2) = [1-alf/2,alf/2];
om2 = weights(-alf,ne,1).';
om = zeros(ne,1);
for k=1:ne
    om(k) = om2(1:k)*om1(k:-1:1);
end
wn1 = (1:ne).^(1+alf)/gamma(2+alf);
k = 1:ne;
for n=1:ne
    wn1(n) =  wn1(n) - k(1:n)*om(n:-1:1);
end
kk = k.^(alf+1)/gamma(2+alf);
C = B; CC = C;
for n=1:ne
    C(n) = wn1(1:n)*om0(n:-1:1);
    CC(n) = kk(1:n)*om0(n:-1:1) - k(1:n)*om1(n:-1:1);
end
C = tau^alf*CC;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% q = 4;   %%%%%%%%%%   
[ss1,ss2] = frac_basis(bta,N,q);
mm = eye(N+1)*h;  mm0 = mm;  mm = mm(2:end-1,:); 
ss = -(ss1+ss2)/h^(bta-1);   

[ssy1,ssy2] = frac_basis(bta,N2,q);
mmy = eye(N2+1)*h2;  mm0y = mmy;  mmy=mmy(2:end-1,:); 
ssy = -(ssy1+ssy2)/h2^(bta-1);   

arr = mm + nu(1)*tau^alf*(1-alf/2)*ss;  
arry = mmy + nu(2)*tau^alf*(1-alf/2)*ssy; 
mat = (arr);             matyt = (arry');
arr = (arr(:,2:end-1));   arryt =(arry(:,2:end-1))';
mmyt = (mmy)';  ssyt = (ssy)';
% % % % % % % % % % % % % % % % % % % % % % % % % % %
%------------- initial ------------
uu = zeros(p*N+1,p*N2+1,ne); 
u0 = ut0(mhx,mhy,alf);      uu(:,:,1) = u0; 
u1 = uv(mhx,mhy,tau,alf);   uu(:,:,2) = u1;
tt = 0:tau:T+1e-10;
ff = zeros(p*N+1,p*N2+1,ne);
for k=1:ne
    ff(:,:,k) = gv(mhx,mhy,tt(k),alf,bta);
end
a0 = 1-alf;  % 
% gg2 = fint(ff,tt,alf,a0);   % fractonal integral of the right hand side
gg2 = fint3(ff,tt,alf,a0,1);
gg=zeros(p*N-1,p*N2-1,ne);
for k=1:ne
    gg(:,:,k) = mm*gg2(:,:,k)*mmyt;     %
end
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
nu0 = nu(1)*nu(2)*(tau^alf*(1-alf/2))^2;  u10 = u1-u0;
% dduu = zeros(p*N-1,p*N2-1,ne);   duu = dduu;
% duu(:,:,1)=mm*uu(:,:,1)*ssyt+ss*uu(:,:,1)*mmyt;
% duu(:,:,2)=mm*uu(:,:,2)*ssyt+ss*uu(:,:,2)*mmyt;
% dduu(:,:,1)=ss*uu(:,:,1)*ssyt;
% dduu(:,:,2)=ss*uu(:,:,2)*ssyt;
duu10 = nu(2)*mm*u10*ssyt+nu(1)*ss*u10*mmyt;
for n = 2:ne-1      
    bndu = uv(mhx,mhy,tt(n+1));  bndu(2:end-1,2:end-1) = 0;
%     ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs =  b(n+1)*u0; 
    rhs1 = w(1)*gg(:,:,n+1);  
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,:,n+1-k);
        rhs1 = rhs1 + w(k+1)*gg(:,:,n+1-k);
    end
    rhs = mm*rhs*mmyt - w2*(nu(2)*mm*uu(:,:,n)*ssyt+nu(1)*ss*uu(:,:,n)*mmyt)....
        - mat*bndu*matyt - B(n+1)*(nu(2)*mm*u0*ssyt+nu(1)*ss*u0*mmyt) - rhs1....
        + nu0*(ss*uu(:,:,n)*ssyt) - C(n)*duu10;
    u2 = arr\rhs;  u2 = (arryt\u2')';
    uu(:,:,n+1) = bndu;  uu(2:end-1,2:end-1,n+1) = u2;
end
e3 = funuv(uv,mhx,mhy,tt,alf)-uu; 
abse1 = max(max(max(abs(e3))));
abse2 = max(max(abs(e3(:,:,ne))));
e4 = zeros(1,ne);  
e3 = e3.^2;
for k=1:ne
    e4(k) = sum(sum((mm0*e3(:,:,k)*mm0y')));
end
e = sqrt(max(e4)); 
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
%END

function [uu,e,e2,e3,abse1,abse2]=adi_L1_fdm(uv,gv,uav,ubv,ucv,udv,N,N2,tau,nodes,nodes2,nu,T,alf,ut0,bta,q)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    xa2=nodes2(1);  xb2=nodes2(2);
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
h2 = (xb2-xa2)/N2;   yy = xa2:h2:xb2+1e-10;  yy=yy';   yyy=yy;
p = 1;
[mhx,mhy] = meshgrid(xxx,yyy);  mhx = mhx'; mhy = mhy';
% % % % % % 
ne = round(T/tau)+1;  
r = tau^alf*gamma(2-alf);     r0 = tau^alf*gamma(2-alf);  r2=nu(1)*nu(2)*r0^2;       
% w = weights(alf,ne,1);  
k = 0:ne; 
b = (k+1).^(1-alf) - k.^(1-alf);
w = b(1:end-1) - b(2:end);
% % % % % % % % % % % % % % % % % %
% [mm,ss] = basis(p,N,xx);     mm0 = mm;
% [mmy,ssy] = basis(p,N2,yy);  mm0y = mmy;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% q = 2;   %%%%%%%%%%   
[ss1,ss2] = frac_basis(bta,N,q);
mm = eye(N+1)*h;  mm0 = mm;  mm = mm(2:end-1,:); 
ss = -(ss1+ss2)/h^(bta-1);   

[ssy1,ssy2] = frac_basis(bta,N2,q);
mmy = eye(N2+1)*h2;  mm0y = mmy;  mmy=mmy(2:end-1,:); 
ssy = -(ssy1+ssy2)/h2^(bta-1);   

% arr = mm + nu(1)*tau^alf*(1-alf/2)*ss;  
% arry = mmy + nu(2)*tau^alf*(1-alf/2)*ssy; 
arr = mm + nu(1)*r*ss;         arry = mmy + nu(2)*r*ssy;
mat = (arr);             matyt = (arry');
arr = (arr(:,2:end-1));   arryt =(arry(:,2:end-1))';
mmyt = (mmy)';  ssyt = (ssy)';
% % % % % % % % % % % % % % % % % % % % % % % % % % %
% 
% mm = eye(N+1)*h; ss = zeros(N+1);
% for i=2:N
%     ss(i,i-1:i+1) = [-1,2,-1];
% end
% ss = ss/h;
% 
% mmy = eye(N2+1)*h2; ssy = zeros(N2+1);
% for i=2:N2
%     ssy(i,i-1:i+1) = [-1,2,-1];
% end
% ssy = ssy/h2;
% mm0 = mm; mm0y = mmy;

% arr = mm + nu(1)*r*ss;         arry = mmy + nu(2)*r*ssy;
% arr = arr(2:end-1,:);    arry = arry(2:end-1,:);  
% mat = sparse(arr);       matyt = sparse(arry');
% 
% arr =sparse(arr(:,2:end-1));     arryt =sparse(arry(:,2:end-1))';
% mm = sparse(mm(2:end-1,:));      ss=sparse(ss(2:end-1,:));
% mmyt = sparse(mmy(2:end-1,:))';  ssyt=sparse(ssy(2:end-1,:))';
%------------- initial ------------
uu = zeros(p*N+1,p*N2+1,ne); 
u0 = ut0(mhx,mhy,alf);      uu(:,:,1) = u0; 
tt = 0:tau:T+1e-10;
% ff = zeros(p*N+1,p*N2+1,ne);
% for k=1:ne
%     ff(:,:,k) = gv(mhx,mhy,tt(k),alf);
% end
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
for n = 1:ne-1      
    bndu = uv(mhx,mhy,tt(n+1));  bndu(2:end-1,2:end-1) = 0;
    rhs =  b(n)*u0;  gg = mm*gv(mhx,mhy,tt(n+1),alf,bta)*mmyt;
    for k=1:n-1
        rhs = rhs + w(k)*uu(:,:,n+1-k);
    end
    rhs = mm*rhs*mmyt - mat*bndu*matyt + r0*gg + r2*(ss*uu(:,:,n)*ssyt);
    u2 = arr\rhs;  u2 = (arryt\u2')';
    uu(:,:,n+1) = bndu;  uu(2:end-1,2:end-1,n+1) = u2;
end

e3 = funuv(uv,mhx,mhy,tt,alf)-uu;  
abse1 = max(max(max(abs(e3))));
abse2 = max(max(abs(e3(:,:,ne))));
e4 = zeros(1,ne);  
e3 = e3.^2;
for k=1:ne
    e4(k) = sum(sum((mm0*e3(:,:,k)*mm0y')));
end
e = sqrt(max(e4)); 
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
%END

function gg2 = fint(ff,tt,alf,a0)
% ne = length(tt);
[rows,cols,ne] = size(ff);      tau = tt(2)-tt(1);
w0 = weights(-alf,ne,1);   ww2 = w0;
ww2(1) = (1-alf/2)*w0(1);  
ww2(2:end) = (1-alf/2)*w0(2:end)+alf/2*w0(1:end-1);
% a0 = 1-alf; 
wj = w0;   j = (0:ne).^a0;
for n=0:ne
    wj(n+1)=gamma(a0+1)/gamma(a0+1+alf)*n^(alf+a0)-j(1:n+1)*ww2(n+1:-1:1);
end
% 
ww2 = (tau)^alf*ww2;    wj = (tau)^alf*wj;
gv0 = ff(:,:,1);  ff2 = ff;
for k=1:ne
    ff2(:,:,k) = ff(:,:,k) - gv0;
end
gg2 = zeros(rows,cols,ne);
for n = 1:ne
    tem = wj(n)*ff2(:,:,2);
    for k=1:n
        tem = tem + ww2(n+1-k)*ff2(:,:,k);
    end
    gg2(:,:,n) = tem + tt(n)^alf/gamma(alf+1)*gv0;
end
%END



function gg2 = fint2(ff,tt,alf,a0)
% ne = length(tt);
[rows,cols,ne] = size(ff);      tau = tt(2)-tt(1);
w0 = weights(-alf,ne,1);   ww2 = w0;
ww2(1) = (1-alf/2)*w0(1);  
ww2(2:end) = (1-alf/2)*w0(2:end)+alf/2*w0(1:end-1);

% ww2 = weights(-alf,ne,2);
% a0 = 1-alf; 
ww2 = (tau)^alf*ww2;    
gv0 = ff(:,:,1);  ff2 = ff;
for k=1:ne
    ff2(:,:,k) = ff(:,:,k) - gv0;
end
gg2 = zeros(rows,cols,ne);
for n = 1:ne
    tem = 0;
    for k=1:n
        tem = tem + ww2(n+1-k)*ff2(:,:,k);
    end
    gg2(:,:,n) = tem + tt(n)^alf/gamma(alf+1)*gv0;
end
%END

function gg2 = fint3(ff,tt,alf,a0,a1)
% ne = length(tt);
[rows,cols,ne] = size(ff);      tau = tt(2)-tt(1);
w0 = weights(-alf,ne,1);   ww2 = w0;
ww2(1) = (1-alf/2)*w0(1);  
ww2(2:end) = (1-alf/2)*w0(2:end)+alf/2*w0(1:end-1);
% a0 = 1-alf; 
wj = w0;   j = (0:ne).^a0;
wj1 = w0;  j1 = (0:ne).^a1;
for n=0:ne
    b1 = gamma(a0+1)/gamma(a0+1+alf)*n^(alf+a0) - j(1:n+1)*ww2(n+1:-1:1);
    b2 = gamma(a1+1)/gamma(a1+1+alf)*n^(alf+a1) - j1(1:n+1)*ww2(n+1:-1:1);
    wj1(n+1) = (b2-b1)/(2^a1-2^a0);
    wj(n+1) = b2 - wj1(n+1)*2^a1;
end
% 
ww2 = (tau)^alf*ww2;    wj = (tau)^alf*wj;  wj1 = (tau)^alf*wj1;
gv0 = ff(:,:,1);  ff2 = ff;
for k=1:ne
    ff2(:,:,k) = ff(:,:,k) - gv0;
end
gg2 = zeros(rows,cols,ne);
for n = 1:ne
    tem = wj(n)*ff2(:,:,2) + wj1(n)*ff2(:,:,3);
    for k=1:n
        tem = tem + ww2(n+1-k)*ff2(:,:,k);
    end
    gg2(:,:,n) = tem + tt(n)^alf/gamma(alf+1)*gv0;
end
%END

function [uv,gv,Nx,Ny,tau,xnodes,ynodes,nu,T,nalf,uav,ubv,ucv,udv,ut0,nbeta]=solution(sel)
% FLMM finite difference methods for D_t^{alf}u = mu1*D^{beta}_{|x|}u + mu2*D^{beta}_{|y|}u + f(x,y,t)
switch sel
    case 0
        xa=0; xb=1; yc=0; yd=1;
        xnodes=[xa,xb];  ynodes=[yc,yd];
        T = 1;  mu1 = 1;  mu2 = 1;  nu = [mu1,mu2];  
        alf = 1; bta = 1.8; % fractional order in time (alf) and space (bta)
        Nx = [8,16,32,64]';  Ny = [8,16,32,64]';  tau = [1e-3,1e-3,1e-3,1e-3]';
%         nalf = [1,1,1,1]*0.2; Nx = [4,8,16,32]';  Ny = [4,8,16,32]'; 
        nalf = [1,1,1,1]*alf;   nbeta = [1 1 1 1]*bta;
%         Nx = [100,100,100,100]'*2;  Ny = [100,100,100,100]'*2;  
%         tau = 1./[8,16,32,64]';
%         nv = 1;  nv2 = 2.5; 
        uv = @(x,y,t,alf) cos(t)*(x-xa).^2.*(xb-x).^2.*(y-yc).^2.*(yd-y).^2;
        gv = @(x,y,t,alf,alf1) -sin(t)*(x-xa).^2.*(xb-x).^2.*(y-yc).^2.*(yd-y).^2 .....
            - nu(1)*cos(t)*(y-yc).^2.*(yd-y).^2.*((xb-xa).^2*gamma(3)/gamma(3-alf1)*((x-xa).^(2-alf1)+(xb-x).^(2-alf1)).....
            - 2*(xb-xa)*gamma(4)/gamma(4-alf1)*((x-xa).^(3-alf1)+(xb-x).^(3-alf1))......
            + gamma(5)/gamma(5-alf1)*((x-xa).^(4-alf1)+(xb-x).^(4-alf1))) ...
            - nu(2)*cos(t)*(x-xa).^2.*(xb-x).^2.*((yd-yc)^2*gamma(3)/gamma(3-alf1)*((y-yc).^(2-alf1)+(yd-y).^(2-alf1)).....
            - 2*(yd-yc)*gamma(4)/gamma(4-alf1)*((y-yc).^(3-alf1)+(yd-y).^(3-alf1))......
            + gamma(5)/gamma(5-alf1)*((y-yc).^(4-alf1)+(yd-y).^(4-alf1)));
        xa=xnodes(1);  xb=xnodes(2);
        yc=ynodes(1);  yd=ynodes(2);
        uav = @(y,t,alf) uv(xa,y,t,alf);
        ubv = @(y,t,alf) uv(xb,y,t,alf);
        ucv = @(x,t,alf) uv(x,yc,t,alf);
        udv = @(x,t,alf) uv(x,yd,t,alf);
        ut0 = @(x,y,alf) uv(x,y,0,alf);
    case 1
        xnodes=[0,1];  ynodes=[0,1];
        T=1;  
        alf = 0.8; bta = 1.8; % fractional order in time (alf) and space (bta)
        nalf = [1,1,1,1]*alf;   nbeta = [1 1 1 1]*bta;
        Nx = [8,16,32,64]';  Ny = [8,16,32,64]';  tau = [1e-3,1e-3,1e-3,1e-3]';
%         Nx = [20,20,20,20,20,20]'*2; Ny=Nx;
%         tau=1./[200,400,500,800,1000,1600];
%         tau = 1/1000;
%         tau=1./[20,40,80,160,320,640]';;
%         N=[10,20,40]';
%         tau=1./N;
        mu1 = 1;   mu2 = 1;  nu = [mu1,mu2];  
%         nu1 = @(x,t) mu1;  nu2 = @(x,t) mu2;
        nv = 1;  nv2 = 2.5;  
        uv = @(x,y,t,alf) (x.^4.*(1-x).^4).*(y.^4.*(1-y).^4)*(t^nv2+t.^nv + 1);
        gv=@(x,y,t,alf,bta) (gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf) + gamma(nv2+1)/gamma(nv2+1-alf)*t^(nv2-alf))*(x.^4.*(1-x).^4.*y.^4.*(1-y).^4)....
            - mu1*(t^nv2+t^nv + 1)*(gamma(9)/gamma(9-bta)*(x.^(8-bta)+(1-x).^(8-bta)) - 4*gamma(8)/gamma(8-bta)*(x.^(7-bta)+(1-x).^(7-bta)).....
            + 6*gamma(7)/gamma(7-bta)*(x.^(6-bta)+(1-x).^(6-bta)) - 4*gamma(6)/gamma(6-bta)*(x.^(5-bta)+(1-x).^(5-bta))....
            + gamma(5)/gamma(5-bta)*(x.^(4-bta)+(1-x).^(4-bta))).*(y.^4.*(1-y).^4)....
            - mu2*(t^nv2+t^nv + 1)*(gamma(9)/gamma(9-bta)*(y.^(8-bta)+(1-y).^(8-bta)) - 4*gamma(8)/gamma(8-bta)*(y.^(7-bta)+(1-y).^(7-bta)).....
            + 6*gamma(7)/gamma(7-bta)*(y.^(6-bta)+(1-y).^(6-bta)) - 4*gamma(6)/gamma(6-bta)*(y.^(5-bta)+(1-y).^(5-bta))....
            + gamma(5)/gamma(5-bta)*(y.^(4-bta)+(1-y).^(4-bta))).*(x.^4.*(1-x).^4);
        
        xa=xnodes(1);  xb=xnodes(2);
        yc=ynodes(1);  yd=ynodes(2);
        uav = @(y,t,alf) uv(xa,y,t,alf);
        ubv = @(y,t,alf) uv(xb,y,t,alf);
        ucv = @(x,t,alf) uv(x,yc,t,alf);
        udv = @(x,t,alf) uv(x,yd,t,alf);
        ut0 = @(x,y,alf) uv(x,y,0,alf);
    case 2
% Ya-nan Zhang, Zhi-zhong Sun, JCP 230 (2011) 8713C8728.  Example 2
        xa=0; xb=1; yc=0; yd=2;
        xnodes=[xa,xb];  ynodes=[yc,yd];
        T = 1;  mu1 = 1;   mu2 = 1;  nu = [mu1,mu2];  
%         Nx = [10,20,40]';  Ny = [10,20,40]';  tau = [1e-3,1e-3,1e-3]';
        nalf = [1,1,1,1]*0.2; Nx = [4,8,16,32]';  Ny = [4,8,16,32]'; 
        nalf = [1,1,1,1];  nbeta = [1 1 1 1]*1.2;
        Nx = [100,100,100,100]'*2;  Ny = [100,100,100,100]'*2;  
        tau = 1./[8,16,32,64]';
        nv = 1;  nv2 = 2.5; 
        uv = @(x,y,t,alf) (t^nv2+t.^nv + 1)*(x-xa).^2.*(xb-x).^2.*(y-yc).^2.*(yd-y).^2;
        gv = @(x,y,t,alf,alf1) (gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf) + gamma(nv2+1)/gamma(nv2+1-alf)*t^(nv2-alf))*(x-xa).^2.*(xb-x).^2.*(y-yc).^2.*(yd-y).^2 .....
            - nu(1)*(t^nv2+t^nv + 1)*(y-yc).^2.*(yd-y).^2.*((xb-xa).^2*gamma(3)/gamma(3-alf1)*((x-xa).^(2-alf1)+(xb-x).^(2-alf1)).....
            - 2*(xb-xa)*gamma(4)/gamma(4-alf1)*((x-xa).^(3-alf1)+(xb-x).^(3-alf1))......
            + gamma(5)/gamma(5-alf1)*((x-xa).^(4-alf1)+(xb-x).^(4-alf1))) ...
            - nu(2)*(t^nv2+t^nv + 1)*(x-xa).^2.*(xb-x).^2.*((yd-yc)^2*gamma(3)/gamma(3-alf1)*((y-yc).^(2-alf1)+(yd-y).^(2-alf1)).....
            - 2*(yd-yc)*gamma(4)/gamma(4-alf1)*((y-yc).^(3-alf1)+(yd-y).^(3-alf1))......
            + gamma(5)/gamma(5-alf1)*((y-yc).^(4-alf1)+(yd-y).^(4-alf1)));
        xa=xnodes(1);  xb=xnodes(2);
        yc=ynodes(1);  yd=ynodes(2);
        uav = @(y,t,alf) uv(xa,y,t,alf);
        ubv = @(y,t,alf) uv(xb,y,t,alf);
        ucv = @(x,t,alf) uv(x,yc,t,alf);
        udv = @(x,t,alf) uv(x,yd,t,alf);
        ut0 = @(x,y,alf) uv(x,y,0,alf);

    otherwise
        fprintf('Please select the solution!')
        return; 
end
%END

function odr=myorder(ntau,NN,err)
n=length(ntau); 
tau(1:n,1)=ntau; N(1:n,1)=NN(1:n); 
er(1:n,1)=err(1:n);
if n==1
    odr=0; return;
end
if n>1
    if abs(ntau(1)-ntau(2))>1e-10
        odr=(log(er(1:end-1)./er(2:end)))./log(tau(1:n-1,:)./tau(2:n,:));
    else 
        odr=(log(er(1:end-1)./er(2:end)))./log(N(1:n-1,:)./N(2:n,:));
    end
end
odr=[0;odr];
%END 

function [w2,w]=weights(alf,n0,p)
n=n0+1;   j = 0:n-1;   x0 = j.'*2*pi/n;
switch p
    case 1
        w = ones(n,1); 
        f=@(z)(1-z).^alf;
        for k=1:n-1
            w(k+1)=(k-1-alf)/k*w(k); 
        end
        w2=w;
        w = real(fft(f(exp(i*x0))))/n;
    case 2
        f=@(z)(3/2 - 2*z + 0.5*z.^2).^alf;
%         fs=(3/2 - 2*t + t^2/2).^alf;
%         d1f=diff(fs,t); d1f=subs(d1f,t,0);
%         d2f=diff(fs,t,2); d2f=subs(d2f,t,0);
        w = real(fft(f(exp(i*x0))))/n;
        u=zeros(n,1);  u(1:3) = [3/2;  -2; 1/2];  
        w2=w; w2(1) = (3/2)^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
%         eee=max(w2-w)
%         eee
    case 3
        f=@(z)(11/6 - 3*z + 3/2*z.^2 - z.^3/3).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [11/6; -3; 3/2;-1/3];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
        eee;
    case 4
        f=@(z)(25/12 - 4*z + 3*z.^2 - 4/3*z.^3 + 1/4*z.^4).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [25/12; -4; 3;-4/3;1/4];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
        eee;
    case 5
        f=@(z)(137/60 - 5*z + 5*z.^2 - 10*z.^3/3 + 5*z.^4/4 - z.^5/5).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [137/60; -5; 5; -10/3; 5/4; -1/5];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
        eee;
    case 6
        f=@(z)(147/60 - 6*z + 15/2*z.^2 - 20/3*z.^3 + 15/4*z.^4 - 6/5*z.^5 +z.^6/6).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [147/60; -6; 15/2; -20/3; 15/4; -6/5; 1/6];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
%         eee
end

function [mm,ss] = basis(e,N,varargin)
switch e
    case 1
        M = [ 2/3, 1/3
              1/3, 2/3];
        S=[  1/2, -1/2
            -1/2,  1/2];
    case 2
        M = [4/15,  2/15, -1/15
             2/15, 16/15,  2/15
            -1/15,  2/15,  4/15];
        S = [ 7/6, -4/3,  1/6
             -4/3,  8/3, -4/3
              1/6, -4/3,  7/6];
    case 3
        M = [16/105,  33/280,   -3/70, 19/840
             33/280,   27/35, -27/280,  -3/70
              -3/70, -27/280,   27/35, 33/280
             19/840,   -3/70,  33/280, 16/105];
        
        S=[  37/20, -189/80,   27/40,  -13/80
           -189/80,    27/5, -297/80,   27/40
             27/40, -297/80,    27/5, -189/80
            -13/80,   27/40, -189/80,   37/20];
    case 4
    otherwise
        mm=[]; ss=[]; return;
end
e1 = e+1;
% mm = sparse(zeros(e*N+1));  
mm = sparse((e*N+1));  
ss = mm; 
no = numel(varargin);
if no<0.5
    h = 1;
else
    nodes = varargin{1};
    h = nodes(2:end) - nodes(1:end-1); h = h/2;
end
if length(h)<1.5
    h = ones(1,N)*h;
end
for k=0:N-1
    mm(e*k+1:e*k+e1,e*k+1:e*k+e1) = M*h(k+1);
    ss(e*k+1:e*k+e1,e*k+1:e*k+e1) = S/h(k+1);
end
for k=1:N-1
    mm(e*k+1:e*k+1,e*k+1:e*k+1) = M(1,1)*h(k) + M(e1,e1)*h(k+1);
    ss(e*k+1:e*k+1,e*k+1:e*k+1) = S(1,1)/h(k) + S(e1,e1)/h(k+1);
end
%END

function [arr1,arr2] = frac_basis(alf,N,sel)
arr1 = zeros(N-1, N+1); arr2 = arr1;
w  = weights(alf,N+2,1)';
switch sel
    case 1
        for i=1:N-1
            arr1(i,1:i+2) = w(i+2:-1:1);  arr2(i,i:N+1) = w(1:N+2-i);
        end
    case 2
        arr1 = w_arr_or2(alf,N);  arr1 = arr1(2:end-1,:);  arr2 = arr1;
        arr1 = arr1*cos(alf*pi/2); arr2 = arr2*cos(alf*pi/2); 
    case 3
        ww = w; ww(1) = alf/2*w(1); 
        ww(2:end) = alf/2*w(2:end) + (2-alf)/2*w(1:end-1);
        for i=1:N-1
            arr1(i,1:i+2) = ww(i+2:-1:1);  arr2(i,i:N+1) = ww(1:N+2-i);
        end
    case 4
        ww = w; ww(1) = (2+alf)/4*w(1);  ww(2) = (2+alf)/4*w(2);
        ww(3:end) = (2+alf)/4*w(3:end) + (2-alf)/4*w(1:end-2);
        for i=1:N-1
            arr1(i,1:i+2) = ww(i+2:-1:1);  arr2(i,i:N+1) = ww(1:N+2-i);
        end
    case 5
        for i=1:N-1
            arr1(i,1:i+1) = w(i+1:-1:1);  arr2(i,i+1:N+1) = w(1:N+1-i);
        end
    otherwise
        arr1 = []; arr2 = [];
end
function w = w_arr_or2(alf,N)
% second order matrix
w0 = weight_order2(N+1,alf);   w = zeros(N+1);        
for i=1:N+1
    w(i,1:i) = w0(i:-1:1);  w(i,i+1:N+1) = w0(2:N+2-i);   
end
w;
% END
function w = weight_order2(ne,alf)
w = zeros(ne+1,1);
w(1) = gamma(alf+1)/gamma(alf/2+1)^2;
for k=0:ne
    w(k+2) = (k-alf/2)/(alf/2+k+1)*w(k+1);
end


function output(s,A)
[row,col] = size(A);
fprintf('%-10s%-10s%-16s%-8s%-16s%-8s%-16s%-8s%-16s%-8s%-16s%-8s',s{1},s{2},s{3},s{4},s{5},s{6},s{7},s{8},s{9},s{10},s{11},s{12});
fprintf('\n');
for k=1:row
    fprintf('%-10s%-10s%-16.4e%-8.4f%-16.4e%-8.4f%-16.4e%-8.4f%-16.4e%-8.4f%-16.4e%-8.4f',......
        num2str(A(k,1)),num2str(A(k,2)),A(k,3),A(k,4),A(k,5),A(k,6),A(k,7),A(k,8),A(k,9),A(k,10),A(k,11),A(k,12));
    fprintf('\n');
end
% fprintf('\n');
% END

function output1(s,A,fid)
[row,col] = size(A);
fprintf(fid,'%-10s%-10s%-16s%-8s%-16s%-8s%-16s%-8s%-16s%-8s%-16s%-8s',s{1},s{2},s{3},s{4},s{5},s{6},s{7},s{8},s{9},s{10},s{11},s{12});
fprintf(fid,'\n');
for k=1:row
    fprintf(fid,'%-10s%-10s%-16.4e%-8.4f%-16.4e%-8.4f%-16.4e%-8.4f%-16.4e%-8.4f%-16.4e%-8.4f',......
        num2str(A(k,1)),num2str(A(k,2)),A(k,3),A(k,4),A(k,5),A(k,6),A(k,7),A(k,8),A(k,9),A(k,10),A(k,11),A(k,12));
    fprintf(fid,'\n');
end
fprintf(fid,'\n');
% END
function u=funuv(uv,x,y,t,alf)
nt=length(t); N=length(x); N2=length(y);
u=zeros(N,N2,nt);
for k=1:nt
    u(:,:,k)=uv(x,y,t(k),alf);
end

function [r,w]=RootsJacobiLobatto(alf,bet,N)
[r0,w0] = RootsJacobi(alf+1,bet+1,N-2);
r = [-1;r0;1];
w(1,1)=2^(alf+bet+1)*gamma(bet+2)*gamma(alf+1)/gamma(alf+bet+2);
w(N,1)=2^(alf+bet+1)*gamma(bet+1)*gamma(alf+2)/gamma(alf+bet+2);
w(1,1)=w(1,1)/(N-1);  w(N,1)=w(N,1)/(N-1);
for k=1:N-1
    w(1,1)=w(1,1)*k/(k+1+alf+bet)*(alf+k)/(bet+k);
    w(N,1)=w(N,1)*k/(k+1+alf+bet)*(bet+k)/(alf+k);
end
w(2:N-1,1)=w0./(1+r0)./(1-r0);
%END
%--------- By eigenvalue ---------
function y=RootsofJacobi(alf,bet,N)
j=0:N;
aj=(bet^2-alf^2)./((2*j+alf+bet).*(2*j+alf+bet+2));
j=1:N;
bj=4*j./(2*j+alf+bet-1).*(j+alf)./(2*j+alf+bet).....
    .*(j+bet)./(2*j+alf+bet).*(j+alf+bet)./(2*j+alf+bet+1);
aj(1)=(bet-alf)/(alf+bet+2);
A=diag(aj)+diag(sqrt(bj),1)+diag(sqrt(bj),-1);
y=eig(A);
%END
%========== 
function [y,w]=RootsJacobiPoly(alf,bet,N)
y=zeros(N,1);
dt=1/N^2;
a0=(N+alf+bet+1)/2; vps = 1e-15;
xa=-1; fa=jacobi(alf,bet,N,xa);
for k=1:N
    xb = xa+dt;
    fb = jacobi(alf,bet,N,xb);
    for j=1:10000
        if fa*fb<0
            break;
        else
            xa = xb;
            xb = xb + dt;
            fb = jacobi(alf,bet,N,xb);
        end
    end
%-------------- Newton Method ---------------
    xc=(xa+xb)/2;  % initial value
    for j=1:100
        xd = xc - jacobi(alf,bet,N,xc)/jacobi(alf+1,bet+1,N-1,xc)/a0;
        if abs(xd-xc)<vps
            xc=xd;
            break;
        else
            xc=xd;
        end
    end
%---------------------------------------------
    y(k)=xc;
    xa=xc+dt;  fa=jacobi(alf,bet,N,xa);
end
% [y0,w0]=RootsJacobiPoly(alf,bet,N);

N = N-1;
GN = 2^(alf+bet)*(2*N+alf+bet+2)*gamma(alf+1)*gamma(bet+1)/gamma(alf+bet+2);
for k=1:N
    GN=GN*(alf+k)/(k+1+alf+bet)*(bet+k)/(k+1);
end
t1=jacobi(alf,bet,N,y);
t2=djacobi(alf,bet,N+1,1,y);
w=GN./(t1.*t2);
w;
%END

function [y,w]=RootsJacobi(alf,bet,N)
dt=1/N^2/10;
a0=(N+alf+bet+1)/2; vps = 1e-15;
xx=-1:dt:1;  xx(end+1)=1;
yy=jacobi(alf,bet,N,xx);
yy1=yy>0; 
str=num2str(yy1);
str(find(isspace(str)))=[] ;
id1=findstr(str, '01');  id2=findstr(str, '10');
xc1=(xx(id1)+xx(id1+1))/2;
xc2=(xx(id2)+xx(id2+1))/2;
xc=[xc1,xc2]; xc=sort(xc).';
%-------------- Newton Method ---------------
for j=1:100
    xd = xc - jacobi(alf,bet,N,xc)./jacobi(alf+1,bet+1,N-1,xc)/a0;
    err=norm(xd-xc,inf);
    if err<vps
        xc=xd;
        break;
    else
        xc=xd;
    end
end
y=sort(xc);
%------------------------------------------
N = N-1;
GN = 2^(alf+bet)*(2*N+alf+bet+2)*gamma(alf+1)*gamma(bet+1)/gamma(alf+bet+2);
for k=1:N
    GN=GN*(alf+k)/(k+1+alf+bet)*(bet+k)/(k+1);
end
t1=jacobi(alf,bet,N,y); t2=djacobi(alf,bet,N+1,1,y);
w=GN./(t1.*t2);
%END
function y = jacobi(alf,bet,N,x)
if N==0
    y=ones(length(x),1); return;
end
if N==1
    y=((alf+bet+2)*x + alf-bet)/2;
    return;
end
y0 = 1;
y1 = ((alf+bet+2)*x + alf-bet)/2;
n = 1:N;
a = (2*n+alf+bet+1)./(2*n+2).*((2*n+alf+bet+2)./(n+alf+bet+1));
b = (bet^2-alf^2)*(2*n+alf+bet+1)./(2*n+2)./(n+alf+bet+1)./(2*n+alf+bet);
c = (n+alf)./(n+1).*((n+bet)./(n+alf+bet+1))......
    .*((2*n+alf+bet+2)./(2*n+alf+bet));
for n=1:N-1
    y = (a(n)*x-b(n)).*y1 - c(n)*y0;
    y0 = y1; y1 = y;
end
%END
function dy=djacobi(alf,bet,N,m,x)
dnk=1;
for k=1:m
    dnk=dnk*(N+k+alf+bet)/2;
end
% dnk0=gamma(N+m+alf+bet+1)/gamma(N+alf+bet+1)/2^m;
dy=0;
if m<N
    dy=jacobi(alf+m,bet+m,N-m,x)*dnk;
end
dy;
%END
function y = fracgamma(a,b,n)
k=0:n-1;
g = (k+a)./ (k+b);
y = gamma(a)/gamma(b)*prod(g);
