function L1FEM
% u_t = D^{1-alf}_{0,t}u_xx + f
% Two methods are used: FDM (4.67) and FEM (5.96)
% Table 5.1: meth = 1, sel = 1
% Table 5.2: meth = 1, sel = 1
% Table 5.3: meth = 2 for method (4.67) and meth = 1 for method (5.96)
tic
sel = 1; 
p = 3;  % order of FEM
nalf = [0.25 0.5 0.75];   % fractional order
% nalf = [0.4 0.5 0.6 0.8 0.9 1];
meth = 2;
for kk=1:length(nalf)
    alf = nalf(kk);  
    [uv,gv,N,tau,nodes,nu,T,alpha,uav,ubv,ut0] = solution(sel,alf);
    nn=length(tau); ee=zeros(nn,1); ee2=ee;
    for k=1:nn
        switch meth
            case 1
                [uu,e,e2]=fem_L1(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,p);
            case 2
                [uu,e,e2]=cn_zhang(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0);
        end
        ee(k) = e; ee2(k) = e2;
    end
odr = myorder(1./N,tau,ee);  odr2 = myorder(1./N,tau,ee2);  
fid = fopen('fem_L1.txt','a');
fprintf('alf = %s,  T = %s\n',num2str(alf),num2str(T));
A=[N,1./tau,ee,odr,ee2,odr2];
s={'N','1/tau','max-error','order','L^2-error','order'};
output(s,A);
fprintf('cputime = %-10.4f\n\n',toc);
% % % % % % % % % % % % % % % % % % % % % 
fprintf(fid,'alf = %s, T = %s\n',num2str(alf),num2str(T));
output1(s,A,fid); 
fprintf(fid,'cputime = %-10.4f\n\n',toc);
end
% END


function [uu,e,e2]=cn_zhang(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf0,ut0)
% % uniform grids
% zhang yanan, sun zhizhong
xa = nodes(1);         xb = nodes(2);    
h = (xb-xa)/N;         xx = xa:h:xb+1e-10;  xx=xx';
alf = 1-alf0;
% % % % % % 
ne = round(T/tau);   n = 0:ne;
b = (n+1).^(1-alf) - n.^(1-alf);
r = nu*tau^(1-alf)/gamma(2-alf)/h^2;
% % % % % % % % % % % % % % % % % % % % % % %  
ss = sparse(zeros(N-1,N+1));
for k=1:N-1
    ss(k,k) = 1;  ss(k,k+1) = -2;  ss(k,k+2) = 1;
end
mm = sparse(eye(N-1)); mm = [sparse(zeros(N-1,1)),mm,sparse(zeros(N-1,1))];
arr = mm - r/2*ss;      arrR= mm + r/2*ss;
bnd = [arr(:,1),arr(:,end)]; 
arr = arr(:,2:end-1);    
arr = sparse(arr); ss=sparse(ss);
%------------- initial ------------
uu = zeros(N+1,ne); 
u0 = ut0(xx);    uu(:,1) = u0;  
u1 = uv(xx,tau); uu(:,2) = u1;   
tt = 0:tau:T+tau/10;  
R = r*(1-alf)*(n.^(-alf)+(n+1).^(-alf))/2;
d2u0 = ss*u0;
ff = zeros(N-1,ne);
for k=1:ne
    ff(:,k)=(gv(xx(2:end-1),tt(k))+gv(xx(2:end-1),tt(k+1)))/2;
end
ff=tau*ff;
%--------- first step --------
arr0 = mm - r*ss;      arrR0 = mm - r*alf*ss;
bnd0 = [arr0(:,1),arr0(:,end)]; 
arr0 = arr0(:,2:end-1);  
ua = uav(tt(2));   ub = ubv(tt(2));
rhs = -bnd0*[ua;ub] + arrR0*uu(:,1) + tau*gv(xx(2:end-1),tt(2));
uu(:,2) = [ua;arr0\rhs;ub];
%-----------------------------
for n = 2:ne      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs = -bnd*[ua;ub] + arrR*uu(:,n) + R(n)*d2u0 + ff(:,n); 
    rhs1 =  2*b(n)*u0; 
    for k=1:n-1
        rhs1 = rhs1 + (b(n-k)-b(n-k+1))*(uu(:,k+1)+uu(:,k));
    end
% % % 
    rhs = rhs - r/2*(ss*rhs1) ;
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
e = norm(uu(:,end)-uv(xx,T),inf);
e2 = h*(uu(:,end)-uv(xx,T)).^2;  e2 = sqrt(sum(e2));
e3 = funuv(uv,xx,tt);   e = max(max(abs(uu-e3)));
e2 = norm(sqrt(sum(h*(e3-uu).^2)),inf);
e;
%END

function [uu,e,e2]=fem_L1(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf0,ut0,p)
% % uniform grids
% FEM based on the modified L1 method
xa = nodes(1);   xb=nodes(2);    
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
switch p
    case 2
        xxm = (xx(1:end-1)+xx(2:end))/2;
        xxx = zeros(2*N+1,1);  xxx(1:2:end)=xx; xxx(2:2:end)=xxm;
    case 3
        x1 = xx + h/3;         x2 = xx+2*h/3;
        xxx = zeros(3*N+1,1);
        xxx(1:p:end)=xx; xxx(2:p:end)=x1(1:end-1); 
        xxx(p:p:end)=x2(1:end-1);
end
alf = 1-alf0;
% % % % % % 
ne = round(T/tau);
n = 0:ne;
B = 2*((n+1/2).^(1-alf)-n.^(1-alf));   B0 = B(1);
A = B - (1-alf)*(n+1/2).^(-alf);       A0 = A(1);
b = (n+1).^(1-alf) - n.^(1-alf);
B = b - B;
r = nu*tau^(1-alf)/gamma(2-alf);
% % % % % % % % % % % % % % % % % % % % % % %
[mm,ss] = basis(p,N,xx);
mm = sparse(mm(2:end-1,:));  ss=sparse(ss(2:end-1,:));
arr = mm + r/2*ss;           arrR= mm - r/2*ss;
% arr = arr(2:end-1,:); arrR = arrR(2:end-1,:);
bnd = [arr(:,1),arr(:,end)]; 
arr = arr(:,2:end-1);        
% arr = sparse(arr);
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);    uu(:,1) = u0;  
u1 = uv(xx,tau); uu(:,2) = u1;
ff = zeros(p*N+1,ne);
tt = 0:tau:T+1e-10; ttt = (tt(1:end-1)+tt(2:end))/2;
for k=1:ne
    ff(:,k)=gv(xx,ttt(k));
end
ff = tau*ff;
% % % % % % First step:  t = tau   % % % % % % % % % % 
% A00=2*(1/2)^(1-alf)-(1-alf)*(1/2)^(-alf); 
% B00=2*(1/2)^(1-alf); 
arr0 = mm + B0*r/2*ss;    arrR0 = mm - B0*r/2*ss;
bnd0 = [arr0(:,1),arr0(:,end)]; 
arr0 = arr0(:,2:end-1);
ua = uav(tt(2));   ub = ubv(tt(2));
rhs = -bnd0*[ua;ub] + mm*ff(:,1) + arrR0*u0 + r*A0*(ss*u0);
u1 = [ua;arr0\rhs;ub];
uu(:,2) = u1;
% % % % % % % % % % % % % % % % % % % % % % % % % % % 
% u1=uv(xx,tau);  uu(:,2) = u1;
% u1=0;  uu(:,2) = 0;
% 
%-----------------------------
for n = 2:ne      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs = -bnd*[ua;ub] + arrR*uu(:,n) + mm*ff(:,n); 
    rhs1 =  B(n)*(u0+u1) + 2*A(n)*u0; 
    for k=1:n-1
        rhs1 = rhs1 + (b(n-k)-b(n-k+1))*(uu(:,k+1)+uu(:,k));
    end
% % % 
    rhs = rhs + r/2*(ss*rhs1);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
e = norm(uu(:,end)-uv(xx,T),inf);
e2 = h/p*(uu(:,end)-uv(xx,T)).^2;
e2 = sqrt(sum(e2));
e3 = funuv(uv,xx,tt);   e = max(max(abs(uu-e3)));
e2 = norm(sqrt(sum(h/p*(e3-uu).^2)),inf);
e;
%END


function [uv,gv,N,tau,nodes,nu,T,alpha,uav,ubv,ut0]=solution(sel,alf0)
switch sel
case 1
        % Zhang Ya-nan and Sun, SIAM, 2011:  Example 1
        nu = 1;          nodes = [0,1];   
        xa = nodes(1);   xb = nodes(2);     
        alpha = alf0;  T = 1;  
        alf = 1-alf0;
        N = [16 32 64 128 256]';  tau=1./[16 32 64 128 256]';
        N = [2000 2000 2000 2000 2000]'/2; tau=1./[16 32 64 128 256]';
%         N = [2000 2000 2000 2000 2000]'/2; tau=1./[100 200 400 800 1600]';
%         N = [4 8,16,32,64]'; tau = 1./[10000 10000,10000,10000,10000]';
        nv = 2;   
        uv = @(x,t) exp(x)*(t^nv + t + 1);
        gv = @(x,t)(nv*t^(nv-1) + 1)*exp(x)....
            - nu*(gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf)....
            + 1/gamma(2-alf)*t^(1-alf)+ 1/gamma(1-alf)*t^(-alf))*exp(x);
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
    otherwise
        uv=[]; gv=[];  N=[]; tau=[]; 
        nodes=[]; nu=[]; T=[]; alpha=[]; 
end
% nu=[nu,nu2];
%END

function odr=myorder(ntau,NN,err)
n=length(ntau); 
tau(1:n,1)=ntau; N(1:n,1)=NN(1:n); 
er(1:n,1)=err(1:n);
if n==1
    odr=0; return;
end
if n>1
    if abs(ntau(1)-ntau(2))>1e-10
        odr=(log(er(1:end-1)./er(2:end)))./log(tau(1:n-1,:)./tau(2:n,:));
    else 
        odr=(log(er(1:end-1)./er(2:end)))./log(N(1:n-1,:)./N(2:n,:));
    end
end
odr=[0;odr];
%END 

function [mm,ss] = basis(e,N,varargin)
switch e
    case 1
        M = [ 2/3, 1/3
              1/3, 2/3];
        S=[  1/2, -1/2
            -1/2,  1/2];
    case 2
        M = [4/15,  2/15, -1/15
             2/15, 16/15,  2/15
            -1/15,  2/15,  4/15];
        S = [ 7/6, -4/3,  1/6
             -4/3,  8/3, -4/3
              1/6, -4/3,  7/6];
    case 3
        M = [16/105,  33/280,   -3/70, 19/840
             33/280,   27/35, -27/280,  -3/70
              -3/70, -27/280,   27/35, 33/280
             19/840,   -3/70,  33/280, 16/105];
        
        S=[  37/20, -189/80,   27/40,  -13/80
           -189/80,    27/5, -297/80,   27/40
             27/40, -297/80,    27/5, -189/80
            -13/80,   27/40, -189/80,   37/20];
    case 4
    otherwise
        mm=[]; ss=[]; return;
end
e1 = e+1;
% mm = sparse(zeros(e*N+1));  
mm = sparse((e*N+1));  
ss = mm; 
no = numel(varargin);
if no<0.5
    h = 1;
else
    nodes = varargin{1};
    h = nodes(2:end) - nodes(1:end-1); h = h/2;
end
if length(h)<1.5
    h = ones(1,N)*h;
end
for k=0:N-1
    mm(e*k+1:e*k+e1,e*k+1:e*k+e1) = M*h(k+1);
    ss(e*k+1:e*k+e1,e*k+1:e*k+e1) = S/h(k+1);
end
for k=1:N-1
    mm(e*k+1:e*k+1,e*k+1:e*k+1) = M(1,1)*h(k) + M(e1,e1)*h(k+1);
    ss(e*k+1:e*k+1,e*k+1:e*k+1) = S(1,1)/h(k) + S(e1,e1)/h(k+1);
end
%END
function output(s,A)
[row,col] = size(A);
fprintf('%-10s%-10s%-16s%-8s%-16s%-8s',s{1},s{2},s{3},s{4},s{5},s{6});
fprintf('\n');
for k=1:row
    fprintf('%-10s%-10s%-16.4e%-8.4f%-16.4e %-8.4f',......
        num2str(A(k,1)),num2str(A(k,2)),A(k,3),A(k,4),A(k,5),A(k,6));
    fprintf('\n');
end
% fprintf('\n');
% END

function output1(s,A,fid)
[row,col] = size(A);
fprintf(fid,'%-10s%-10s%-16s%-8s%-16s%-8s',s{1},s{2},s{3},s{4},s{5},s{6});
fprintf(fid,'\n');
for k=1:row
    fprintf(fid,'%-10s%-10s%-16.6e%-8.4f%-16.6e %-8.4f',.....
        num2str(A(k,1)),num2str(A(k,2)),A(k,3),A(k,4),A(k,5),A(k,6));
    fprintf(fid,'\n');
end
fprintf(fid,'\n');
% END
function u=funuv(uv,x,t)
nt=length(t); N=length(x);
u=zeros(N,nt);
for k=1:nt
    u(:,k)=uv(x,t(k));
end

