function flmm_fem
%  FEM for time-fractional subdiffusion equation
% D_{0,t}^alf u(x,t) = nu*u_xx + f(x,t)
tic
fem = 6; %
sel = 0; %
p = 3; % p = 1,2,3 
% nalf = [0.4  0.6  0.8]; 
nalf = [0.1  0.5  0.9];  % fractional order


for kk=1:length(nalf)
    alf = nalf(kk);
    [uv,gv,N,tau,nodes,nu,T,alpha,uav,ubv,ut0,a1,a2,a3] = solution(sel,alf);
    nn = length(tau); ee = zeros(nn,1); ee2 = ee;  ee3 = ee;
    for k=1:nn
        switch fem
            case 1
                [uu,e,e2,e3]=pro_fem1(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,p);
            case 2
                [uu,e,e2,e3]=pro_fem2(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,p);
            case 3 % FEM (5.117)
                [uu,e,e2,e3]=pro_fem3(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,p);
            case 4 % FEM (5.118)
                [uu,e,e2,e3]=pro_fem4(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,p);
            case 5 % FEM (5.119)
                [uu,e,e2,e3]=pro_fem5(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,p,a1,a2,a3);
            case 6 % FEM (5.120)
                [uu,e,e2,e3]=pro_fem6(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,p,a1,a2,a3);
            case 7 % FEM (5.105)
                [uu,e,e2,e3]=L1_fem(uv,gv,uav,ubv,N(k),tau(k),nodes,nu,T,alf,ut0,p);
        end
        ee(k) = e; ee2(k) = e2; ee3(k) = e3;
    end

% % % % % % % % % % % % % % % % % % % % % % % % 
odr = myorder(tau,1./N,ee);  odr2 = myorder(tau,1./N,ee2);  
odr3 = myorder(tau,1./N,ee3);
filename = ['flmm_fem_',num2str(fem),'.txt'];
fid = fopen(filename,'a');
fprintf('alf = %s,  T = %s\n',num2str(alf),num2str(T));
A=[N,1./tau,ee,odr,ee2,odr2,ee3,odr3];
s={'N','1/tau','max-2error','order','AV-L^2 error','order','L^2-error','order'};
output(s,A);
fprintf('cputime = %-10.4f\n\n',toc);
% % % % % % % % % % % % % % % % % % % % % 
fprintf(fid,'alf = %s, T = %s\n',num2str(alf),num2str(T));
output1(s,A,fid); 
fprintf(fid,'cputime = %-10.4f\n\n',toc);
end
% END


function [uu,e,e2,e3]=pro_fem1(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf,ut0,p)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
switch p
    case 2
        xxm = (xx(1:end-1)+xx(2:end))/2;
        xxx = zeros(2*N+1,1);  xxx(1:2:end)=xx; xxx(2:2:end)=xxm;
    case 3
        x1 = xx + h/3;         x2 = xx+2*h/3;
        xxx = zeros(3*N+1,1);
        xxx(1:p:end)=xx; xxx(2:p:end)=x1(1:end-1); 
        xxx(p:p:end)=x2(1:end-1);
end
% % % % % % 
ne = round(T/tau)+1;    r = nu*(tau/2)^alf;             
w = weights(alf,ne,1);
w2 = w;  
w2(2:2:end) = -w2(2:2:end); 
w3 = (tau/2)^alf*w2;    w2 = r*w2;
b = w; 
for k=1:ne
    b(k+1)=b(k)+w(k+1);
end
w = -w;
% % % % % % % % % % % %
[mm,ss] = basis(p,N,xx);  mm0=mm;
arr = mm + nu*(tau/2)^alf*ss;
arr = arr(2:end-1,:);  
bnd = [arr(:,1),arr(:,end)]; 
arr =sparse( arr(:,2:end-1)); 
mm = sparse(mm(2:end-1,:)); ss=sparse(ss(2:end-1,:));
%------------- initial ------------
xx=xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);   uu(:,1) = u0; 
ff = zeros(p*N+1,ne);
tt = 0:tau:T+1e-10;
for k=1:ne
    ff(:,k)=gv(xx,tt(k));
end
%-----------------------------
for n = 1:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs = w3(1)*ff(:,n+1) + b(n+1)*u0; 
    rhs1 = 0; 
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,n+1-k) + w3(k+1)*ff(:,n+1-k);
        rhs1 = rhs1 + w2(k+1)*uu(:,n+1-k);
    end
    rhs = mm*rhs - ss*rhs1 - bnd*[ua;ub];
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
e3 = funuv(uv,xx,tt)-uu;  e = max(max(abs(e3(:,ne))));  
e4 = zeros(1,ne);  
for k=1:ne
    e4(k) = (e3(:,k)'*mm0*e3(:,k));
end
e = max(sqrt((e4)));   
e3 = (sqrt(e4(ne))); 
e2 = sqrt(tau*sum(e4));
%END

function [uu,e,e2,e3]=pro_fem2(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf,ut0,p)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
switch p
    case 2
        xxm = (xx(1:end-1)+xx(2:end))/2;
        xxx = zeros(2*N+1,1);  xxx(1:2:end)=xx; xxx(2:2:end)=xxm;
    case 3
        x1 = xx + h/3;         x2 = xx+2*h/3;
        xxx = zeros(3*N+1,1);
        xxx(1:p:end)=xx; xxx(2:p:end)=x1(1:end-1); 
        xxx(p:p:end)=x2(1:end-1);
end
% % % % % % 
ne = round(T/tau)+1;  r = nu*tau^alf*alf/2;             
w = weights(alf,ne,1);
w3 = tau^alf*[1-alf/2,alf/2];
w2 = r;
b = w; 
for k=1:ne
    b(k+1) = b(k) + w(k+1);
end
w = -w;
% % % % % % % % % % % % % % % % % %
[mm,ss] = basis(p,N,xx); mm0=mm;
arr = mm + nu*tau^alf*(1-alf/2)*ss;
arr = arr(2:end-1,:);
bnd = [arr(:,1),arr(:,end)]; 
arr = arr(:,2:end-1);       arr = sparse(arr);
mm = sparse(mm(2:end-1,:)); ss=sparse(ss(2:end-1,:));
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);   uu(:,1) = u0;  
ff = zeros(p*N+1,ne);
tt = 0:tau:T+1e-10;
for k=1:ne
    ff(:,k)=gv(xx,tt(k));
end
%-----------------------------
for n = 1:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs = w3(1)*ff(:,n+1) + w3(2)*ff(:,n) + b(n+1)*u0; 
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,n+1-k);
    end
    rhs = mm*rhs - w2*(ss*uu(:,n)) - bnd*[ua;ub];
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
e3 = funuv(uv,xx,tt)-uu;   e = max(max(abs(e3(:,ne))));
e4 = zeros(1,ne);  
for k=1:ne
    e4(k) = e3(:,k)'*mm0*e3(:,k);
end
e = max(sqrt((e4)));   
e3 = sqrt(e4(ne)); 
e2 = sqrt(tau*sum(e4));
%END

function [uu,e,e2,e3]=pro_fem3(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf,ut0,p)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
switch p
    case 2
        xxm = (xx(1:end-1)+xx(2:end))/2;
        xxx = zeros(2*N+1,1);  xxx(1:2:end)=xx; xxx(2:2:end)=xxm;
    case 3
        x1 = xx + h/3;         x2 = xx+2*h/3;
        xxx = zeros(3*N+1,1);
        xxx(1:p:end)=xx; xxx(2:p:end)=x1(1:end-1); 
        xxx(p:p:end)=x2(1:end-1);
end
% % % % % % 
ne = round(T/tau)+1;    r = nu*(tau/2)^alf;             
w = weights(alf,ne,1);
w2 = w;   w2(2:2:end) = -w2(2:2:end); 
w3 = (tau/2)^alf*w2;    w2 = r*w2;
b = w; 
for k=1:ne
    b(k+1) = b(k)+w(k+1);
end
w = -w;
% 
ww3 = (1/2)^alf*weights(alf,ne,1); ww3(2:2:end)=-ww3(2:2:end);
k=0:ne-1;
B0 = k.^(alf)/gamma(alf+1);  B = B0;
for k=1:ne
    B(k) = (-B0(1:k)*w(k:-1:1)-sum(ww3(1:k)));
end
B = nu*tau^alf*B;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
om0 = weights(alf,ne,1);
om1 = om0/2^(alf);      om1(2:2:end) = - om1(2:2:end);
om2 = weights(-alf,ne,1).';
om = zeros(ne,1);
for k=1:ne
    om(k) = om2(1:k)*om1(k:-1:1);
end
wn1 = (1:ne).^(1+alf)/gamma(2+alf);
k = 1:ne;
for n=1:ne
    wn1(n) =  wn1(n) - k(1:n)*om(n:-1:1);
end
kk = k.^(alf+1)/gamma(2+alf);
C = B; CC = C;
for n=1:ne
    C(n) = wn1(1:n)*om0(n:-1:1);
    CC(n) = kk(1:n)*om0(n:-1:1) - k(1:n)*om1(n:-1:1);
end
C = nu*tau^alf*CC*0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[mm,ss] = basis(p,N,xx);  mm0 = mm;
arr = mm + nu*(tau/2)^alf*ss; arr = arr(2:end-1,:);
arr0 = mm + nu*(tau/2)^alf*ss + C(1)*ss;  arr0 = arr0(2:end-1,:);
bnd = [arr(:,1),arr(:,end)];    bnd0 = [arr0(:,1),arr0(:,end)]; 
arr = sparse(arr(:,2:end-1));   arr0 = sparse( arr0(:,2:end-1)); 
mm = sparse(mm(2:end-1,:)); ss = sparse(ss(2:end-1,:));
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);   uu(:,1) = u0; 
tt = 0:tau:T+1e-10;
ff = zeros(p*N+1,ne);
for k=1:ne
    ff(:,k) = gv(xx,tt(k));
end
a0 = 1-alf;  % 
% gg2 = fint(ff,tt,alf,a0);   % fractonal integral of the right hand side
gg2 = fint2(ff,tt,alf,a0,1);   % fractonal integral of the right hand side
gg = mm*gg2;    

% nv = 2 + alf;
% gint = @(x,t)(t^(nv) + t)*sin(x).....
%     + nu*(gamma(nv+1)/gamma(nv+1+alf)*t^(nv+alf) ....
%     + 1/gamma(2+alf)*t^(1+alf) + 2/gamma(1+alf)*t^(alf))*sin(x);
% for k=1:ne
%     gg2(:,k) = gint(xx,tt(k));
% end
% gg = mm*gg2;
% % % % % % % % % % % First step % % % % % % % % % % % % 
n = 1;      
ua = uav(tt(n+1));   ub = ubv(tt(n+1));
rhs =  b(n+1)*u0;
rhs1 = 0;
rhs2 = w(1)*gg(:,n+1);
for k=1:n
    rhs = rhs + w(k+1)*uu(:,n+1-k);
    rhs1 = rhs1 + w2(k+1)*uu(:,n+1-k);
    rhs2 = rhs2 + w(k+1)*gg(:,n+1-k);
end
rhs = mm*rhs - ss*rhs1 - bnd0*[ua;ub] - rhs2 - B(n+1)*(ss*u0) + C(n)*(ss*u0);
u2 = arr0\rhs;
uu(:,n+1) = [ua;u2;ub];  
u1 = [ua;u2;ub];     u10 = u1-u0;
% % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
for n = 2:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs =  b(n+1)*u0; 
    rhs1 = 0;  
    rhs2 = w(1)*gg(:,n+1);
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,n+1-k);
        rhs1 = rhs1 + w2(k+1)*uu(:,n+1-k);
        rhs2 = rhs2 + w(k+1)*gg(:,n+1-k);
    end
    rhs = mm*rhs - ss*rhs1 - bnd*[ua;ub] - rhs2....
        - B(n+1)*(ss*u0) - C(n)*(ss*u10);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
% e = norm(uu(:,end)-uv(xx,T),inf);
% % e2 = h/3*(uu(:,end)-uv(xx,T)).^2;  e2 = sqrt(sum(e2));
% e2 = uu(:,end)-uv(xx,T); e2=sqrt(e2'*mm0*e2);
% % e3 = funuv(uv,xx,tt);    e = max(max(abs(uu(:,3:end)-e3(:,3:end))));
% % e = norm(uu(:,end)-uv(xx,T),inf);
% % e2 = (uu(:,end)-uv(xx,T)); 
% % e2 = sqrt(sum(h/p*e2.^2));
% e2 = sqrt(e2'*mm0*e2);
e3 = funuv(uv,xx,tt)-uu;  e = max(max(abs(e3)));  
e4 = zeros(1,ne);  
for k=1:ne
    e4(k) = (e3(:,k)'*mm0*e3(:,k));
end
e = max(sqrt(e4));    
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
%END

function [uu,e,e2,e3]=pro_fem4(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf,ut0,p)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
switch p
    case 2
        xxm = (xx(1:end-1)+xx(2:end))/2;
        xxx = zeros(2*N+1,1);  xxx(1:2:end)=xx; xxx(2:2:end)=xxm;
    case 3
        x1 = xx + h/3;         x2 = xx+2*h/3;
        xxx = zeros(3*N+1,1);
        xxx(1:p:end)=xx; xxx(2:p:end)=x1(1:end-1); 
        xxx(p:p:end)=x2(1:end-1);
end
% % % % % % 
ne = round(T/tau)+1;  r = nu*tau^alf*alf/2;             
w = weights(alf,ne,1);  
w3 = tau^alf*[1-alf/2,alf/2];
w2 = r;
b = w; 
for k=1:ne
    b(k+1) = b(k) + w(k+1);
end
w = -w;
% % % % % % % % % % % % % % % % % % % % % % % % % %
ww3 = zeros(ne,1); ww3(1:2) = [1-alf/2,alf/2];
k=0:ne;
B0 = k.^(alf)/gamma(alf+1);  B=B0;
for k=1:ne
    B(k) = nu*tau^alf*(-B0(1:k)*w(k:-1:1)-sum(ww3(1:k)));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
om0 = weights(alf,ne,1);
om1 = zeros(ne+1,1);    om1(1:2) = [1-alf/2,alf/2];
om2 = weights(-alf,ne,1).';
om = zeros(ne,1);
for k=1:ne
    om(k) = om2(1:k)*om1(k:-1:1);
end
wn1 = (1:ne).^(1+alf)/gamma(2+alf);
k = 1:ne;
for n=1:ne
    wn1(n) =  wn1(n) - k(1:n)*om(n:-1:1);
end
kk = k.^(alf+1)/gamma(2+alf);
C = B; CC = C;
for n=1:ne
    C(n) = wn1(1:n)*om0(n:-1:1);
    CC(n) = kk(1:n)*om0(n:-1:1) - k(1:n)*om1(n:-1:1);
end
C = nu*tau^alf*CC*0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % % % % % % % % % % % % % % % % %
[mm,ss] = basis(p,N,xx); mm0 = mm;
arr = mm + nu*tau^alf*(1-alf/2)*ss;
arr0 = mm + nu*tau^alf*(1-alf/2)*ss + C(1)*ss;  
arr = arr(2:end-1,:);          arr0 = sparse(arr0(2:end-1,:));
bnd = [arr(:,1),arr(:,end)];   bnd0 = [arr0(:,1),arr0(:,end)];
arr = sparse(arr(:,2:end-1));  arr0 = sparse(arr0(:,2:end-1)); 
mm = sparse(mm(2:end-1,:)); ss=sparse(ss(2:end-1,:));
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);   uu(:,1) = u0;  
ff = zeros(p*N+1,ne);   tt = 0:tau:T+1e-10;
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
for k=1:ne
    ff(:,k) = gv(xx,tt(k));
end
a0 = 1-alf;
% gg2 = fint(ff,tt,alf,a0);
gg2 = fint2(ff,tt,alf,a0,1);   % fractonal integral of the right hand side
gg = mm*gg2;
% % % % % % % % % % % First step % % % % % % % % % % % % 
n = 1;      
ua = uav(tt(n+1));   ub = ubv(tt(n+1));
rhs =  b(n+1)*u0;
rhs1 = w(1)*gg(:,n+1);
for k=1:n
    rhs = rhs + w(k+1)*uu(:,n+1-k);
    rhs1 = rhs1 + w(k+1)*gg(:,n+1-k);
end
rhs = mm*rhs - w2*(ss*uu(:,n)) - bnd0*[ua;ub] - B(n+1)*(ss*u0)....
    - rhs1 + C(n)*(ss*u0);
u2 = arr0\rhs;
uu(:,n+1) = [ua;u2;ub];  
u1 = [ua;u2;ub];     u10 = u1-u0;
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
for n = 2:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
%     rhs = w3(1)*ff(:,n+1) + w3(2)*ff(:,n) + b(n+1)*u0; 
    rhs =  b(n+1)*u0; 
    rhs1 = w(1)*gg(:,n+1);
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,n+1-k);
        rhs1 = rhs1 + w(k+1)*gg(:,n+1-k);
    end
    rhs = mm*rhs - w2*(ss*uu(:,n)) - bnd*[ua;ub] - B(n+1)*(ss*u0).....
        - rhs1 - C(n)*(ss*u10);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
% % e = norm(uu(:,end)-uv(xx,T),inf);
% % e2 = (uu(:,end)-uv(xx,T)); 
% % e2 = sqrt(sum(h/p*e2.^2));

e3 = funuv(uv,xx,tt)-uu;   e = max(max(abs(e3))); 
e4 = zeros(1,ne);  
for k=1:ne
    e4(k) = (e3(:,k)'*mm0*e3(:,k));
end
e = max(sqrt(e4)); 
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
% % e2 = sqrt(e2'*mm0*e2);
% % e3 = funuv(uv,xx,tt);    e = max(max(abs(uu(:,3:end)-e3(:,3:end))));
% % eee = max(abs(uu-e3));
% % eee;
% %END
function [uu,e,e2,e3]=pro_fem5(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf,ut0,p,a1,a2,a3)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
switch p
    case 2
        xxm = (xx(1:end-1)+xx(2:end))/2;
        xxx = zeros(2*N+1,1);  xxx(1:2:end)=xx; xxx(2:2:end)=xxm;
    case 3
        x1 = xx + h/3;         x2 = xx+2*h/3;
        xxx = zeros(3*N+1,1);
        xxx(1:p:end)=xx; xxx(2:p:end)=x1(1:end-1); 
        xxx(p:p:end)=x2(1:end-1);
end
% % % % % % 
ne = round(T/tau)+1;    r = nu*(tau/2)^alf;             
w = weights(alf,ne,1);
w2 = w;   w2(2:2:end) = -w2(2:2:end); 
w3 = (tau/2)^alf*w2;    w2 = r*w2;
b = w; 
for k=1:ne
    b(k+1) = b(k)+w(k+1);
end
w = -w;
% 
ww3 = (1/2)^alf*weights(alf,ne,1); ww3(2:2:end)=-ww3(2:2:end);
k=0:ne-1;
B0 = k.^(alf)/gamma(alf+1);  B = B0;
for k=1:ne
    B(k) = (-B0(1:k)*w(k:-1:1)-sum(ww3(1:k)));
end
B = nu*tau^alf*B;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
om0 = weights(alf,ne,1);
om1 = om0/2^(alf);      om1(2:2:end) = - om1(2:2:end);
c0 = B;      
nv = a1;    %% The method is exact for u(t) = t^nv
k = gamma(nv+1)/gamma(nv+1+alf)*(1:ne).^(alf+nv);
kk = (1:ne).^nv;
for n = 1:ne
    c0(n) = k(1:n)*om0(n:-1:1) - kk(1:n)*om1(n:-1:1);
end
C = nu*tau^alf*c0;  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[mm,ss] = basis(p,N,xx);  mm0 = mm;
arr = mm + nu*(tau/2)^alf*ss; arr = arr(2:end-1,:);
arr0 = mm + nu*(tau/2)^alf*ss + C(1)*ss;  arr0 = arr0(2:end-1,:);
bnd = [arr(:,1),arr(:,end)];    bnd0 = [arr0(:,1),arr0(:,end)]; 
arr = sparse(arr(:,2:end-1));   arr0 = sparse( arr0(:,2:end-1)); 
mm = sparse(mm(2:end-1,:)); ss = sparse(ss(2:end-1,:));
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);   uu(:,1) = u0; 
tt = 0:tau:T+1e-10;
ff = zeros(p*N+1,ne);
for k=1:ne
    ff(:,k) = gv(xx,tt(k));
end
% a0 = 1-alf; a0=alf; a1=1; % 
% gg2 = fint(ff,tt,alf,a0);   % fractonal integral of the right hand side
gg2 = fint2(ff,tt,alf,a2,a3);   % fractonal integral of the right hand side
gg = mm*gg2;    

% nv = 2 + alf;
% gint = @(x,t)(t^(nv) + t)*sin(x).....
%     + nu*(gamma(nv+1)/gamma(nv+1+alf)*t^(nv+alf) ....
%     + 1/gamma(2+alf)*t^(1+alf) + 2/gamma(1+alf)*t^(alf))*sin(x);
% for k=1:ne
%     gg2(:,k) = gint(xx,tt(k));
% end
% gg = mm*gg2;
% % % % % % % % % % % First step % % % % % % % % % % % % 
n = 1;      
ua = uav(tt(n+1));   ub = ubv(tt(n+1));
rhs =  b(n+1)*u0;
rhs1 = 0;
rhs2 = w(1)*gg(:,n+1);
for k=1:n
    rhs = rhs + w(k+1)*uu(:,n+1-k);
    rhs1 = rhs1 + w2(k+1)*uu(:,n+1-k);
    rhs2 = rhs2 + w(k+1)*gg(:,n+1-k);
end
rhs = mm*rhs - ss*rhs1 - bnd0*[ua;ub] - rhs2 - B(n+1)*(ss*u0) + C(n)*(ss*u0);
u2 = arr0\rhs;
uu(:,n+1) = [ua;u2;ub];  
u1 = [ua;u2;ub];     u10 = u1-u0;
% % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
for n = 2:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
    rhs =  b(n+1)*u0; 
    rhs1 = 0;  
    rhs2 = w(1)*gg(:,n+1);
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,n+1-k);
        rhs1 = rhs1 + w2(k+1)*uu(:,n+1-k);
        rhs2 = rhs2 + w(k+1)*gg(:,n+1-k);
    end
    rhs = mm*rhs - ss*rhs1 - bnd*[ua;ub] - rhs2....
        - B(n+1)*(ss*u0) - C(n)*(ss*u10);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
% e = norm(uu(:,end)-uv(xx,T),inf);
% % e2 = h/3*(uu(:,end)-uv(xx,T)).^2;  e2 = sqrt(sum(e2));
% e2 = uu(:,end)-uv(xx,T); e2=sqrt(e2'*mm0*e2);
% % e3 = funuv(uv,xx,tt);    e = max(max(abs(uu(:,3:end)-e3(:,3:end))));
% % e = norm(uu(:,end)-uv(xx,T),inf);
% % e2 = (uu(:,end)-uv(xx,T)); 
% % e2 = sqrt(sum(h/p*e2.^2));
% e2 = sqrt(e2'*mm0*e2);
e3 = funuv(uv,xx,tt)-uu;   e = max(max(abs(e3(:,ne))));
e4 = zeros(1,ne);  
for k=1:ne
    e4(k) = (e3(:,k)'*mm0*e3(:,k));
end
e = max(sqrt(e4));    
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
%END

function [uu,e,e2,e3]=pro_fem6(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf,ut0,p,a1,a2,a3)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
switch p
    case 2
        xxm = (xx(1:end-1)+xx(2:end))/2;
        xxx = zeros(2*N+1,1);  xxx(1:2:end)=xx; xxx(2:2:end)=xxm;
    case 3
        x1 = xx + h/3;         x2 = xx+2*h/3;
        xxx = zeros(3*N+1,1);
        xxx(1:p:end)=xx; xxx(2:p:end)=x1(1:end-1); 
        xxx(p:p:end)=x2(1:end-1);
end
% % % % % % 
ne = round(T/tau)+1;  r = nu*tau^alf*alf/2;             
w = weights(alf,ne,1);  
w3 = tau^alf*[1-alf/2,alf/2];
w2 = r;
b = w; 
for k=1:ne
    b(k+1) = b(k) + w(k+1);
end
w = -w;
% % % % % % % % % % % % % % % % % % % % % % % % % %
ww3 = zeros(ne,1); ww3(1:2) = [1-alf/2,alf/2];
k=0:ne;
B0 = k.^(alf)/gamma(alf+1);  B=B0;
for k=1:ne
    B(k) = nu*tau^alf*(-B0(1:k)*w(k:-1:1)-sum(ww3(1:k)));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
om0 = weights(alf,ne,1);
% c0 = B;  
nv = a1;  %% The method is exact for u(t) = t^nv
k = gamma(nv+1)/gamma(nv+1+alf)*(1:ne).^(alf+nv);
% for n = 1:ne
%     c0(n) = k(1:n)*om0(n:-1:1) - (ww3(1)*n^nv + ww3(2)*(n-1)^nv);
% end
k1=(1:ne).^nv;
c0 = fast_conv(k(1:ne),om0(1:ne)) - fast_conv(ww3(1:ne),k1(1:ne));
C = nu*tau^alf*c0;
% n0 = log2(ne); n0 = ceil(n0); n0 = 2^n0;
% k0 = zeros(1,n0);  k1 = k0; k2 = k0;  k3 = k0;
% k0(1:ne) = gamma(nv+1)/gamma(nv+1+alf)*(1:ne).^(alf+nv);
% k1(1:ne) = om0(1:ne);  
% k2(1:ne) = (0:ne-1).^nv;
% k3(1:2) = ww3(1:2);
% 
% fftk0 = fft(k0);  fftk1 = fft(k1);
% k0k1 = ifft(fftk0.*fftk1);
% 
% fftk2 = fft(k2);  fftk3 = fft(k3);
% k2k3 = ifft(fftk2.*fftk3);
% cc2 = nu*tau^alf*(k0k1-k2k3);
% cc = nu*tau^alf*(conv(k0,k1)-conv(k2,k3));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % % % % % % % % % % % % % % % % %
[mm,ss] = basis(p,N,xx); mm0 = mm;
arr = mm + nu*tau^alf*(1-alf/2)*ss;
arr0 = mm + nu*tau^alf*(1-alf/2)*ss + C(1)*ss;  
arr = arr(2:end-1,:);          arr0 = sparse(arr0(2:end-1,:));
bnd = [arr(:,1),arr(:,end)];   bnd0 = [arr0(:,1),arr0(:,end)];
arr = sparse(arr(:,2:end-1));  arr0 = sparse(arr0(:,2:end-1)); 
mm = sparse(mm(2:end-1,:)); ss=sparse(ss(2:end-1,:));
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);   uu(:,1) = u0;  
ff = zeros(p*N+1,ne);   tt = 0:tau:T+1e-10;
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
for k=1:ne
    ff(:,k) = gv(xx,tt(k));
end
% a0 = 1-alf;   a0 = alf;
% gg2 = fint(ff,tt,alf,a0);
gg2 = fint2(ff,tt,alf,a2,a3);   % fractonal integral of the right hand side
gg = mm*gg2;
% % % % % % % % % % % First step % % % % % % % % % % % % 
n = 1;      
ua = uav(tt(n+1));   ub = ubv(tt(n+1));
rhs =  b(n+1)*u0;
rhs1 = w(1)*gg(:,n+1);
for k=1:n
    rhs = rhs + w(k+1)*uu(:,n+1-k);
    rhs1 = rhs1 + w(k+1)*gg(:,n+1-k);
end
rhs = mm*rhs - w2*(ss*uu(:,n)) - bnd0*[ua;ub] - B(n+1)*(ss*u0)....
    - rhs1 + C(n)*(ss*u0);
u2 = arr0\rhs;
uu(:,n+1) = [ua;u2;ub];  
u1 = [ua;u2;ub];     u10 = u1-u0;
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
for n = 2:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
%     rhs = w3(1)*ff(:,n+1) + w3(2)*ff(:,n) + b(n+1)*u0; 
    rhs =  b(n+1)*u0; 
    rhs1 = w(1)*gg(:,n+1);
    for k=1:n
        rhs = rhs + w(k+1)*uu(:,n+1-k);
        rhs1 = rhs1 + w(k+1)*gg(:,n+1-k);
    end
    rhs = mm*rhs - w2*(ss*uu(:,n)) - bnd*[ua;ub] - B(n+1)*(ss*u0).....
        - rhs1 - C(n)*(ss*u10);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
% % e = norm(uu(:,end)-uv(xx,T),inf);
% % e2 = (uu(:,end)-uv(xx,T)); 
% % e2 = sqrt(sum(h/p*e2.^2));

e3 = funuv(uv,xx,tt)-uu;    e = max(max(abs(e3(:,ne))));
e4 = zeros(1,ne);  
for k=1:ne
    e4(k) = (e3(:,k)'*mm0*e3(:,k));
end
e = max(sqrt(e4)); 
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
% % e2 = sqrt(e2'*mm0*e2);
% % e3 = funuv(uv,xx,tt);    e = max(max(abs(uu(:,3:end)-e3(:,3:end))));
% % eee = max(abs(uu-e3));
% % eee;
% %END

function [uu,e,e2,e3]=L1_fem(uv,gv,uav,ubv,N,tau,nodes,nu,T,alf,ut0,p)
% % uniform grids
xa = nodes(1);   xb=nodes(2);    
h = (xb-xa)/N;   xx = xa:h:xb+1e-10;  xx=xx';   xxx=xx;
switch p
    case 2
        xxm = (xx(1:end-1)+xx(2:end))/2;
        xxx = zeros(2*N+1,1);  xxx(1:2:end)=xx; xxx(2:2:end)=xxm;
    case 3
        x1 = xx + h/3;         x2 = xx+2*h/3;
        xxx = zeros(3*N+1,1);
        xxx(1:p:end)=xx; xxx(2:p:end)=x1(1:end-1); 
        xxx(p:p:end)=x2(1:end-1);
end
% % % % % % 
ne = round(T/tau)+1;  
r = nu*tau^alf*gamma(2-alf);     r0 = tau^alf*gamma(2-alf);        
% w = weights(alf,ne,1);  
k = 0:ne; 
b = (k+1).^(1-alf) - k.^(1-alf);
w = b(1:end-1) - b(2:end);

% w2 = r;
% b = w; 
% for k=1:ne
%     b(k+1) = b(k) + w(k+1);
% end
% w = -w;
% % % % % % % % % % % % % % % % % % % % % % % % % %
% ww3 = zeros(ne,1); ww3(1:2) = [1-alf/2,alf/2];
% k=0:ne;
% B0 = k.^(alf)/gamma(alf+1);  B=B0;
% for k=1:ne
%     B(k) = nu*tau^alf*(-B0(1:k)*w(k:-1:1)-sum(ww3(1:k)));
% end
% % % % % % % % % % % % % % % % % %
[mm,ss] = basis(p,N,xx);  mm0 = mm;
arr = mm + r*ss;
arr = arr(2:end-1,:);
bnd = [arr(:,1),arr(:,end)]; 
arr = arr(:,2:end-1);       arr = sparse(arr);
mm = sparse(mm(2:end-1,:)); ss=sparse(ss(2:end-1,:));
%------------- initial ------------
xx = xxx;
uu = zeros(p*N+1,ne); 
u0 = ut0(xx);   uu(:,1) = u0;  
ff = zeros(p*N+1,ne);   tt = 0:tau:T+1e-10;
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
for k=1:ne
    ff(:,k) = gv(xx,tt(k));
end
% a0 = 1-alf;
% gg2 = fint(ff,tt,alf,a0);
gg = r0*(mm*ff);
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%-----------------------------
for n = 1:ne-1      
    ua = uav(tt(n+1));   ub = ubv(tt(n+1)); 
%     rhs = w3(1)*ff(:,n+1) + w3(2)*ff(:,n) + b(n+1)*u0; 
    rhs =  b(n)*u0; 
    for k=1:n-1
        rhs = rhs + w(k)*uu(:,n+1-k);
    end
    rhs = mm*rhs  - bnd*[ua;ub]  + gg(:,n+1);
    u2 = arr\rhs; 
    uu(:,n+1) = [ua;u2;ub];   
end
% 
e3 = funuv(uv,xx,tt)-uu;   e = max(max(abs(e3))); 
e4 = zeros(1,ne);  
for k=1:ne
    e4(k) = (e3(:,k)'*mm0*e3(:,k));
end
e = sqrt(max(max(abs(e4)))); 
e2 = sqrt(tau*sum(e4));
e3 = sqrt(e4(ne));
%END

function [uv,gv,N,tau,nodes,nu,T,alpha,uav,ubv,ut0,a1,a2,a3]=solution(sel,alf)
switch sel
    case 0
        nu = 1;          nodes = [0,1];
        xa = nodes(1);   xb = nodes(2);
%         alf = 0.9;       
        alpha = alf;  T = 1;  
        N = [16,32,64,128,256]'*2;  
        tau = 1./N;
        N = [1000,1000,1000,1000,1000]';
        nv = 2 + alf;  mu = 1;  c = 1;  d = 2;
        a1 = mu; a2 = 1-alf; a3 = 1;
        p = 2*pi;
        uv = @(x,t) sin(p*x)*(t^nv + c*t^mu + d);
        gv = @(x,t)(gamma(nv+1)/gamma(nv+1-alf)*t^(nv-alf).....
                 + c*gamma(mu+1)/gamma(mu+1-alf)*t^(mu-alf))*sin(p*x).....
                 + p^2*nu*(t^nv + c*t^mu + d)*sin(p*x);
        uav = @(t) uv(xa,t);
        ubv = @(t) uv(xb,t);
        ut0 = @(x) uv(x,0);
    otherwise
        uv=[]; gv=[];  N=[]; tau=[]; 
        nodes=[]; nu=[]; T=[]; alpha=[]; 
end
%END

function y = func(alf,x,t)
y=0;
t0 = pi^2*t^alf;
for n=50:-1:1
    y = y + (1-(-1)^n)/n^3*mlf(alf,1,-n^2*t0,12)*sin(n*pi*x);
end
y=16/pi^3*y;



function odr=myorder(ntau,NN,err)
n=length(ntau); 
tau(1:n,1)=ntau; N(1:n,1)=NN(1:n); 
er(1:n,1)=err(1:n);
if n==1
    odr=0; return;
end
if n>1
    if abs(ntau(1)-ntau(2))>1e-10
        odr=(log(er(1:end-1)./er(2:end)))./log(tau(1:n-1,:)./tau(2:n,:));
    else 
        odr=(log(er(1:end-1)./er(2:end)))./log(N(1:n-1,:)./N(2:n,:));
    end
end
odr=[0;odr];
%END 

function [w2,w]=weights(alf,n0,p)
n=n0+1;   j = 0:n-1;   x0 = j.'*2*pi/n;
switch p
    case 1
        w = ones(n,1); 
        f=@(z)(1-z).^alf;
        for k=1:n-1
            w(k+1)=(k-1-alf)/k*w(k); 
        end
        w2=w;
        w = real(fft(f(exp(i*x0))))/n;
    case 2
        f=@(z)(3/2 - 2*z + 0.5*z.^2).^alf;
%         fs=(3/2 - 2*t + t^2/2).^alf;
%         d1f=diff(fs,t); d1f=subs(d1f,t,0);
%         d2f=diff(fs,t,2); d2f=subs(d2f,t,0);
        w = real(fft(f(exp(i*x0))))/n;
        u=zeros(n,1);  u(1:3) = [3/2;  -2; 1/2];  
        w2=w; w2(1) = (3/2)^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
%         eee=max(w2-w)
%         eee
    case 3
        f=@(z)(11/6 - 3*z + 3/2*z.^2 - z.^3/3).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [11/6; -3; 3/2;-1/3];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
        eee;
    case 4
        f=@(z)(25/12 - 4*z + 3*z.^2 - 4/3*z.^3 + 1/4*z.^4).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [25/12; -4; 3;-4/3;1/4];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
        eee;
    case 5
        f=@(z)(137/60 - 5*z + 5*z.^2 - 10*z.^3/3 + 5*z.^4/4 - z.^5/5).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [137/60; -5; 5; -10/3; 5/4; -1/5];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
        eee;
    case 6
        f=@(z)(147/60 - 6*z + 15/2*z.^2 - 20/3*z.^3 + 15/4*z.^4 - 6/5*z.^5 +z.^6/6).^alf;
        w=real(fft(f(exp(i*x0))))/n;
        
        u=zeros(n,1);  u(1:p+1) = [147/60; -6; 15/2; -20/3; 15/4; -6/5; 1/6];  
        w2=w;  w2(1) = (u(1))^alf;
        ii=0:n-1;  u=sparse(u);
        for j=1:n-1
            w2(j+1) = (alf - ii(1:j)/j*(alf+1))*(w2(1:j).*u(j+1:-1:2))/u(1);
        end
        eee=max(w2-w);
%         eee
end

function [mm,ss] = basis(e,N,varargin)
switch e
    case 1
        M = [ 2/3, 1/3
              1/3, 2/3];
        S=[  1/2, -1/2
            -1/2,  1/2];
    case 2
        M = [4/15,  2/15, -1/15
             2/15, 16/15,  2/15
            -1/15,  2/15,  4/15];
        S = [ 7/6, -4/3,  1/6
             -4/3,  8/3, -4/3
              1/6, -4/3,  7/6];
    case 3
        M = [16/105,  33/280,   -3/70, 19/840
             33/280,   27/35, -27/280,  -3/70
              -3/70, -27/280,   27/35, 33/280
             19/840,   -3/70,  33/280, 16/105];
        
        S=[  37/20, -189/80,   27/40,  -13/80
           -189/80,    27/5, -297/80,   27/40
             27/40, -297/80,    27/5, -189/80
            -13/80,   27/40, -189/80,   37/20];
    case 4
    otherwise
        mm=[]; ss=[]; return;
end
e1 = e+1;
% mm = sparse(zeros(e*N+1));  
mm = sparse((e*N+1));  
ss = mm; 
no = numel(varargin);
if no<0.5
    h = 1;
else
    nodes = varargin{1};
    h = nodes(2:end) - nodes(1:end-1); h = h/2;
end
if length(h)<1.5
    h = ones(1,N)*h;
end
for k=0:N-1
    mm(e*k+1:e*k+e1,e*k+1:e*k+e1) = M*h(k+1);
    ss(e*k+1:e*k+e1,e*k+1:e*k+e1) = S/h(k+1);
end
for k=1:N-1
    mm(e*k+1:e*k+1,e*k+1:e*k+1) = M(1,1)*h(k) + M(e1,e1)*h(k+1);
    ss(e*k+1:e*k+1,e*k+1:e*k+1) = S(1,1)/h(k) + S(e1,e1)/h(k+1);
end
%END

function output(s,A)
[row,col] = size(A);
fprintf('%-10s%-10s%-16s%-8s%-16s%-8s%-16s%-8s',s{1},s{2},s{3},s{4},s{5},s{6},s{7},s{8});
fprintf('\n');
for k=1:row
    fprintf('%-10s%-10s%-16.4e%-8.4f%-16.4e%-8.4f%-16.4e%-8.4f',.....
        num2str(A(k,1)),num2str(A(k,2)),A(k,3),A(k,4),A(k,5),A(k,6),A(k,7),A(k,8));
    fprintf('\n');
end
fprintf('\n');
% END

function output1(s,A,fid)
[row,col] = size(A);
fprintf(fid,'%-10s%-10s%-16s%-8s%-16s%-8s%-16s%-8s',s{1},s{2},s{3},s{4},s{5},s{6},s{7},s{8});
fprintf(fid,'\n');
for k=1:row
    fprintf(fid,'%-10s%-10s%-16.4e%-8.4f%-16.4e%-8.4f%-16.4e%-8.4f',.....
        num2str(A(k,1)),num2str(A(k,2)),A(k,3),A(k,4),A(k,5),A(k,6),A(k,7),A(k,8));
    fprintf(fid,'\n');
end
fprintf(fid,'\n');
% END
function u=funuv(uv,x,t)
nt=length(t); N=length(x);
u=zeros(N,nt);
for k=1:nt
    u(:,k)=uv(x,t(k));
end

function gg2 = fint2(ff,tt,alf,a0,a1)
% ne = length(tt);
[rows,ne] = size(ff);      tau = tt(2)-tt(1);
w0 = weights(-alf,ne,1);   ww2 = w0;
ww2(1) = (1-alf/2)*w0(1);  
ww2(2:end) = (1-alf/2)*w0(2:end)+alf/2*w0(1:end-1);
% a0 = 1-alf; 
wj = w0;   j = (0:ne).^a0;
wj1 = w0;  j1 = (0:ne).^a1;
for n=0:ne
    b1 = gamma(a0+1)/gamma(a0+1+alf)*n^(alf+a0) - j(1:n+1)*ww2(n+1:-1:1);
    b2 = gamma(a1+1)/gamma(a1+1+alf)*n^(alf+a1) - j1(1:n+1)*ww2(n+1:-1:1);
    wj1(n+1) = (b2-b1)/(2^a1-2^a0);
    wj(n+1) = b2 - wj1(n+1)*2^a1;
end
% 
ww2 = (tau)^alf*ww2;    wj = (tau)^alf*wj;  wj1 = (tau)^alf*wj1;
gv0 = ff(:,1);  ff2 = ff;
for k=1:ne
    ff2(:,k) = ff(:,k) - gv0;
end
gg2 = zeros(rows,ne);
for n = 1:ne
    tem = wj(n)*ff2(:,2) + wj1(n)*ff2(:,3);
    for k=1:n
        tem = tem + ww2(n+1-k)*ff2(:,k);
    end
    gg2(:,n) = tem + tt(n)^alf/gamma(alf+1)*gv0;
end
%END


function y = fast_conv(u,v)
n1 = length(u); n2 = length(v);
n = n1+n2-1;    nm = max(n1,n2);
u1 = zeros(1,n); v1 = u1;
u1(1:n1) = u; v1(1:n2) = v;
y = ifft(fft(u1).*fft(v1));
y = y(1:nm);
%END

